#!/usr/bin/perl

package BaseTest::CheckInfo;

use File::Path;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest);

=head1 NAME

EDGUtils.pm - A set of utilities for EDG site certification tests

=head1 SYNOPSIS

EDGUtils.pm 

=head1 DESCRIPTION 

B<EDGUtils.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckInfo: setContext\n";

    # Add the resource option.
    $self->addOption('mdshost=s',
		     'The top-level mds node.',
                     'lxshare0406.cern.ch');
    $self->addOption('mdsport=i',
		     'The top-level mds port number.',
                     '2135');
    $self->addOption('mdsname=s',
		     'The mds-vo-name for MDS.',
                     'local');
    $self->addOption('bdiihost=s',
		     'The default BDII node.',
                     'lxshare0410.cern.ch');
    $self->addOption('bdiiport=i',
		     'The BDII port number.',
                     '2170');
    $self->addOption('bdiiname=s',
		     'The mds-vo-name for BDII.',
                     'local');

    # Commands necessary to run this test.
    $self->{commands} = ["ldapsearch"];

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::CheckInfo [options]

END
    ;

    $self->{DescText} = <<'END'

This is a parent skeleton of tests which check the information
systems. 

END
    ;

}


# This takes the JDL file and submits it.  It follows the status of
# the job and retrieves the output when it finishes.  It sets the
# locations of the standard output and error for evaluation. 
sub runTest {
    my $self = shift;

    print "CheckInfo: runTest\n";

    # Collect the information.
    my $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{mdshost} . 
        ":" . 
        $self->{opts}->{mdsport} . 
        " -x -b 'mds-vo-name=local,o=grid' " .
        " 'objectclass=GlueCE' GlueCEUniqueID";
    my $arrayref = timeout($command,60);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out');
        return 1;
    }

    print "Output: $output[1]\n";
    
    return 0;
}


# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

    print "CheckInfo: cleanup\n";

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
