#!/usr/bin/perl

package BaseTest::CheckInfo::CheckConsistency;

use File::Path;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::CheckInfo);

=head1 NAME

CheckConsistency.pm - Check consistency of resources in MDS and BDII.
# $Id: CheckConsistency.pm,v 1.14 2003/09/05 14:13:03 loomis Exp $

=head1 SYNOPSIS

CheckConsistency.pm 

=head1 DESCRIPTION 

B<CheckConsistency.pm> checks the list of CEs and SEs in the given MDS
and BDII servers and ensures that they are consistent. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckConsistency: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::CheckConsistency [options]

END
    ;

    $self->{DescText} = <<'END'

This test looks up all of the CEs and SEs in the top-level MDS and in
the BDII.  It compares the lists and reports any discrepancies.  

END
    ;

}


# This does the full test.  It queries the information system for each
# CE and SE and then performs a generic query on the GRIS.
sub runTest {
    my $self = shift;

    print "CheckConsistency: runTest\n";

    # Collect the information from MDS.
    my $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{mdshost} . 
        ":" . 
        $self->{opts}->{mdsport} . 
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=GlueCE)(objectclass=GlueSE))'" . 
        " GlueCEUniqueID GlueSEUniqueID ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on MDS');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %mdsres;
    my @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $mdsres{$1} =1 if ($line =~ m/(Glue[CS]EUniqueID:\s*[^\s]+)\s*/);
    }

    # Collect the information from BDII.  
    $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{bdiihost} . 
        ":" . 
        $self->{opts}->{bdiiport} . 
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=GlueCE)(objectclass=GlueSE))'" . 
        " GlueCEUniqueID GlueSEUniqueID ";
    $arrayref = timeout($command,90);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on BDII');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %bdiires;
    @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $bdiires{$1} =1 if ($line =~ m/(Glue[CS]EUniqueID:\s*[^\s]+)\s*/);
    }

    # What's in the mds but not bdii?
    my @notbdii;
    foreach my $key (keys %mdsres) {
        push @notbdii, $key if (!exists($bdiires{$key}));
    }
    if ($#notbdii>=0) {
        foreach my $val (@notbdii) {
            print "NOT in BDII: $val\n";
        }
    }

    # What's in bdii but no mds?
    my @notmds;
    foreach my $key (keys %bdiires) {
        push @notmds, $key if (!exists($mdsres{$key}));
    }
    if ($#notmds>=0) {
        foreach my $val (@notmds) {
            print "NOT in MDS: $val\n";
        }
    }

    # Send back an error message if necessary.
    if ($#notmds>=0 or $#notbdii>=0) {
#fail        my $list = join(", ",@failed);
        $self->setErrorMessage("MDS/BDII mismatch in CE/SE resources");
	$self->{globalresult} .= " [FAIL]" if $self->{globalresult};
        return 1;
    }

    $self->{globalresult} .= " [OK]" if $self->{globalresult};
    return 0;
}

sub getLDAPResources {

    my ($self, $host, $port, $name) = @_;

    # Collect the information from MDS.
    my $command = "ldapsearch -LLL -H ldap://" .
        $host . 
        ":" . 
        $port . 
        " -x -b 'mds-vo-name=$name,o=grid' " .
        "'(|(objectclass=GlueCE)(objectclass=GlueSE))'" . 
        " GlueCEUniqueID GlueSEUniqueID ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on MDS');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my @resources;
    my @lines = split("\n","$output[1]\n");
    foreach (@lines) {
        push @resources, $1 if (m/(Glue[CS]EUniqueID:\s*[^\s]+)\s*/);
    }

    return \@resources;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
