#!/usr/bin/perl

package BaseTest::CheckInfo::CheckGRIS;

use File::Path;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::CheckInfo);

=head1 NAME

EDGUtils.pm - A set of utilities for EDG site certification tests

=head1 SYNOPSIS

EDGUtils.pm 

=head1 DESCRIPTION 

B<EDGUtils.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckGRIS: setContext\n";

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::CheckGRIS [options]

END
    ;

    $self->{DescText} = <<'END'

This test looks up all of the available CEs and SEs in the information
system.  It then queries each one to ensure that each GRIS is alive. 

END
    ;

}

# This does the full test.  It queries the information system for each
# CE and SE and then performs a generic query on the GRIS.
sub runTest {
    my $self = shift;

    print "CheckGRIS: runTest\n";

    # Collect the information.
    my $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{mdshost} . 
        ":" . 
        $self->{opts}->{mdsport} . 
        " -x -b 'mds-vo-name=" .
        $self->{opts}->{mdsname} .
        ",o=grid' " .
        "'(|(objectclass=GlueCE)(objectclass=GlueSE))'" . 
        " GlueCEUniqueID GlueSEUniqueID ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %grises;
    my @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $grises{$1} =1 if ($line =~ m/Glue[CS]EUniqueID:\s*([^:\s]+)\s*(:.*)?/);
    }

    # Now do an ldapsearch on each gris.  Look only for timeouts here.
    my @failed;
    foreach my $gris (sort keys %grises) {
        print "Trying... $gris:2135\n";

        # Note that the port is hardwired here because there is no way
        # to discover it from the information system.
        my $command = "ldapsearch -LLL -H ldap://" .
            $gris . 
            ":" . 
            "2135 " . 
            " -x -b 'mds-vo-name=local,o=grid'";

        my $arrayref = timeout($command,30);
        my @output = @$arrayref;
        
        # Catch errors from the execution of the command. 
        if ($output[0]==1) {
            push @failed, $gris;
        } elsif ($output[0]==2) {
            push @failed, $gris;
        }
    }

    # Send back an error message if necessary.
    if ($#failed>=0) {
        my $list = join(", ",@failed);
        $self->setErrorMessage("Unreachable GRISes: $list");
        return 1;
    }

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
