#!/usr/bin/perl

package BaseTest::CheckInfo::CheckRGMA;

use File::Path;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::CheckInfo);

=head1 NAME

CheckRGMA.pm - check consistency between listed CEs and SEs in RGMA and LDAP

=head1 SYNOPSIS

CheckRGMA.pm 

=head1 DESCRIPTION 

B<CheckRGMA.pm> checks the consistency between the listed CEs and SEs in
RGMA and in LDAP (either a MDS or BDII server). 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckRGMA: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::CheckRGMA [options]

END
    ;

    $self->{DescText} = <<'END'

This test looks up all of the CEs and SEs in the top-level MDS and in
the BDII.  It compares the lists and reports any discrepancies.  

END
    ;

}


# This does the full test.  It queries the information system for each
# CE and SE and then performs a generic query on the GRIS.
sub runTest {
    my $self = shift;

    print "CheckRGMA: runTest\n";

    # Get the RGMA CEs and SEs and put these into a hash. 
    my $ref = $self->getRGMAResources();
    my @reslist = @$ref;

    my %rgma;
    foreach (@reslist) {
	$rgma{$_} = 1;
    }

    # Get the LDAP CEs and SEs and put these into a hash.
    my $ref = $self->getLDAPResources($self->{opts}->{mdshost},
				      $self->{opts}->{mdsport},
				      $self->{opts}->{mdsname});
    undef @reslist;
    my @reslist = @$ref;

    my %ldap;
    foreach (@reslist) {
	$ldap{$_} = 1;
    }

    # What's in RGMA but not in LDAP?
    my @notldap;
    foreach (keys %rgma) {
        push @notldap, $_ if (!exists($ldap{$_}));
    }
    if ($#notldap>=0) {
        foreach (@notldap) {
            print "NOT in LDAP: $_\n";
        }
    }

    # What's in LDAP but not in RGMA.
    my @notrgma;
    foreach (keys %ldap) {
        push @notrgma, $_ if (!exists($rgma{$_}));
    }
    if ($#notrgma>=0) {
        foreach (@notrgma) {
            print "NOT in RGMA: $_\n";
        }
    }

    # Send back an error message if necessary.
    if ($#notrgma>=0 or $#notldap>=0) {
#fail        my $list = join(", ",@failed);
        $self->setErrorMessage("RGMA/LDAP mismatch in CE/SE resources");
        return 1;
    }

    return 0;
}

# This just collects the CEs and SEs together into one list because
# it takes two queries to do this with RGMA. 
sub getRGMAResources {

    my ($self) = @_;

    my @resources;

    my $ref = $self->getRGMAList("GlueCE");
    my @celist = @$ref;

    my $ref = $self->getRGMAList("GlueSE");
    my @selist = @$ref;

    foreach (@celist,@selist) {
	push @resources, $_;
    }

    return \@resources;
}

# Retrieve the CEs or SEs from RGMA.  Pass in the value GlueCE or GlueSE
# for the table to use.  The attribute in both cases is UniqueID.
sub getRGMAList {

    # Get the table name (GlueCE or GlueSE)
    my ($self, $table) = @_;

    # Collect the information from RGMA.
    my $command = "edg-rgma -c 'latest select UniqueID from $table'";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork edg-rgma');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('edg-rgma timed out');
        return 1;
    }

    # Pull out the list of IDs from the output.
    my @resources;
    my @lines = split("\n","$output[1]\n");
    foreach (@lines) {
	if (m/\|\s*([^\s]+)\s*\|/) {
	    my $value = $1;
	    push @resources, $value unless ("$value" eq 'UniqueID');
	}
    }
    return \@resources;
}

sub getLDAPResources {

    my ($self, $host, $port, $name) = @_;

    # Collect the information from LDAP.
    my $command = "ldapsearch -LLL -H ldap://" .
        $host . 
        ":" . 
        $port . 
        " -x -b 'mds-vo-name=$name,o=grid' " .
        "'(|(objectclass=GlueCE)(objectclass=GlueSE))'" . 
        " GlueCEUniqueID GlueSEUniqueID ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my @resources;
    my @lines = split("\n","$output[1]\n");
    foreach (@lines) {
        push @resources, $1 if (m/Glue[CS]EUniqueID:\s*([^\s]+)\s*/);
    }

    return \@resources;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
