#!/usr/bin/perl

package BaseTest::CheckInfo::CheckTree;

use File::Path;
use EDGUtils;

use strict;
use vars qw(@ISA @failed);
@ISA=qw(BaseTest::CheckInfo);

=head1 NAME

CheckTree.pm - Check that the MDS hierarchy is functioning correctly.

=head1 SYNOPSIS

CheckTree.pm 

=head1 DESCRIPTION 

B<CheckTree.pm> descends into the MDS tree ensuring that all of the
"non-local" GIISes have children and that the GRISes respond to 
queries.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckTree: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::CheckTree [options]

END
    ;

    $self->{DescText} = <<'END'

This test starts with the given host, port, and Mds-vo-name and
decends to all GIISes and GRISes below.  It checks that each is simply
alive.

END
    ;

}

# This does the full test.  It queries the information system for each
# CE and SE and then performs a generic query on the GRIS.
sub runTest {
    my $self = shift;

    print "CheckTree: runTest\n";
    
    my $rc = $self->processNode($self->{opts}->{mdshost}, 
				$self->{opts}->{mdsport}, 
				"Mds-Vo-name=$self->{opts}->{mdsname},o=grid");
    return $rc;
}

# Process a node.  This involves doing a search on the node and then
# any children. 
sub processNode {

    my ($self, $host, $port, $suffix ) = @_;

    my @failed;

    print "Processing: $host:$port ($suffix)\n";

    my $childref = $self->getChildren($host, $port, $suffix );

    my @hosts = @{$$childref{host}};
    my @ports = @{$$childref{port}};
    my @names = @{$$childref{name}};
    my @status = @{$$childref{status}};


    # Print a warning if this is not a "local" GIIS (meaning a GRIS)
    # and it has no children. 
    if ($#hosts < 0 && !($suffix =~ /.*local.*/)) {
	print "WARNING: NO CHILDREN for $host:$port, $suffix\n";
    } elsif ($#hosts >= 0) {
	foreach my $i (0 .. $#hosts) {
	    if  ( $status[$i] eq "VALID" ) { 
		my $fail = $self->checkNode($hosts[$i],$ports[$i],$names[$i]);
		if ($fail) {
		    push @failed, $fail;
		    print "Unreachable GRIS: $fail (-b $names[$i])\n";
		} else {
		    if ( $status[$i] eq "VALID" ) {
			my $fail = $self->processNode($hosts[$i], $ports[$i], $names[$i]);
			if ($fail) {
			    push @failed, $fail;
			    print "Error processing: $hosts[$i]:$ports[$i] ($names[$i])\n";
			}
		    }    
		}
	    }
        }
    }

    # Send back an error message if necessary.
    if ($#failed>=0) {
        my $list = join(", ",@failed);
        $self->setErrorMessage("Unreachable GRISes: $list");
        return 1;
    }
    return 0;
}


# Actually checks that the given GIIS (or GRIS) responds. 
sub checkNode {

    my ($self, $host, $port, $suffix) = @_;

    # Now do an ldapsearch on each gris.  Look only for timeouts here.
 
    my $command = "ldapsearch -LLL -H ldap://$host:$port -x -b '$suffix'";
#    print "Executing : $command\n";
    my $output = timeout($command,30);

    # Catch errors from the execution of the command. 
    return $host if ($$output[0]==1 || $$output[0]==2);
    return '';
}


# Get the list of GIISes which register to the given one.
sub getChildren {

    my ($self, $host, $port, $suffix ) = @_;

    # Now do an ldapsearch on each gris.  Look only for timeouts here.
    my $command = "ldapsearch -LLL -H ldap://$host:$port" .
	" -x -b '$suffix' -s base giisregistrationstatus";
#    print "Executing : $command\n";
    my $output = timeout($command,90);

    # Catch errors from the execution of the command. 
    if ($$output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($$output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %child;
    $child{host} = [];
    $child{port} = [];
    $child{name} = [];
    $child{status} = [];
    print "@{$child{status}}\n";
    my @lines = split("\n","$$output[1]\n");
    my $line;
    foreach $line (@lines) {
	$line =~ /\s*([-\w]+)\s*:\s*(.*[^\s]+)\s*/;
	my $tag = $1;
	my $val = $2;
	my $index = '';
#       skip empty line 
	if ("$tag" eq "Mds-Service-hn") {
	    $index = 'host' if ($line);
	} elsif ("$tag" eq "Mds-Service-port") {
	    $index = 'port' if ($line);
	} elsif ("$tag" eq "Mds-Service-Ldap-suffix") {
	    $index = 'name' if ($line);
	} elsif ("$tag" eq "Mds-Reg-status") {
	    $index = 'status' if ($line);
	}    
	push @{$child{$index}}, $val if ($index);
    }
    return \%child;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
