#!/usr/bin/perl

package BaseTest::CheckRMC;

use File::Path;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest);

=head1 NAME

CheckRMC.pm - A set of utilities for EDG site certification tests
# $Id: CheckRMC.pm,v 1.6 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

CheckRMC.pm 

=head1 DESCRIPTION 

B<CheckRMC.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckRMC: setContext\n";

    # Add the resource option.
    $self->addOption('rlshost=s',
		     'The RLS node to test.',
                     'adc0013.cern.ch');
    $self->addOption('insecure',
		     'Use insecure connection to server.',
                     '');
    $self->addOption('vo=s',
		     'The Virtual Organization (VO) name for this service.',
                     'wpsix');

    # Commands necessary to run this test.
    $self->{commands} = ["edg-replica-metadata-catalog-admin",
                         "edg-replica-metadata-catalog"];

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::CheckRMC [options]

END
    ;

    $self->{DescText} = <<'END'

This runs the basic checks on the RLS Local Replica Catalog. 

END
    ;

}


# This takes the JDL file and submits it.  It follows the status of
# the job and retrieves the output when it finishes.  It sets the
# locations of the standard output and error for evaluation. 
sub runTest {
    my $self = shift;

    print "CheckRMC: runTest\n";

    # The default is to use a secure connection. 
    my $insecure = '';
    $insecure = '--insecure' if $self->{opts}->{insecure};
    my $server = $self->{opts}->{rlshost};
 
    # The default vo is wpsix
    my $vo = 'wpsix';
    $vo = $self->{opts}->{vo} if $self->{opts}->{vo};

    # Just ping the server to see if it responds. 
    my $command = "edg-replica-metadata-catalog-admin $insecure ping --vo $vo -h " . 
        $self->{opts}->{rlshost};
    my $arrayref = timeoutWithRC($command,60);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    

    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure removeAlias guid:guid1 alias:alias1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure removeAlias guid:guid1 alias:alias2 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure addAlias guid:guid1 alias:alias1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure addAlias guid:guid1 alias:alias2 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure aliasExists alias:alias1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure guidExists  guid:guid1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure guidForAlias alias:alias1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure aliasesForGuid guid:guid1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure mappingsByAlias '*' --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure mappingsByGuid '*' --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure addAliasAttrDefn Creator string --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure aliasAttrDefnExists Creator --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure listAliasAttrDefns --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure setAliasAttr alias:alias1 Creator Leanne --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure getAliasAttr alias:alias1 Creator --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure removeAliasAttr alias:alias1 Creator --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure removeAliasAttrDefn Creator --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    # Remove guid.
    $command = "edg-replica-metadata-catalog $insecure removeAlias guid:guid1 alias:alias1 --vo $vo -h " . 
        $self->{opts}->{rlshost};
    $arrayref = timeoutWithRC($command,60);
    @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]!=0) {
        $self->setErrorMessage('command timed out');
        return 1;
    } elsif ($output[1]!=0) {
        $self->setErrorMessage('command failed with error code: ' . 
                               ($output[1]>>8));
        print "Output: $output[2]\n";
        return 1;
    }

    print "Output: $output[2]\n";
    
    return 0;
}


# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

    print "CheckRMC: cleanup\n";

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
