#!/usr/bin/perl

package BaseTest::EDGLifecycle;

use strict;
use vars qw(@ISA);
use File::Path;
use EDGUtils;
use IO::Handle;

@ISA=qw(BaseTest);
use vars qw($dg_job_submit $dg_job_get_output $dg_job_status $statusTag $destTag);

=head1 NAME

EDGLifecycle.pm - A set of utilities for EDG site certification tests
# $Id: EDGLifecycle.pm,v 1.26 2003/10/03 12:42:35 chollet Exp $

=head1 SYNOPSIS

EDGLifecycle.pm 

=head1 DESCRIPTION 

B<EDGLifecycle.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

use vars qw($statusTag $destTag $jobidTag);
$statusTag = 'Current Status';
$destTag = 'Destination';
$jobidTag = 'Printing status info for the Job';

use vars qw($doneState $abortState $cancelState $runningState $clearState);
$doneState = 'Done (Success)';
$abortState = 'Aborted';
$cancelState = 'Done (Cancelled)';
$runningState = 'Running';
$clearState = 'Cleared';

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "EDGLifecycle: setContext\n";

    # Add the resource option.
    $self->addOption('resource=s',
		     'Submit directly to given resource.');

    # Add the rank option.  This will be added to the JDL if set.
    $self->addOption('rank=s',
		     'ClassAds Rank expression.');

    # Add the requirements option.  This will be added to the JDL if set.
    $self->addOption('requirements=s',
		     'Requirements expression.');

    # Number of identical jobs to send and number of parallel streams. 
    # These are useful for job storms. 
    $self->addOption('njobs=i',
		     'Number of jobs in each stream.',
                     '1');
    $self->addOption('nstreams=i',
		     'Number of streams.',
                     '1');

    # Allow the resubmissions to be specified.
    $self->addOption('retrycount=i',
		     'Retry count.');

    # Allow the MyProxy server to be specified. 
    $self->addOption('myproxyhost=s',
		     'MyProxy server host');

    # Allow the virtual organization to be specified.  Needed for job
    # submissions involving data requirements.  
    $self->addOption('vo=s',
		     'Virtual Organization (VO) name');

    # Avoid erasing Temporary Directory when test done
    $self->addOption('eraseTemp!',
		     'Erase Temp Dir when set',
		     1);

    # Commands necessary to run this test.
#     our $dg_job_submit; 
#     our $dg_job_get_output; 
#     our $dg_job_status;
    if ( $ENV{EDG_FLAVOR} eq 'edg1' ) {
	(${dg_job_submit}, ${dg_job_get_output}, ${dg_job_status}) = ( "dg-job-submit", "dg-job-get-output", "dg-job-status");
    } else {
	(${dg_job_submit}, ${dg_job_get_output}, ${dg_job_status}) = ( "edg-job-submit", "edg-job-get-output", "edg-job-status");
    }
    $self->{commands} = ["${dg_job_submit}", 
			 "${dg_job_get_output}", 
			 "${dg_job_status}",
			 "grid-proxy-info"];

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle [options] 

END
    ;

    $self->{DescText} = <<'END'

This is a parent skeleton of tests which use a typical EDG
job submission.

If the "resource" option is used, then the job is submitted directly
to the given resource bypassing the matchmaking process.

END
    ;

    $self->{job} = {};  ##  Hash, used for reporting (gg)

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "EDGLifecycle: checkPrerequisites\n";

    $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}

# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "EDGLifecycle: setupTest\n";

    # Create a temporary directory to hold job submission files and
    # job output.
    my $tmpdir;
    do {
        my $value = int(rand(32768));
        $tmpdir = "/tmp/$ENV{USER}-$$-$value";
    } while (-e $tmpdir);

    # Now create the directory.
    mkpath($tmpdir, 0, 0755);
    if ($?) {
        $self->setErrorMessage('could not create temporary directory');
        return 1;
    }
    print "Temporary directory: $tmpdir\n";
    $self->{tempdir} = $tmpdir;

    return 0;
}


# This takes the JDL file and submits it.  It follows the status of
# the job and retrieves the output when it finishes.  It sets the
# locations of the standard output and error for evaluation. 
sub runTest {

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

    my $self = shift;

    print "EDGLifecycle: runTest\n";
#p    my @resJDL = `cat $self->{jdl}`;
#p    print STDOUT " resJDL: \n@resJDL\n";

    # Set the prefix to use for the job ids.  Currently hardcoded
    # here.  It contains the full path of the file excepting the
    # stream suffix. 
    $self->{prefix} = "$self->{tempdir}/jobids";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nstreams}) {
	my $fname = "$self->{prefix}.$str";
	open IDS, ">$fname";
	close IDS;
	print "Created Job ID file: $fname\n";
    }

    my $idfile = "$self->{prefix}.1";
    print "JobID File: $idfile\n";
    
    # Submit the job.
    my $pidref = $self->parallelSubmit();

    # Waiting on child processes. 
    my $list = join(", ",@$pidref);
    print localtime() . "\n";
    print "Waiting on children... $list \n";
    waitforpids($pidref);

    # Monitor the progress of the jobs.
    print "Beginning monitoring  \n\n";
    $self->monitorJobs();

    # Retrieve the output for all jobs.
    print "Retrieving output from jobs... $self->{prefix}.total\n";
    my $rc = $self->retrieveAll();
    if ($rc) {
        $self->setErrorMessage('output retrieval failed');
        return 1;
    }
    
    return 0;
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

    print "EDGLifecycle: evaluateTest\n";

    # Loop over all of the streams. 
    my $numTotal = 0;
    my $numOK = 0;

    # Loop over all jobids.
    open IDS, "<$self->{prefix}.total";
    while (<IDS>) {
	my $jobid = $_;
	chomp($jobid);
	$jobid =~ /(.*\/)(\S+)/;
	my $subdir = $2;
	
	my $rc = $self->evaluateOutput("$self->{tempdir}/$subdir/std.out",
				       "$self->{tempdir}/$subdir/std.err");
	print STDERR "Output check failed for $jobid.\n" if ($rc); 
	
	$numTotal++;
	$numOK++ unless ($rc);
    }
    close IDS;

    # Returns 0 is all OK, 1 if some OK, 2 if all failed. 
    # Rene Metery fix: when 0 jobs were submitted the test failed
    if ($numOK == $numTotal) {
	if (($numOK == 0) && ($numTotal == 0)) {
	    $self->{globalresult} .= " [FAIL]" if $self->{globalresult};
	    return 2;
	} else {
	    $self->{globalresult} .= " [OK]" if $self->{globalresult};
	    return 0;
	}
    } elsif ($numOK > 0) {
	$self->{globalresult} .= " [FAIL]" if $self->{globalresult};
	return 1;
    } else {
	$self->{globalresult} .= " [FAIL]" if $self->{globalresult};
	return 2;
    }
}


# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

    print "EDGLifecycle: cleanup\n";

    if ( $self->{opts}->{eraseTemp} ) {
	my $dir = $self->{tempdir};
	rmtree($dir) if $dir;
    }

    return 0;
}


# By default this will simply check that the standard error has zero
# size and that the standard output contains the string '[OK]'.  The
# names of the standard output and error files are passed through the
# class context with keys 'stdout' and 'stderr', respectively. 
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;

    # Check that the std.out and std.err files exist.
    return 1 if (! (-e "$stdout"));
    return 1 if (! (-e "$stderr"));

    # Check that the standard error has a zero size.
    return 1 if (-s "$stderr");

    # Check the standard output contains the string [OK]. 
    open FILE, "<$stdout";
    my $value = 1;
    while (<FILE>) {
        $value = 0 if m/.*\[OK\].*/;
    }

    if ( $ENV{MAINSCRIPT} ) {
	my @resC = `cat $stdout`;
	print " stdout: $stdout \n @resC";
    }

    return $value;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job or jobs writing the job id(s) to the specified file.
# The function takes a single array reference for an argument.  The
# array should contain: the name of the jobid file and the number of
# copies to submit.  Both arguments are optional.  The job ids will be
# written to "jobids.txt" if not specified.  If the number of copies
# is not given, it defaults to 1.
#
#  The name of the jdl file is taken from the "$self->{jdl}".  If
# "$self->{opts}->{resource}" is set, then the job bypasses the match
# making sending the job directly to that resource.  The resource must
# be fully specified.
sub submit { 

    my ($self, $argref) = @_;

    # Set the defaults if not specified. 
    my ($idfile, $njobs) = @$argref; 
    $idfile ||= "jobids.txt";
    $njobs ||= 1;

    # Construct the submit command.  
    my $command = "${dg_job_submit} -nomsg -noint ";
    $command .= " -vo $self->{opts}->{vo} " if ($self->{opts}->{vo});
    my $logfile = "$self->{tempdir}/${dg_job_submit}_$<_$$.log";
    $command .= " -logfile $logfile";
    $command .= " -resource $self->{opts}->{resource} " 
	if ($self->{opts}->{resource});
    $command .= " $self->{jdl}";

    # Open the output file for the jobids. 
    open IDS, ">$idfile";

    # Loop over the job submissions writing out job id for each job. 
    my $numOK = 0;
    my $numNotOK = 0;
    foreach (1 .. $njobs) {

        my $jobid = '';

        # Note that the noint option should suppress all interactive
        # commands, but doesn't suppress obtaining a proxy if one
        # doesn't exist.  This screws up getting the jobid.  A valid
        # proxy is required in the prerequisites, but if it gets to
        # this point without one, this function will fail.

        $jobid = `$command`;
#p	print " jobid: $jobid $? \n" if ( $jobid =~ /https/ );
##	$self->{job}->{submit}->{log} = $jobid if ( $njobs == 1 );
        chomp $jobid;

        if ( $jobid =~ /https/ ) {
            print IDS "$jobid\n";
            $numOK++;
        } else {
	    print STDERR " Jobid: $jobid \n Submission status: $? \n";
            print STDERR "Job submission failed.  Continuing...\n";
            $numNotOK++;
        }
    }

    # Close the output file.
    close IDS;

    # Set the return code depending on the situation. 
    if ($numOK>0 && $numNotOK==0) {
        # All OK. 
        return 0;
    } elsif ($numOK>0 && $numNotOK>0) {
        # Some OK. 
        return 1;
    } else {
        # None OK. 
        return 2;
    }
}


# Follows the job progress.  It stops when the job state becomes
# OutputReady or Aborted.  (There is currently no timeout, but this
# should be added.) 
sub follow {
    my $jobid = shift(@_);

    my $running = 1;
    while ($running) {
	sleep 10;

	my $pref = status($jobid);
	my %params = %{$pref};
	$running = 0 if ("$params{$statusTag}" eq $doneState);
	print "Status: $params{$statusTag}, $params{$destTag}\n";

	if ("$params{$statusTag}" eq $abortState) {
	    foreach my $key (keys %params) {
		print "$key: $params{$key}\n";
	    }
            return 1;
	}
    }
    return 0;
}


# Returns a hash with the status parameters and the values.
sub status {
    my $jobid = shift(@_);

    open RESPONSE, "${dg_job_status} $jobid 2>/dev/null |";
    my %params;
    while (<RESPONSE>) {
	if (m/(\S[^:]+\S)(\s*:\s*)(\S.*\S)\s*/) {
	    my $param = $1;
	    my $value = $3;
	    $params{$param} = $value;
	}
    }
    close RESPONSE;

# Send back a reference to the hash of parameters.
    return \%params;
}


# Retrieve the job output.
sub retrieve {
    my ($jobid, $dir) = @_;
    my $logfile = "$dir/${dg_job_get_output}_$<_$$.log";
    mkdir $dir, 0777;
    `${dg_job_get_output} -noint -dir $dir -logfile $logfile $jobid`;
    return "" if ($?);

    opendir OUTPUT, "$dir";
    my @files = readdir(OUTPUT);

    # This loop assumes that there is only one subdirectory!
    my $outdir;
    foreach my $file (@files) {
	$outdir = "$dir/$file" if (-d "$dir/$file");
    }
    return $outdir;
}



# Retrieve all of the outputs from a list of jobs. 
sub retrieveAll {
    my ($self) = @_;
    open IDS, "<$self->{prefix}.total";
    while (<IDS>) {
        my $jobid = $_;
	chomp($jobid);

	my $pref = status($jobid);
	my %params = %{$pref};

        if ("$params{$statusTag}" eq $doneState) {
	    my $logfile = "$self->{tempdir}/${dg_job_get_output}_$<_$$.log";
            my @resDJGO = `${dg_job_get_output} -noint -dir $self->{tempdir} -logfile $logfile $jobid`;
#p	    print " resDJGO: \n@resDJGO";
            print STDERR "${dg_job_get_output} failed for $jobid\n" if ($?);
	    $self->{job}->{getout}->{log} = "@resDJGO";
        } else {
	       print STDERR "$jobid in status '$params{$statusTag}'\n"; 
	       my $status_reason=`${dg_job_status} -noint $jobid | grep -e Reason -e Destination -e reached`;
	       print STDERR "$status_reason\n"
	}    

    }
    close IDS;

    return 0;
}



# This routine monitors the jobs listed in a job id file.  It keeps
# following them until all of the jobs are in a terminal state
# (Aborted, Cancelled, or 'Done (Success)'). 
sub monitorJobs {

    my ($self, $idfile) = @_;

    # Pull in the job ids from all streams.
    opendir TEMP, "$self->{tempdir}";
    my @idfiles = grep /jobids\.\d+/, readdir TEMP;
    closedir TEMP;
    my @full = map { $_ = "$self->{tempdir}/$_"} @idfiles;

    # Merge all of these into single file.
    my $tfile = "$self->{prefix}.total";
    open TOTAL, ">$tfile";
    foreach my $file (@full) {
	open IDS, "<$file";
	while (<IDS>) {
	    print TOTAL $_ if (m/^https:.*/);
	}
	close IDS;
    }
    close TOTAL;

    my %status = ();
    my %location = ();

    $self->{job}->{status}->{log} = "";

#p    my $notDone;
    use vars qw($notDone);
    do {

        # Reinitialize all existing keys to be zero in status and location. 
        foreach (keys %status) {
            $status{$_} = 0;
        }
        foreach (keys %location) {
            $location{$_} = 0;
        }

        # Update the status from the logging and bookkeeping. 
	my $logfile = "$self->{tempdir}/${dg_job_status}_$<_$$.log";
	my $scount = 0;
	my $response;
	my $statusVal;

      STAT: {
        open STATUS, "${dg_job_status} -noint -logfile $logfile -i $tfile |";
	$response = "";
	$statusVal= "";
        while (<STATUS>) {

	    $response .= $_;
#p	    print " DJS: $_";

	    if ( /ECONNRESET/ ) {
		print STDERR " DJSeconnc: $_";
		if ( ! $scount++ ) {
		    close STATUS;
		    sleep 5;
		    redo STAT; 
		}
	    }

            if ( /Status Reason:\s*(\(nil\))\s*/ ) {
		if ( $statusVal ne $doneState ) { next; }
                print STDERR " DJSstatus: $_";
		$status{$doneState}--;
		$status{$runningState}++;
	    }

            if (/\s*${statusTag}\s*:\s*(\S.*\S)\s*/o) {
                my $key = $1;
		$statusVal = $key;
		$status{$key}++;
            }

            if (/\s*${destTag}\s*:\s*(\S+)\s*/o) {
                my $key = $1;
		$location{$key}++;
            }

        }
        close STATUS;
    $self->{job}->{status}->{log} .= $response;
    }

        ## The last response should be enough ...
#    	$self->{job}->{status}->{log} = $response;
        $self->{job}->{submit}->{log} = `cat $self->{job}->{submit}->{file}` if $self->{job}->{submit}->{file};

        # Print out the summary of the jobs. 
        print localtime() . " ($self->{tempdir})\n";
        my $total = 0;
        $notDone = 0;
        foreach my $key (sort keys %status) {
            print "$key: $status{$key}\n";
            $total += $status{$key};
            if (("$key" ne $abortState) && ("$key" ne $doneState) &&
		("$key" ne $clearState) && ("$key" ne $cancelState)) {
                $notDone += $status{$key};
            }
        }
        my $completed = $total - $notDone;
        print "TOTAL: $completed of $total completed.\n\n";

        foreach my $key (sort keys %location) {
            print "$key: $location{$key}\n";
            $total += $location{$key};
        }


        # Sleep for a while.
        sleep 30 if ($notDone>0);

    } while ($notDone>0); 
}


# Run job submissions in parallel.  This takes all of the parameters
# from the state of the object. 
sub parallelSubmit {

    my ($self) = @_;
    my @pids;

    my $i;
    foreach $i (1 .. $self->{opts}->{nstreams}) {
      FORK: { 
	    my $pid;
          if ($pid = fork) {
              # parent here
              # child process pid is available in $pid
              push @pids, $pid;
          } elsif (defined $pid) {
              # $pid is zero here if defined
              # child here
              # parent process available with getppid

	      # Do the actual submission.  The error return is ignored
	      # because there is no way to feed back this information
	      # easily.  
	      my $rc = $self->submit(["$self->{prefix}.$i",
				      "$self->{opts}->{njobs}"]);
	      
              # This is important otherwise child may execute parent code!
              exit;
          } elsif ($! =~ /No more process/) {
              # Recoverable error (EAGAIN); try again. 
              sleep 5;
              redo FORK;
          } else {
              # Strange error! Print error message. 
              print STDERR "Fork failure.\n";
          }
      }
	$self->{job}->{submit}->{file} = "$self->{prefix}.$i" if ( $self->{opts}->{nstreams} == 1 );
    }

    return \@pids;
}


# Wait for the processes with the given pids to end. 
sub waitforpids {
    my $pidref = shift;

    # Wait for each process in turn.
    foreach my $pid (@$pidref) {
        my $result = waitpid $pid, 0;
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
