#!/usr/bin/perl

package BaseTest::EDGLifecycle::BrokerInfo;

use strict;
use vars qw(@ISA);
use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

BrokerInfo.pm - A set of utilities for EDG site certification tests
# $Id: BrokerInfo.pm,v 1.11 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

BrokerInfo.pm 

=head1 DESCRIPTION 

B<BrokerInfo.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "BrokerInfo: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::BrokerInfo [options] 

END
    ;

    $self->{DescText} = <<'END'

This submits a job which checks that the .BrokerInfo file
exits, the environmental variable is set, and the it has a
non-zero size.

Note that specifying the --resource option will cause this 
test to fail because this option bypasses the matchmaking. 
(And hence does not generate a .BrokerInfo file.)

END
    ;

}

# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "BrokerInfo: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script);
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
Arguments     = "none";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
END
    ; 

    print JDL "Requirements = $self->{opts}->{requirements};\n" 
	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/sh

# Is the BrokerInfo variable set?
if [ -z "$EDG_WL_RB_BROKERINFO" ]; then
    echo "EDG_WL_RB_BROKERINFO not set. [FAIL]"
    exit
else 
    echo EDG_WL_RB_BROKERINFO:  $EDG_WL_RB_BROKERINFO
fi

# Does the file exist?
if [ -e $EDG_WL_RB_BROKERINFO ]; then
    echo ".BrokerInfo file exists."
else
    echo ".BrokerInfo file does not exist. [FAIL]"
    exit
fi

# Check that it actually contains something.
if [ -s $EDG_WL_RB_BROKERINFO ]; then
    echo ".BrokerInfo has non-zero size."
    ls -l $EDG_WL_RB_BROKERINFO
    echo "Contents: "
    cat $EDG_WL_RB_BROKERINFO
    echo "End Contents."
else
    echo ".BrokerInfo file has zero size. [FAIL]"
    exit
fi

# Check that the CLI can parse the file
edg-brokerinfo getCloseSEs
if [ $? != 0 ] ; then
    echo "Can not getCloseSEs from .BrokerInfo file. [FAIL]"
    exit
fi

# If it made it this far, everything's OK.
echo "Goodbye. [OK]"
exit
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
