#!/usr/bin/perl

package BaseTest::EDGLifecycle::CheckVOVars;

use File::Basename;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

CheckVOVars.pm - A set of utilities for EDG site certification tests
# $Id: CheckVOVars.pm,v 1.10 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

CheckVOVars.pm 

=head1 DESCRIPTION 

B<CheckVOVars.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CheckVOVars: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::CheckVOVars [options] <VO names>

END
    ;

    $self->{DescText} = <<'END'

This submits a job which checks that the necessary environmental
variables are defined on a site for supported virtual organizations.

END
    ;

}

# Ensure that some VOs have been specified.
sub checkArguments {
    my $self = shift;
    my $error = $self->SUPER::checkArguments();
    if ($error) {
        $self->setErrorMessage('standard argument checking failed');
        return 1;
    }

    print "CheckVOVars: checkArguments\n";

    # Ensure that some VOs have been defined.
    my @allargs = @{$self->{args}};
#p    print " CheckVOVars:allargs: @allargs \n";
    return 1 unless ($#allargs>=0);
    return 0;
}

# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "CheckVOVars: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    # Create the script for this job. (This doesn't depend on VO.)
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script, $self->{args});
    if ($file) {
	$self->{jdl} = "$file";
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $voref, $site) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    my $volist = join(' ',@$voref);

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
Arguments     = "$volist";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
END
    ; 

    print JDL "Requirements = $self->{opts}->{requirements};\n" 
	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/sh

# Check that the main environmental variable is set.
if [ -z "$EDG_LOCATION" ]; then
    echo "EDG_LOCATION not properly set." 
    echo "[FAIL]"
    exit 1;
fi

# Check that the binary is there.
if [ ! -e "$EDG_LOCATION/bin/edg-vo-env" ]; then
    echo "$EDG_LOCATION/bin/edg-vo-env doesn't exist." 
    echo "[FAIL]"
    exit 1;
fi

# Check that it is executable.
if [ ! -x "$EDG_LOCATION/bin/edg-vo-env" ]; then
    echo "$EDG_LOCATION/bin/edg-vo-env isn't executable." 
    echo "[FAIL]"
    exit 1;
fi

for i in $@; do 

    # Status.
    echo Checking VO ${i}

    # Run it to modify the environment.
    eval `$EDG_LOCATION/bin/edg-vo-env --shell=sh ${i}`


    # Software root directory.
    swroot=`echo ${i}_ROOT_DIR | tr a-z A-Z`
    value=`printenv $swroot`
    if [ -z "$value" ]; then
	echo "$swroot is not set." 
	echo "[FAIL]"
	exit 1;
    fi

done

# Everything was OK!
echo "[OK]"
exit 0
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
