#!/usr/bin/perl

package BaseTest::EDGLifecycle::Checksum;

use strict;
use vars qw(@ISA);
use File::Basename;
use Cwd;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

EDGUtils.pm - A set of utilities for EDG site certification tests

=head1 SYNOPSIS

EDGUtils.pm 

=head1 DESCRIPTION 

B<EDGUtils.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "Checksum: setContext\n";

    # Add the size option.
    $self->addOption('sizemb=i',
		     'Size in MBytes of file to create and transfer.',
                     '1');

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Checksum [options]

END
    ;

    $self->{DescText} = <<'END'

This submits a job which creates a file of the given size,
transports it to the given worker node, performs a checksum,
and compares with the true checksum.

END
    ;

}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "Checksum: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};
    my $dfile = "$tmpdir/test.dat";
    my $cfile = "$tmpdir/true.txt";

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script, $dfile, $cfile);
    if ($file) {
	$self->{jdl} = "$file";
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    # Create the data file of the correct size.
    my $file = createData("$dfile", $self->{opts}->{sizemb});
    if (! $file) {
        $self->setErrorMessage('could not create data file');
        return 1;
    }

    # Create the true checksum.
    $file = "$cfile";
    my $output = `cd $tmpdir; cksum test.dat`;
    open CHSUM, ">$file";
    print CHSUM $output;
    close CHSUM;
    if (! (-s $file)) {
        $self->setErrorMessage('could not create checksum file');
        return 1;
    }

    return 0;
}

# Evaluate the test.  Check standard EDG conditions but also check
# the local checksum when finished. 
sub evaluateTest {
    my $self = shift;

    print "Checksum: evaluateTest\n";

    my $rc = $self->SUPER::evaluateTest();
    return $rc if $rc;

    # Loop over all of the streams. 
    my $numTotal = 0;
    my $numOK = 0;

    # Loop over all jobids.
    open IDS, "<$self->{prefix}.total";
    while (<IDS>) {
	my $jobid = $_;
	chomp($jobid);
	$jobid =~ /(.*\/)(\S+)/;
	my $subdir = $2;
	
	# Calculate and check the checksum of local file. 
        my $cdir = getcwd();
	chdir "$self->{tempdir}/$subdir/";

	my $final = `cksum test.dat`;
	chomp($final);
	my $orig = `cat true.txt`;
	chomp($orig);

	chdir $cdir;

	# Set the return code. 
	my $rc = 0;
	$rc = 1 unless ("$final" eq "$orig");
	print STDERR "Local output check failed for $jobid.\n" if ($rc); 
	
	$numTotal++;
	$numOK++ unless ($rc);
    }
    close IDS;

    # Returns 0 is all OK, 1 if some OK, 2 if all failed. 
    if ($numOK == $numTotal) {
	return 0;
    } elsif ($numOK > 0) {
	return 1;
    } else {
	return 2;
    }
}

##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $dfile, $cfile) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script", "$dfile", "$cfile"};
OutputSandbox = {"std.out","std.err","test.dat","true.txt"};
END
    ; 

    print JDL "Requirements = $self->{opts}->{requirements};\n" 
	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Create a data file of the given size (in MBytes). 
sub createData {
    my ($fname, $sizemb) = @_;

    # Open the data file.
    open DATA, ">$fname";

    # Write out the data in 1MByte chunks.
    for (my $s=$sizemb; $s>0; $s--) {
	my @values = ();
	for (my $i=1024*1024; $i>0; $i--) {
	    push @values, int(rand(256));
	}
	my $buf = pack("C*", @values);
	print DATA $buf; 
    }

    close DATA;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $fname) {
        return $fname;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/sh

# The true value of the checksum.
export CSTRUE="`cat true.txt`"

# Calculate the checksum of the local file.
echo "True checksum: '${CSTRUE}'"
export CSTEST="`cksum test.dat`"
echo "Test checksum: '${CSTEST}'"
echo "Done checking."

if [ "${CSTRUE}" = "${CSTEST}" ]; then
    export STATUS=OK;
else 
    export STATUS=FAIL;
fi

# Finished. 
echo "Goodbye. [${STATUS}]"
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
