#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::MatchMaking;

use strict;
use vars qw(@ISA);
use EDGUtils;

use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

MatchMaking.pm - Check the EDG matchmaking mechanism at RB level.
# $Id: MatchMaking.pm,v 1.20 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

 The matchmaking test is achieved in this way:
 For each triplet of SE machines,
a set of 3 different data files is created, registered through the ERM, 
and they are all replicated to only 2 of these SEs. 
On the remaining SE, only 2 out of these 3 files are replicated.

 In the JDL of the job which is built for the test, the 3 files are required 
(with the InputData clause), and they MUST be accessible through
the "file" protocol (DataAccessProtocol clause) .

 Then, the "edg-job-list-match" is used to create a list of matching CEs,
selecting only the fastest queue through the Rank clause,
and the job is actually sent for execution on each of the matching machines.

 Within the job, a printout of each data file is achieved to cross-check that 
the files are indeed visible through the "file" protocol.

 Obviously, out of 3 CEs, only 2 of them must match when they are close to the 
SEs where all 3 files were replicated. 

=head1 DESCRIPTION 

B<MatchMaking.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

#    print "MatchMaking: setContext\n";
    $self->printLocation( (caller(0))[3] );

    # Add the resource option.
    $self->addOption('mds=s',
                     'The top-level mds node.',
                     'gppinfo06.gridpp.rl.ac.uk');
    $self->addOption('mdsport=i',
                     'The top-level mds port number.',
                     '2135');
    $self->addOption('mdsname=s',
                     'The mds-vo-name for MDS.',
                     'local');
    $self->addOption('bdii=s',
                     'The default BDII node.',
                     'ccedgli02.in2p3.fr');
    $self->addOption('bdiiport=i',
                     'The BDII port number.',
                     '2170');
    $self->addOption('bdiiname=s',
                     'The mds-vo-name for BDII.',
                     'local');
#     $self->addOption('vo=s',
# 		     'VO to use in the matchmaking.',
#                      'wpsix');
    $self->addOption('simple',
		     'Matchmaking with a single SRM per SE.');
    $self->addOption('execute',
		     "Send the JDL for execution, in addition to the matchmaking step.\nImplies the oneStep option");
    $self->addOption('oneStep',
		     'Only one file configuration is tried for matchmaking.');
    $self->addOption('infoBDII',
		     'Force using standard (dynamic) configuration instead of the default static one.');

    # The command needed to run this test.
    $self->{commands} = ["grid-proxy-info",
#			 "edg-vo-env",
			 "edg-gridftp-rm",
                         "edg-gridftp-ls",
			 "edg-gridftp-mkdir",
#p			 "gdmp_ping",
# 			 "gdmp_host_subscribe",
# 			 "gdmp_job_status",
# 			 "globus-replica-catalog",
# 			 "ldapsearch",
# 			 "edg-replica-manager-replicateFile",
# 			 "edg-replica-manager-copyAndRegisterFile",
# 			 "edg-replica-manager-deleteFile",
# 			 "edg-replica-manager-listReplicas"
 			 "edg-replica-manager"
			 ];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::MatchMaking [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'
NEEDS TO BE REWRITTEN ...
Check the basic setup of an SE.  This test checks:
 * That the SE GRIS is running.
 * Supported VO(s) are defined in the GRIS.
 * The GridFTP server is running.
 * The storage path exists.
 * The "magic" softlink exists for NFS access (via BrokerInfo).
 * The edg_query_vo_storage cron job is running.
This does not check the data management software or configuration.

This test was supplied by Stephen Burke. 

END
    ;

	$self->{reqs}   = {};    ## Hash
	$self->{res}    = {};    ## Hash
	$self->{steps}  = {};    ## Hash
	$self->{plargs} = ();    ## List

        $self->{stepI} = 0;
	$self->{SRM}  = {};    ## Hash of hashes
	$self->{GUIDs}  = {};    ## Hash

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if $rc;

#    print "MatchMaking: checkPrerequisites\n";
    $self->printLocation( (caller(0))[3] );

       $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if $rc;

    return $rc;
}


sub runTest {
    my $self = shift;

#    print "MatchMaking: runTest\n";
    $self->printLocation( (caller(0))[3] );

    my $rc;
    my @SEs;
    my %SPs = {};
    if (0) {
	($rc, @SEs) = getSEs($self);
	return $rc if $rc;

	($rc, %SPs) = getSPs($self, @SEs);
	return $rc if $rc;
    } else {
	@SEs = @{$self->{args}};
	push @SEs, ($SEs[0], $SEs[1]);
	print "\n Final full list of SEs: @SEs \n";
    }

#   exit 0;

    # Keep a list of all SEs which fail
    my @failed;

    # Loop over each SE pair doing a full test for each.
    for my $n ( 0 .. ($#SEs-2) ) {
	my @SEtrip = ($SEs[$n], $SEs[$n+1], $SEs[$n+2]);
	print "\n Current SE triplet: @SEtrip \n\n";
	my $SEprev = ( $n == 0 ) ? " " : $SEs[$n-1];
        my $error = $self->testMatchMaking(\@SEtrip, $SEprev, $n, \%SPs);
        push @failed, @SEtrip if ($error);
    }

    # If there are any servers in the list, then the test has failed.
    if ($#failed>=0) {
#        my $names = join(",",@failed);
        my $names = join("\n",@failed);
        $self->setErrorMessage("MatchMaking test failed for: \n$names");
        return 1;
    }

    return 0;
}


## 
## The actual test is here.  It just takes a server name, extracts the
## supported VOs and runs checks. 
##

sub testMatchMaking {

    my ($self, $arrayref, $SEprev, $iter, $hashref) = @_;

    $self->printLocation( (caller(0))[3] );

    my $SE2;  ## deprecated ??
    my @SEtrip = @$arrayref;
    my %SPs = %$hashref;
    my $rc = 0;
     print "\n Iteration: $iter ; Working this SE triplet: @SEtrip\n";
#     print "\n Working this SE triplet: @SEtrip with these mount points: \n";
#     foreach my $se ( @SEtrip ) { 
# 	print "     FS: $se => $SPs{$se} \n";
#     }

    ## Step Index (used in XML display)
    my $stepI = $self->{stepI};
    my $clout;
    my $command;
    my $stat;

    my $subDir = "testGG";
#    my $edgRMopts = "--config=edg-replica-manager-CertTB.conf --insecure --vo=$self->{opts}->{VO}";
    my $config = " ";
    if ( $self->{opts}->{infoBDII} ) {
	$config = "--config=/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf";
    } else {
	## Because we are using now the static InfoStub for ERM related stuff ...
	$config = "--config=$ENV{ERM_CONFIG}" if ( $ENV{ERM_CONFIG} );
    }
    ## Default options to be used for all ERM related commands ...
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{VO}";
    print " edgRMopts: $edgRMopts \n";
#    my $edgRMopts = "--vo=atlas --verbose";

    if ( $SEprev eq " " ) {  
	# grodid-20643-0-rmstorm.txt
	my $upid = $$;
	    foreach my $loop ( 0..2 ) {
		my $file = "$ENV{USER}-$upid-$loop-mm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
		if ( $stat ) { next; }

		$command = "edg-replica-manager $edgRMopts deleteFile --all $resRcLR";
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
		my $resRcLRagter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRagter: $file -$stat- \n$resRcLRagter";
	    }
    } 

##E    my $nn = int(rand(3));
    my $nn = 0;
    my $SE1 = $SEtrip[$nn];
    my $SEA = $SEtrip[($nn+1)%3];
    my $SEB = $SEtrip[($nn+2)%3];
    my $SRMaster;
    my $file;
    chomp(my $hostname = `hostname`);
    ## We have yet to find a solution to make this in a better way. Hardcoded for the time being.
    ## One can use $ENV{SE_HOSTS_} and or $ENV{CE_CLOSE_SE_ID_} + $siteTB
#    my %SECE = (
#		 "lxshare0235.cern.ch" => "lxshare0236.cern.ch",
#		 "lxshare0241.cern.ch" => "lxshare0236.cern.ch",
#		 "lxshare0290.cern.ch" => "lxshare0291.cern.ch",
#		 "lxshare0277.cern.ch" => "lxshare0278.cern.ch"
#	 );
    my @CElist = split /,/, $ENV{CEhostname_};
#p    print " CElist: @CElist \n";
    my @SElist = split /,/, $ENV{CE_CLOSE_SE_ID_};
    my %SECE = ();
    foreach my $ncs ( 0 .. $#CElist ) {
	$SECE{$CElist[$ncs]} = $SElist[$ncs];
    }
    print " SECElist:\n", map { "    CE: $_ => SE: $SECE{$_}\n" } sort { $SECE{$a} cmp $SECE{$b} } keys %SECE; 

##E    my %GUIDs = ();
    my %LFNs  = ();
    my %SRMs  = ();
    my @LFNl  = ();
##E    my $loopS = int(rand(3));
    my $loopS = 2;
##E    my $SRM  = {};  ## a hash of hashes
    my $tmpdir = "$self->{tempdir}";
    my @resRcLR = ();

    foreach my $loop (0..2) {

	if ( $self->{opts}->{simple} ) {
	    ##  Special short
	    my $nm = $nn + $loop;
	       $SE1 = $SEtrip[$nm%3];
	       $SEA = $SEtrip[($nm+1)%3];
	       $SEB = $SEtrip[($nm+2)%3];
	    ##  Special short
	}

    ## Create a temporary file ...
#	my $value = int(rand(32768));
    $file = "$ENV{USER}-$$-$loop-rmstorm.txt";
#tmp    open TMP, ">/tmp/$file";

	if ( ! -e "$tmpdir/$file" ) {  ##E
	    open TMP, ">$tmpdir/$file";
	    print TMP "Good_Morning_to_all_Everybody_! ";
	    #tmp    print TMP "/tmp/$file\n";	
	    print TMP "$tmpdir/$file\n";	
	    close TMP;
	}

    my %PFNs  = ();

	if ( ! $self->{SRM}->{$SE1}->{$file} ) {  ##E

##edg2.0    my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s ${hostname}//tmp/${file} -d $SE1/$SP1/$file";
#tmp    $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:/tmp/${file} -l lfn:${file} -d $SE1";
    $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1";
    print "\n CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";

    ## Brand new for command routine design ...
    my ($stat, $resRcCARF) = $self->commandProc( $command,
						 "copyAndRegisterFile 1",
						 "$file",
						 ++$stepI);

    if ( $stat != 0 ) {
##edg2.0	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    $self->{GUIDs}->{$file} = $resRcCARF;
#    $PFNs{"$SE1/$SP1/$file"} = "created";
    $PFNs{"$SE1/$file"} = "created";
    $LFNs{$file} = "lfn:${file}";
    push @LFNl, "${file}";

##edg2.0    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
#        @resRcLR = `$command 2>&1`;
# #	@resRcLR = grep { s/ srm:/srm:/g } @resRcLR ;
#     print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";

    my ($stat, @resRcLR) = $self->commandProc( $command,
					       "listReplicas 1",
					       "$file",
					       ++$stepI);

    foreach my $l ( @resRcLR ) {
        chomp $l;
##edg2.0        my @i = split /: /, $l;
        my @i = split /[:\/]/, $l;
	if ( $i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $gsrm previously existed: $l ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    $SRMs{$l} = "existing";
	    $self->{SRM}->{$SE1}->{$file} = $l;
	    $SRMaster = $l;
#p	    print " SRM keys SE1: $SE1 $file value: $self->{SRM}->{$SE1}->{$file} \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

}

    foreach my $SE2 ( $SEA, $SEB ) {

	## To limit arbitrarily the number of replicates ...
	next if $self->{opts}->{simple};  ##  Special short
	next if $loop == $loopS && $SE2 eq $SEB;  ## To create a hole

	if ( ! $self->{SRM}->{$SE2}->{$file} ) {  ##E

##edg2.0    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
#    my $resRcRF = `edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2 2>&1`;
##E    $command = "edg-replica-manager $edgRMopts replicateFile $SRMaster -d $SE2";
    $command = "edg-replica-manager $edgRMopts replicateFile $self->{SRM}->{$SE1}->{$file} -d $SE2";
#     my $resRcRF = `$command 2>&1`;
#     $stat = $?;
#     print "\n resRcRF2: $file -$stat- \n$resRcRF";

    my ($stat, $resRcRF) = $self->commandProc( $command,
					       "replicateFile 2",
					       "$file",
					       ++$stepI);

    if ( $stat != 0 ) {
##edg2.0        print STDERR " File: ${subDir}/${file} on: $SE2 not correctly registered or created \n";
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcRF);
	if ( $resRcRF !~ "srm:" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $resRcRF \n";
	    return 1;
	} else {
	    $PFNs{"$SE2/$file"} = "created";
	    $SRMs{$resRcRF} = "existing";
	    $self->{SRM}->{$SE2}->{$file} = $resRcRF;
#p	    print " SRM keys SE2: $SE2 $file value: $self->{SRM}->{$SE2}->{$file} \n";
	}
    }
##    $PFNs{"$SE2/$SP2/$file"} = "created";

##edg2.0       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
#       @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
#     @resRcLR = `$command 2>&1`;
# 	my $resRcLR = join("\n", grep { s/\s+//g } @resRcLR);
# 	print "\n resRcLR2: $file -$?- =$#resRcLR= \n$resRcLR\n";

    my ($stat, @resRcLR) = $self->commandProc( $command,
					       "listReplicas 2",
					       "$file",
					       ++$stepI);

    if (0) {
    foreach my $l ( @resRcLR ) {
	chomp $l;
##edg2.0	my @i = split /: /, $l;
        my @i = split /[:\/]/, $l;
	if ( $i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( ( $csrm eq $gsrm ) && ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) ) {
	    print STDERR " Step SE2: $SE2 File: $gsrm previously existed: $l ! \n";
	    return 1;
	} else {
	    $PFNs{$gsrm} = "existing";
	    if ( ! $SRMs{$l} ) {
		print STDERR " Step SE2: $SE2 SRM: $l not previously existing ! \n";
		return 1;
	    }
	}
    }
}

    }  ## End of if SRM

}  ## End of foreach my $SE2

## Temporarily commented out ##E

    my $limit = 3;
    $limit-- if ( $loop == $loopS );
    $limit = 1 if $self->{opts}->{simple};  ##  Special short

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } sort keys %PFNs;
    print "-----------------------------------\n";
#     if ( $#resRcLR < $limit-1 || scalar(keys(%PFNs)) != $limit ) {
# 	print STDERR " Replicates not correctly created on: $SE1 $SEA $SEB \n";
# 	return 1;
#     }

}  ## End of foreach my $loop

    ## Non-first pass case ...
    if ( $self->{LFNs} ) {
	## Restoring some useful arrays ..
	%LFNs = %{$self->{LFNs}};
	@LFNl = @{$self->{LFNl}};

	## Removing some unwanted files ...
	my $n0 = 2;
	my $machSE = $SEB;
	if ( $self->{SRM}->{$machSE}->{$LFNl[$n0]} ) {
	    print " Removing: $self->{SRM}->{$machSE}->{$LFNl[$n0]} from: $machSE for: $LFNl[$n0] \n";
	    $command = `edg-replica-manager $edgRMopts unregisterFile $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]} 2>&1`;
	    my $resRcURF = `$command 2>&1`;
	    print " resRcURF1: \n$resRcURF\n";
	    delete $self->{SRM}->{$machSE}->{$LFNl[$n0]};
	}	
	if ( $SEprev ne " " ) {
	    foreach my $n0 ( 0..2 ) {
		my $machSE = $SEprev;
		if ( $self->{SRM}->{$machSE}->{$LFNl[$n0]} ) {
		    print " Removing: $self->{SRM}->{$machSE}->{$LFNl[$n0]} from: $machSE for: $LFNl[$n0] \n";
		    $command = `edg-replica-manager $edgRMopts unregisterFile $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]} 2>&1`;
		    my $resRcURF = `$command 2>&1`;
		    print " resRcURF3: \n$resRcURF\n";
		    delete $self->{SRM}->{$machSE}->{$LFNl[$n0]};
		}	
	    }
	}
    ## First pass case ...
    } else {
	## Saving some useful arrays ..
	%{$self->{LFNs}} = %LFNs;
	@{$self->{LFNl}} = @LFNl;
    }

    print "\nGUIDs: \n", map { "$_ => $self->{GUIDs}->{$_}\n" } sort keys %{$self->{GUIDs}};
    print "\nLFNs: \n", map { "$_ => $LFNs{$_}\n" } sort keys %LFNs;
    print "\nLFNlist: @LFNl \n\n";

    my $loopM = 20;
    ##E May be always put this one to 0 ??
    $loopM    = 0 if $self->{opts}->{oneStep};
    my $try = 0;
##    my $n0 = int(rand(3));
    foreach my $loopT ( 0 .. $loopM ) {
	my $n0 = $loopT%3;
	my $n1 = ($n0+1)%3;
	my $n2 = ($n0+2)%3;


	## Section where the JDL is being matched ...

	## Adding required InputData clause to JDL file ...
	my $resCp = `cp -p $self->{jdlm} $self->{jdl}`;
##edg2.0    my $resE = `echo '    InputData = "lfn:${subDir}/${file}";' >> $self->{jdl}`;
	my $resE = `echo '    InputData = {"$LFNs{$LFNl[$n0]}", "$self->{GUIDs}->{$LFNl[$n1]}", "$LFNs{$LFNl[$n2]}"};' >> $self->{jdl}`;
#old	$resE =    `echo '    Arguments = "${file}";'    >> $self->{jdl}`;
#humm	$resE =    `echo '    Arguments = "$LFNs{$LFNl[$n0]} $LFNs{$LFNl[$n1]} $LFNs{$LFNl[$n2]}";'    >> $self->{jdl}`;
	$resE =    `echo '    Arguments = "$LFNs{$LFNl[$n0]} $self->{GUIDs}->{$LFNl[$n1]} $LFNs{$LFNl[$n2]}";'    >> $self->{jdl}`;
##edg2.0    $resE =    `echo '    ReplicaCatalog = "$ENV{RC_LOGICAL_COLLECTION}";' >> $self->{jdl}`;

	$self->{plargs} = ();  ##  To reuse it during the evaluation step after execution (if any) ...
	$self->{plargs}->{lfn}->{1} = "$LFNs{$LFNl[$n0]}";
	$self->{plargs}->{lfn}->{2} = "$self->{GUIDs}->{$LFNl[$n1]}";
	$self->{plargs}->{lfn}->{3} = "$LFNs{$LFNl[$n2]}";
	$self->{plargs}->{pfn}->{1} = "$tmpdir/$LFNl[$n0]";
	$self->{plargs}->{pfn}->{2} = "$tmpdir/$LFNl[$n1]";
	$self->{plargs}->{pfn}->{3} = "$tmpdir/$LFNl[$n2]";
# 	foreach my $in (1..3) {
# 	    $self->{plargs}->{pfn}->{$in} = $LFNl[$n0]};
# 	}

	print " Try # $try ============================================================================================= \n";
	## Listing the current replicas ...
	foreach my $nl ( $n0, $n1, $n2) {
	    my $file = $LFNl[$nl];
#	    my @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:$file 2>&1`;
	    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
# 	    my @resRcLR = `$command 2>&1`;
# 	    my $resRcLR = join("\n", grep { s/\s+//g } @resRcLR);
# 	    print "> resRcLRsum: $file -$?- =$#resRcLR= \n$resRcLR\n";

	    my ($stat, @resRcLR) = $self->commandProc( $command,
						       "listReplicas sum",
						       "$file",
						       ++$stepI);

	    if (0) {
	## Listing the current GUIDs ...
#	       @resRcLR = `edg-replica-manager $edgRMopts listGUID lfn:$file 2>&1`;
	    $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
	    @resRcLR = `$command 2>&1`;
#	    print "> resRcLG3: $file -$?- =$#resRcLR= \n@resRcLR";
	    print "\t                                   @resRcLR";

	$stat = $?;
	$clout = "@resRcLR";
	$stepI++;
	$self->{reqs}->{$stepI}->{mess} = "$clout";
	$self->{reqs}->{$stepI}->{stat1} = 0 ? ($stat == 0) : 1;
	$self->{reqs}->{$stepI}->{stat1} = ($stat == 0) ? 0 : 1;
	$self->{reqs}->{$stepI}->{stat2} = $stat;
	$self->{step}->{$stepI}->{brfinfo} = "listGUID";
	$self->{step}->{$stepI}->{detinfo} = $command;

	}

	}

# 	my $resCp = `cat $self->{jdl}`;
# 	print " resCpMatch: \n$resCp\n";

	## Achieving the matching indeed ...
	$command = "edg-job-list-match $self->{jdl}";
	print "\n InputDatal = {$LFNs{$LFNl[$n0]}, $LFNs{$LFNl[$n1]}, $LFNs{$LFNl[$n2]}} \n";
	print " InputData  = {$LFNs{$LFNl[$n0]}, $self->{GUIDs}->{$LFNl[$n1]}, $LFNs{$LFNl[$n2]}} \n";
# 	my $resEJLM = `$command 2>&1`;
# 	print " resEJLM: \n$resEJLM\n";

	my ($stat, $resEJLM) = $self->commandProc( $command,
						   "edg-job-list-match $iter",
						   "",
						   ++$stepI);

	## Retrieving the matching CEs ...
	my %seen = ();
	my @machL = grep { ! $seen{$_}++ } map { /(\S+):2119\/jobmanager-/ } split /\s+/, $resEJLM;
	print " matchingCEs: @machL\n";

	## Retrieving the matching CE queues ...
     	%seen = ();
	my @machS = grep { ! $seen{$_}++ } map { /(\S+:2119)\/jobmanager-/ } split /\s+/, $resEJLM;
	   @machS = grep { /\S+:2119\/jobmanager-/ } split /\s+/, $resEJLM;
	my %CEq;
	foreach my $ma ( @machS ) {
#p	    print " ma: $ma \n";
	    $ma =~ /(\S+:2119)\/(jobmanager-\S+)/;
#p	    print " ma12: $1 $2 $seen{$1} \n";
	    if ( $seen{$1} ) {
		$CEq{$1} = $ma;
	    }
	}
	my @CEl;
	foreach my $ma ( values %CEq ) {
	    push @CEl, $ma;
	}
	print " MatchingCEqueues: @CEl \n";
	@{$self->{CEq}} = @CEl;
#	@{$self->{CEq}} = split /\s+/, $resEJLM;

	last if ( scalar(@machL) == 0 );  ##  No more matching ...
	last if ( $loopT == $loopM );     ## Don't remove any new SE after last iteration ...

	## Section where some files were removed in the previous version
	## May be not keeping this part in the new version

	%seen = ();
	@machL = grep { ! $seen{$_}++ } map { $SECE{$_} } @machL;
	my $np = int(rand(3));
	if ( $np ) {
	    if ( $np == 1 ) { unshift(@machL, pop(@machL)); }
	    else            { push(@machL, shift(@machL));  }
	}
	print " machSE: @machL\n";

#old	foreach my $machSE ( keys %{$SRM} ) {
	foreach my $machSE ( @machL ) {
	    if ( $self->{SRM}->{$machSE}->{$LFNl[$n0]} ) {
		print " Removing: $self->{SRM}->{$machSE}->{$LFNl[$n0]} from: $machSE for: $LFNl[$n0] \n";
#		my $resRcURF = `edg-replica-manager $edgRMopts unregisterFile $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]} 2>&1`;
		$command = `edg-replica-manager $edgRMopts unregisterFile $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]} 2>&1`;
		my $resRcURF = `$command 2>&1`;
		print " resRcURF: \n$resRcURF\n";

	$stat = $?;
	$clout = "$resRcURF";
	$stepI++;
	$self->{reqs}->{$stepI}->{mess} = "$clout";
	$self->{reqs}->{$stepI}->{stat1} = 0 ? ($stat == 0) : 1;
	$self->{reqs}->{$stepI}->{stat1} = ($stat == 0) ? 0 : 1;
	$self->{reqs}->{$stepI}->{stat2} = $stat;
	$self->{step}->{$stepI}->{brfinfo} = "unregisterFile";
	$self->{step}->{$stepI}->{detinfo} = $command;

		delete $self->{SRM}->{$machSE}->{$LFNl[$n0]};  ##E was forgotten in the previous version (bug !)

		next if ( $stat );
#		my @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:$LFNl[$n0] 2>&1`;
		$command = "edg-replica-manager $edgRMopts listReplicas lfn:$LFNl[$n0]";
		my @resRcLR = `$command 2>&1`;
		print "\n resRcLR4: $LFNl[$n0] -$?- =$#resRcLR= \n@resRcLR\n";

	$stat = $?;
	$clout = "@resRcLR";
	$stepI++;
	$self->{reqs}->{$stepI}->{mess} = "$clout";
	$self->{reqs}->{$stepI}->{stat1} = 0 ? ($stat == 0) : 1;
	$self->{reqs}->{$stepI}->{stat1} = ($stat == 0) ? 0 : 1;
	$self->{reqs}->{$stepI}->{stat2} = $stat;
	$self->{step}->{$stepI}->{brfinfo} = "listReplicas";
	$self->{step}->{$stepI}->{detinfo} = $command;

		if ( "@resRcLR" =~ "No GUID" ) {

# 		    @resRcLR = `edg-replica-manager $edgRMopts removeAlias $self->{GUIDs}->{$LFNl[$n0]} lfn:$LFNl[$n0] 2>&1`;
# 		    print "\n resRcRA: $LFNl[$n0] -$?- =$#resRcLR= \n@resRcLR\n";

#		    @resRcLR = `edg-replica-manager $edgRMopts deleteFile --all $self->{GUIDs}->{$LFNl[$n0]} 2>&1`;
		    $command = "edg-replica-manager $edgRMopts deleteFile --all $self->{GUIDs}->{$LFNl[$n0]}";
		    @resRcLR = `$command 2>&1`;
		    print "\n resRcDF: $LFNl[$n0] -$?- =$#resRcLR= \n@resRcLR\n";

	$stat = $?;
	$clout = "@resRcLR";
	$stepI++;
	$self->{reqs}->{$stepI}->{mess} = "$clout";
	$self->{reqs}->{$stepI}->{stat1} = 0 ? ($stat == 0) : 1;
	$self->{reqs}->{$stepI}->{stat1} = ($stat == 0) ? 0 : 1;
	$self->{reqs}->{$stepI}->{stat2} = $stat;
	$self->{step}->{$stepI}->{brfinfo} = "deleteFile";
	$self->{step}->{$stepI}->{detinfo} = $command;

		}
		last;
	    }
	}
	$try++;
    }

    $self->{stepI} = $stepI; 
    return 0 unless $self->{opts}->{execute};  ##  Do not execute the JDL unless requested ...

    ## To loop over all possible CEs in the matching team ...
    foreach my $cem ( @{$self->{CEq}} ) {

	$stepI = $self->{stepI};

    ## New: under tests ...
    my $resAdd = $self->laddArguments($cem);

# 	my $resCp = `cat $self->{jdl}`;
# 	print " resCpSubmit: \n$resCp\n";

    my $resJob = $self->SUPER::runTest();
    if ( $resJob ) { print " MatchMaking::Job : Failed ! \n"; }

#	$stat = $resJob;
	$stepI++;
	$self->{reqs}->{$stepI}->{stat1} = $resJob;
#         $self->{reqs}->{$stepI}->{stat2} = "MatchMaking::Job : Success";
#         $self->{reqs}->{$stepI}->{stat2} = "Sent to CE: $cem";
#         $self->{reqs}->{$stepI}->{stat2} = "MatchMaking::Job : Failed" if $resJob;
	$self->{reqs}->{$stepI}->{stat2} = $resJob ? "MatchMaking::Job : Failed" : "Sent to CE: $cem";
	$self->{step}->{$stepI}->{brfinfo} = "edg-job-submit";
	my $resCatJ = `cat $self->{jdl}`;
	my $resCatS = `cat $self->{script}`;
#	$self->{step}->{$stepI}->{detinfo} = "JDL:\n$self->{jdl}\n\nSCRIPT:\n$self->{script}";
	$self->{step}->{$stepI}->{detinfo} = "JDL:\n$resCatJ\n\nSCRIPT:\n$resCatS";
    $self->{stepI} = $stepI; 

    ## New: under tests ...
	my $rc = 0;
    open IDS, "<$self->{prefix}.total";
    while (<IDS>) {
	chomp(my $jobid = $_);
#	chomp($jobid);
	$jobid =~ /(.*\/)(\S+)/;
	my $subdir = $2;
	
	$rc = $self->levaluateOutput("$self->{tempdir}/$subdir/std.out",
				     "$self->{tempdir}/$subdir/std.err");
	print STDERR "Output check failed for $jobid.\n" if ($rc);
    }
    close IDS;

}  ## End of @CEl loop

    ## Temp ...
    if (1) { return $rc; }

    my %PFNs  = ();

    my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
    print "\n resRcDF3: $file -$?- =$#resRcDF= \n@resRcDF";
    foreach my $l ( @resRcDF ) {
	chomp $l;
        my @i = split /deleting /, $l;
        $PFNs{$i[1]} = "deleting";
    }

    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    print "\n resRcLR3: $file -$?- =$#resRcLR= \n";
    if ( $#resRcLR > -1 ) {
	foreach my $l ( @resRcLR ) {
	    chomp $l;
	    my @i = split /: /, $l;
	    $PFNs{$i[1]} = "still-existing";
	}
    }

    print "\nPFNs: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    foreach my $se ( $SE1, $SE2 ) {
	my $resDel = `edg-gridftp-rm gsiftp://${se}$SPs{$se}/$subDir`;
    }

## To delete a location:
## source grc.conf
## 1183  globus-replica-catalog -host "$RC_LOGICAL_COLLECTION" -manager "$RC_REP_CAT_MANAGER_DN" -password pwd.conf  -location lxshare0232.cern.ch -delete
## 1184  globus-replica-catalog -host "$RC_LOGICAL_COLLECTION" -manager "$RC_REP_CAT_MANAGER_DN" -password pwd.conf  -location lxshare0303.cern.ch -delete
    unlink "/tmp/$file";

    return $rc;
}


# Add any necessary arguments to the existing local JDL file before submission.
sub laddArguments {
    my ($self, $chosenCE) = @_;

#old    print "JobStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $bid;
##    $bid = `echo 'Arguments = "S$ENV{GGSERIE} I${index} rien"; ' >> $jdlTemp`;

    print STDERR " CElist: @{$self->{CEq}} \n";
#no    my $resCp = `cp -p $self->{jdlm} $self->{jdl}`;

#     my $nCE = scalar(@{$self->{CEq}});
#     my $rnd = int(rand($nCE));
#     my $ce = ${$self->{CEq}}[$rnd];
    my $ce = $chosenCE;
    if (1) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $self->{jdl}`;
	my @m = split /:/, $ce;
#temp	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
#temp	$self->{reqs}->{$index}->{mace} = "free";
    }


    return 0;
}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.)
sub setupTest {
    my $self = shift;

    $self->{debug} = 1;  ## auxiliary to new printLocation method.
    $self->printLocation( (caller(0))[3] );


    # Call the parent method to ensure that the temporary directory is
    # created.
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};
#p  print " tmpdir: $tmpdir \n";

    $self->{opts}->{oneStep} = 1 if $self->{opts}->{execute};
    $self->{opts}->{VO} = $self->{opts}->{vo};

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    } else {
	$self->{script} = $script;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script);
    if ($file) {
        $self->{jdl} = $file;
	## Creating JDLmaster ...
        $self->{jdlm} = "${file}m";
	my $resCp = `cp -p $self->{jdl} $self->{jdlm}`;
# 	$resCp = `cat $self->{jdlm}`;
# 	print " resCpMaster: \n$resCp\n";
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

## gg for XML creation and processing ...
    if ( $ENV{MAINSCRIPT} ) {
        my $base = "$self->{tempdir}";
        $self->{type}     = "MatchMaking";
        $self->{xmlfile}  = "$base/".$self->{type}.".XML";
        $self->{xmlxist}  = "$self->{type}";
        my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
        $html_dir = "${html_dir}/$self->{type}";
        my $resmk = `mkdir -p $html_dir`;
        $self->{htmldir}  = "$html_dir";
        print " HTMLDIR: $self->{htmldir} \n";
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


sub createJDL {
    my ($self, $dir, $script, $site) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";
    my $bin = dirname($ENV{MAINSCRIPT});

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
    Executable    = "$sname";
    StdOutput     = "std.out";
    StdError      = "std.err";
#    InputSandbox  = {"$script", "$bin/gg-bi"};
    InputSandbox  = "$script";
    OutputSandbox = {"std.out","std.err", "BrokerInfo"};
    StdInput = "/dev/null";
    RetryCount = 3;
#    DataAccessProtocol = "gridftp";
    DataAccessProtocol = "file";
    VirtualOrganisation = "$self->{opts}->{VO}";
##    Requirements = IsMember(other.RunTimeEnvironment, other.CEId=="lxshare0290.cern.ch:2119/jobmanager-pbs-short");
    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    print JDL "Requirements = $self->{opts}->{requirements};\n"
        if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n"
        if $self->{opts}->{rank};
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/bash
echo Arg: $1 $2 $3
GSF=`edg-brokerinfo getSelectedFile $1 file`
echo GSF: $GSF
TFN=`echo $GSF | cut -d " " -f 2`
echo Printing local file $TFN:
cat $TFN
echo "=+-+-+-+-+-+-+-+-"
/bin/uname -a
/bin/echo "Hello From Sgript"
/bin/echo "Goodbye. [OK]"
GCE=`edg-brokerinfo getCE`
echo CE: $GCE
GSE=`edg-brokerinfo getSEs`
echo SEs: $GSE
CSE=`edg-brokerinfo getCloseSEs`
echo getCloseSEs: $CSE
CSE=`echo $CSE | cut -d " " -f 1`
SMP=`edg-brokerinfo getSEMountPoint $CSE`
echo getSEMountPoint: $SMP

F1 = `./gg-bi.pl $1`
F2 = `./gg-bi.pl $2`
F3 = `./gg-bi.pl $3`

F1="/"`edg-brokerinfo getLFN2SFN $1 | grep $CSE | cut -d"/" -f4-`
F2="/"`edg-brokerinfo getLFN2SFN $2 | grep $CSE | cut -d"/" -f4-`
F3="/"`edg-brokerinfo getLFN2SFN $3 | grep $CSE | cut -d"/" -f4-`


cp .BrokerInfo BrokerInfo
echo "+-+-+-+-+-+-+-+-"
echo LFN 1 $1 $F1
contents=`cat $F1`
echo LFC 1 $contents
echo "+-+-+-+-+-+-+-+-"
echo "LFN 2" $2 $F2
contents=`cat $F2`
echo "LFC 2" $contents
echo "+-+-+-+-+-+-+-+-"
echo "LFN 3" $3 $F3
contents=`cat $F3`
echo "LFC 3" $contents
echo "+-+-+-+-+-+-+-+-"
END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


## Should be called by the evaluateTest method of the Parent Class
## Dummy from now on
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;
    return 0;
}

## Local now ...
sub levaluateOutput {
    my ($self,$stdout,$stderr) = @_;

    $self->printLocation( (caller(0))[3] );

    # Check that the std.out and std.err files exist.
    return 1 if (! (-e "$stdout"));
    return 1 if (! (-e "$stderr"));

#   $self->{job}->{getout}->{log}
    my $stepI = $self->{stepI};
    my $logsum = " --> Submit: \n\n$self->{job}->{submit}->{log}\n" 
               . " --> Status: \n$self->{job}->{status}->{log}" 
	       . " --> Getout: \n\n$self->{job}->{getout}->{log}";
    $self->{reqs}->{$stepI}->{mess} = "$logsum";
    $stepI++;

    # Check that the standard error has a NON zero size.
    return 1 if ( -z "$stderr");

    # Check the standard output ... 
    my $value = 0;
    $self->{res} = {};
    open FILE, "<$stdout";
    while ( (my $l = <FILE>) !~ '\+-\+-' ) {	
	    print " LINB: ", $l;
    }
    while ( <FILE> ) {
	if ( /^\+-\+-/ ) { next; }
	elsif ( /^Linux/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
	    $self->{reqs}->{$stepI}->{stat2} = "Matches, was run on WN: $m[1]";
	}
	elsif ( /LFN/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
#old	    $self->{res}->{$m[1]}->{lfn} = "$m[2]";
	    $self->{res}->{lfn}->{$m[1]} = "$m[2]";
	}
	elsif ( /LFC/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
	    if ( "$m[2]" !~ "^Good_" ) { next; }
	    if ( "$m[3]" !~ "^/tmp/" ) { next; }
#old	    $self->{res}->{$m[1]}->{pfn} = "$m[3]";
	    $self->{res}->{pfn}->{$m[1]} = "$m[3]";
	}
    }
    close FILE;

    my $cumul = "";
    foreach my $arg ( sort keys %{$self->{plargs}->{lfn}} ) {
	print " ARG: $arg lfnres:$self->{res}->{lfn}->{$arg} lfnarg:$self->{plargs}->{lfn}->{$arg} pfnres:$self->{res}->{pfn}->{$arg} pfnarg:$self->{plargs}->{pfn}->{$arg} \n";
	$cumul .= " ARG: $arg lfnres:$self->{res}->{lfn}->{$arg} lfnarg:$self->{plargs}->{lfn}->{$arg} pfnres:$self->{res}->{pfn}->{$arg} pfnarg:$self->{plargs}->{pfn}->{$arg} \n";
	$value = 1 if ( $self->{res}->{lfn}->{$arg} ne $self->{plargs}->{lfn}->{$arg} );
	$value = 1 if ( $self->{res}->{pfn}->{$arg} ne $self->{plargs}->{pfn}->{$arg} );
    }

#above	$stepI++;
#p    print " Current stepI: $stepI \n";
	$self->{reqs}->{$stepI}->{olog} = "$stdout";
	$self->{reqs}->{$stepI}->{stat1} = $value;
        $self->{reqs}->{$stepI}->{stat2} = "Wrong" if $value;
#        $self->{reqs}->{$stepI}->{stat2} = "Matching" unless $value;
	$self->{step}->{$stepI}->{brfinfo} = "Cross-checking";
	$self->{step}->{$stepI}->{detinfo} = "Checking that contents of output file match with submitted filenames:\n"."$cumul";

    $self->{stepI} = $stepI;

#     if ( $ENV{MAINSCRIPT} ) {
# 	my @resC = `cat $stdout`;
# 	print " stdout: $stdout \n @resC";
#     }

    return $value;
}


## Type with 1 column only
sub buildXMLFile {  
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
#    my @lof_rows;
#old    my $Title = "MatchMaking";
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
#     my $brf_info = "Rather Simple Storm";
#     my $Jdl = `cat $self->{jdlFile}`;
#     my $Scr = `cat  $self->{shFile}`;
#     my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
#    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile1Col() to write $self->{xmlfile} \n";

#     my $cols = {};
#     my $order = 0;
# #    my $machType = 'mawn';
#     my $machType = 'mace';
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#         $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
#         if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++; }
# 	my $mac = " ";
# 	if ( $self->{reqs}->{$index}->{mawn} ) { $mac = $self->{reqs}->{$index}->{mawn}; }
# 	else { $mac = "CE:".$self->{reqs}->{$index}->{mace}; }
# 	push @lof_rows, $mac;
#     }

#prev	 foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#prev	     print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#prev	     push @lof_cols, $col;
#prev	 }
    push @lof_cols, "Status";
    
    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    my $rowM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	## directly coded as 0/1 ...
#	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
	my $retv = $self->{reqs}->{$index}->{stat1};
	my $subs = $self->{reqs}->{$index}->{stat2};
	my $logfn;
	my $logf;

# 	if ( $retv == 0 ) {
# 	    $logfn = $self->{reqs}->{$index}->{olout};
# 	    $logf = `cat $logfn if $logfn`;
# 	} elsif ( $retv == 5 ) {  ## UNTESTED
# ##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
# 	    $logfn = $self->{reqs}->{$index}->{olog};
# 	    $logf = `cat $logfn if $logfn`;
# 	} else {
# ##edg2.0            if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
# ##edg2.0            else {
# 	    $logfn = $self->{reqs}->{$index}->{olog};
# 	    $logf = `cat $logfn if $logfn`;
# ##edg2.0	}
# 	}

	if ( $self->{reqs}->{$index}->{olog} ) { $logf = `cat $self->{reqs}->{$index}->{olog}`; }
	elsif ( $self->{reqs}->{$index}->{mess} ) { $logf = "$self->{reqs}->{$index}->{mess}"; }
	else { $logf = " "; }

	unlink $logfn if $logfn;
	my $coln = 0;
#	$coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
#bad	$rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
#		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step"), "col");
		$rowM .= makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step");
	    } else {
#		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
		$rowM .= makeX(makeX(" ", "output").makeX(" ", "retval"), "step");
	    }
	}
#	$colM .= makeX($rowM, "row");
    }

    ## To make each of the above a single step ...
    ## then merging it into a single col and single row for the time being ...
    $rowM .= makeX($rowM, "col");
    $colM .= makeX($rowM, "row");

    ## Building of the Step Info stack...
#my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    my $stepM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{step}} ) {
#old	$stepM .= makeX(makeX("$self->{step}->{$index}->{brfinfo}", "brf_info").makeX("$self->{step}->{$index}->{detinfo}", "det_info"), "step");
	my $stepb = makeX("$self->{step}->{$index}->{brfinfo}", "brf_info");
	my $steps = makeX("_", "substatus_info");
	my $stepd = makeX("$self->{step}->{$index}->{detinfo}", "det_info");
	$stepM .= makeX("$stepb.$steps.$stepd", "step");
    }

my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
#$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}


## To extract the available SE list for a given VO 
sub getSEs {
    my $self = shift;

    ## Set up some variables ...
#   my $edgl = `$ENV{EDG_LOCATION}/bin/edg-vo-env --shell=csh tstg`;
    my $edgl = `edg-vo-env --shell=csh $self->{opts}->{VO}`;
#p  print " EDGL: $edgl \n";
    my @ls = split /\n/, "$edgl";
    foreach my $l ( @ls ) {
	my @its = split /\s+/, $l;
	chop($its[2]);
	$ENV{$its[1]} = $its[2];
	print " ENV $its[1]: $ENV{$its[1]} \n";
    }
    my $gdmb = dirname( dirname( $ENV{RC_CONFIG_FILE}));
#   my $gdmb = basename( $ENV{RC_CONFIG_FILE});
    my $gdmp = `grep ^GDMP_LOCAL_HOST ${gdmb}/gdmp.shared.conf`;
#p  print " GDMP: $gdmp";
    chomp($gdmp);
    my @it = split /=/, $gdmp;
    $ENV{$it[0]}= $it[1];
    print " ENV $it[0]: $ENV{$it[0]} \n";

    # Collect the information from MDS.
    my $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{mds} .
        ":" .
	    $self->{opts}->{mdsport} .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=computingelement)(objectclass=storageelement))'" .
	    " SEId ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command.
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on MDS');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %mdsres;
    my @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $mdsres{$1} =1 if ($line =~ m/([CS]EId:\s*[^\s]+)\s*/);
    }

## On CertTB: MDS==0242:2135, BDII==0243:2170
## On  DevTB: MDS==0406:2135, BDII==0410:2170
## On CertTB: ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid'
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBind)(objectclass=storageelement))' GlueCESEBindSEUniqueID
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBind)(objectclass=storageelement))' GlueCESEBindCEAccesspoint
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBindGroup)(objectclass=storageelement))' GlueCESEBindGroupSEUniqueID
## Use lxshare0410 for DevTB BDII

    # Collect the information from BDII.
    $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{bdii} .
        ":" .
	    $self->{opts}->{bdiiport} .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=computingelement)(objectclass=storageelement))'" .
	    " SEId ";
    $arrayref = timeout($command,90);
    @output = @$arrayref;

    # Catch errors from the execution of the command.
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on BDII');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %bdiires;
    @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $bdiires{$1} =1 if ($line =~ m/([CS]EId:\s*[^\s]+)\s*/);
    }

## For this, run it with:
## ./edg-testbed-test BaseTest::EDGLifecycle::MatchMaking --mds=lxshare0233.cern.ch --bdii=lxshare0231.cern.ch

#   print map { "$_ => $mdsres{$_}\n" } keys %mdsres;
    print " BDIIres: @{[ %bdiires ]}\n";
    print " MDSres:  @{[ %mdsres ]}\n";

    my %seen;
    @seen{keys %mdsres} = ();
    my @unseen = grep { ! exists $seen{$_} } keys %bdiires;
    print " NOT in BDII: @unseen $#unseen\n";
    my $unSeenBdii = $#unseen;
    %seen = ();
    @unseen = ();
    @seen{keys %bdiires} = ();
    @unseen = grep { ! exists $seen{$_} } keys %mdsres;
    print " NOT in MDS:  @unseen $#unseen\n";
    my $unSeenMds = $#unseen;

    if ( ($unSeenMds+$unSeenBdii) != -2 ) {
	$self->setErrorMessage('MDS/BDII mismatch in SE resources');
	return 1;
    }
    @unseen = keys %mdsres;
    my @SEs = ();
#    my %SPs = ();
#    my %SPst= ();
    foreach my $se ( @unseen ) {
	$se =~ s/SEId: //;
	push @SEs, $se;
    }
    print " SEs: @SEs \n";

    if ( $#SEs <= 0 ) {
	$self->setErrorMessage('Less than 2 SEs: Test is Impossible. \n');
	return 1;
    }

    ## Temp ...
    ##if (1) { exit 0;}

    return (0, @SEs);

}


## To extract the SE Mount Points and all that ...
sub getSPs {

    my $self = shift;
    my @SEs = @_;

    my %SPs = ();
    my %SPst= ();

    ## Check that "tstg" VO exists on all SEs ...
    foreach my $se ( @SEs ) {
        my $command = "ldapsearch -x -h $se -p 2135 -b \"mds-vo-name=local,o=grid\" \"objectclass=storageelement\" sevo | grep -i sevo:";
        my $arrayref = timeout($command, 90);
        my @output = @$arrayref;
#p      print " SE: $se \n @output \n";

    # Catch errors from the execution of the command.
	if ($output[0]==1) {
	    $self->setErrorMessage('could not fork ldapsearch');
	    return 1;
	} elsif ($output[0]==2) {
	    $self->setErrorMessage('ldapsearch timed out on SE $se');
	    return 1;
	}
        my @lines = split("\n","$output[1]\n");
        my @voex = grep { /$self->{opts}->{VO}/ } @lines;
        print " VOEX: $se @voex \n";
        if ( $#voex != 0 ) {
            $self->setErrorMessage("VO \"$self->{opts}->{VO}\" does not exist on this SE: $se");
            return 1;
        }
        my @p = split /\s+/, $voex[0];
        @p = split /:/, $p[1];
        if ( $p[0] ne "$self->{opts}->{VO}" ) {
            $self->setErrorMessage("Wrong VO \"$self->{opts}->{VO}\" exists on this SE: $se, $voex[0]");
            return 1;
        }
        $SPs{$se} = $p[1];


        ## Another check about these mount points ...
        my $gdmpSt = `gdmp_job_status -S $se -c gdmp.conf | grep GDMP_STORAGE_DIR`;
#p      print " GDMPST: $se $gdmpSt";
        @p = split /=/, $gdmpSt;
        $SPst{$se} = $p[1];
        chomp($SPst{$se});
        if ( $SPst{$se} ne $SPs{$se} ) {
            print STDERR " Mount point mismatch on SE $se: $SPst{$se} $SPs{$se} \n";
            $self->setErrorMessage("Mount point mismatch between GDMP_STORAGE_DIR and LDAPSEARCH on this SE: $se");
            return 1;
        }

    }
    print "SPs: \n", map { "$_ => $SPs{$_}\n" } keys %SPs;

    return (0, %SPs);

}


## Mutual subscription check for SPs
sub mutualSPs {
    my ($subDir, $arrayref, $hashref) = @_;
    my @SEpair = @$arrayref;
    my %SPs = %$hashref;
    my $rc = 0;

    ## Mutual subscription test ...
    my $errSum = 0;
    for my $n ( 0..1 ) {
#       my $gdmpS = `gdmp_host_subscribe -c -S $SEpair[$n] -r $SEpair[1-$n]`;
#       print " gdmp_host_subscribe: \n $gdmpS";
        my $command = "gdmp_host_subscribe -c -S $SEpair[$n] -r $SEpair[1-$n]";
        my $arrayref = timeout ($command, 60);
        my @output = @$arrayref;
        print " gdmp_host_subscribe: $output[0]\n $output[1]";
        $errSum += $output[0];

        ## Creating the individual subdir whenever needed ... and checking the actual contents if already there !
        my $se = $SEpair[$n];
        my $resMk = `edg-gridftp-mkdir gsiftp://${se}$SPs{$se}/$subDir`;
        print " resMk: $se $SPs{$se} -$?- \n$resMk";
        chomp(my $resLs = `edg-gridftp-ls --verbose gsiftp://${se}$SPs{$se}/$subDir`);
        print " resLs1: $se $SPs{$se} -$?- \n$resLs";

        ## Cleaning up left over files ...
        chomp(my @resLs = `edg-gridftp-ls gsiftp://${se}$SPs{$se}/$subDir`);
        if ( $#resLs > -1 ) {
            foreach my $f ( @resLs ) {
                print " ==> Removing file: -$f- \n";
                if ( "$f" eq "" ) { last; }
                my $resDel = `edg-gridftp-rm gsiftp://${se}$SPs{$se}/$subDir/$f`;
            }
        }
        chomp(my $resLs2 = `edg-gridftp-ls --verbose gsiftp://${se}$SPs{$se}/$subDir`);
        print " resLs2: $se $SPs{$se} -$?- \n$resLs2";
        if ( $resLs2 !~ "total 0" ) {
            $rc = 1;
            print STDERR " Dir: $subDir on: $se not created or not empty \n";
        }
    }

    if ( $errSum ) {
        $rc = 1;
        print STDERR " GDMPs are not mutually subscribed: @SEpair \n";
    }

    return $rc;

}


## Managing the GLOBUS locations if they both exist or do not exist ...
sub locationStuff {
    my ($SE1, $SE2, $SP1, $SP2, $subDir, $file, $hostname) = @_;

    ## Initialization of the location stuff ...

        print "\n";
        open CNF, "$ENV{RC_CONFIG_FILE}";
        while (<CNF>) {
            if ( $_ =~ /(.*?)=(.*)/ ) {
                my $var = $1;
                chomp (my $val = "$2");
                print " CNF: $var => $val \n";
                $ENV{$var} = $val;
            }
        }
        close CNF;
        print "\n";
        my $echo = `echo $ENV{RC_REP_CAT_MANAGER_PWD} >> /tmp/$ENV{USER}-$$-pwd.conf`;

        foreach my $se ( $SE1, $SE2 ) {
            my @resD = `globus-replica-catalog -host "$ENV{RC_LOGICAL_COLLECTION}" -manager "$ENV{RC_REP_CAT_MANAGER_DN}" -password /tmp/$ENV{USER}-$$-pwd.conf  -location $se -delete 2>&1`;
            print " resD: on $se with $file -$?- =$#resD= \n@resD";
        }
        unlink "/tmp/$ENV{USER}-$$-pwd.conf";

        my @resRcCARF = `edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s ${hostname}//tmp/${file} -d $SE1/$SP1/$file 2>&1`;
#p      print "   resRcCARF0: $file -$?- \n@resRcCARF";
        print "   resRcCARF0: $file -$?- \n";
        my @resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2/$SP2/$file 2>&1`;
#p      print "   resRcRF0:   $file -$?- \n@resRcRF";
        print "   resRcRF0:   $file -$?- \n";
        my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
#p      print "   resRcDF0:   $file -$?- =$#resRcDF= \n@resRcDF";
        print "   resRcDF0:   $file -$?- =$#resRcDF= \n";
        my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
#p      print "   resRcLR0:   $file -$?- =$#resRcLR= \n@resRcLR";
        print "   resRcLR0:   $file -$?- =$#resRcLR= \n";

	return 0;

}

sub commandProc {

    my ($self, $command, $ctitle,  $file, $stepI) = @_;

    my $resComm = `$command 2>&1`;
    my $stat = $?;
    print "\n $ctitle: $file -$stat- \n$resComm";

	$self->{reqs}->{$stepI}->{mess} = $resComm;
	$self->{reqs}->{$stepI}->{stat1} = ($stat == 0) ? 0 : 1;
	$self->{reqs}->{$stepI}->{stat2} = $stat;
	$self->{step}->{$stepI}->{brfinfo} = "$ctitle";
	$self->{step}->{$stepI}->{detinfo} = $command;

    return ($stat, $resComm);

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
