#!/usr/bin/perl

package BaseTest::EDGLifecycle::MatchMakingScript;

use strict;
use vars qw(@ISA);
use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

MatchMakingScript.pm - A job needed for the Matchmaking test

=head1 SYNOPSIS

MatchMakingScript.pm 

=head1 DESCRIPTION 

B<MatchMakingScript.pm> This module is based on the HelloScript.pm module done by Charles Loomis.
But here the job is done to test the Matchmaking on the EDG 2.0  

=head1 AUTHOR

Charles Loomis (charles.loomis.cern.ch) and Eric Fede (eric.fede@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Eric Fede and Charles Loomis. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "MatchMakingScript: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::MatchMakingScript[options]

END
    ;

    $self->{DescText} = <<'END'


END
    ;
    # new option
    $self->addOption('SEhost=s',
                     'Name of the target SE.',
                     'lxshare0408.cern.ch');

    $self->addOption('VOname=s',
                     'Name of user VO.',
                     'wpsix');

    $self->addOption('RCConf=s',
                     'Path to the edg-replica-manager.conf.',
                     '/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf');


}

# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "MatchMakingScript: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    # Create the list of file to put on SE
    my @listofSEfile = createfilename(3);

    # Create the file on tempory directory
  
    createphysicalfile (@listofSEfile,$tmpdir);

    # Do the copy and registration

    my $copyregisterstatus = CopyAndRegister(@listofSEfile,$tmpdir,$self->{opts}->{VOname},$self->{opts}->{RCConf},$self->{opts}->{SEhost});

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script);
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }
 
    my $toto = AddInputData(@listofSEfile,$self->{opts}->{VOname},$file);

    return 0;
    #return 1;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $site) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
END
    ; 

    print JDL "Requirements = $self->{opts}->{requirements};\n" 
	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if ("$self->{opts}->{retrycount}" ne "");
	#Modify By Fede
	#if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = $self->{opts}->{myproxyhost};\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
SE=`/opt/edg/bin/edg-brokerinfo getCloseSEs`
echo \$SE;
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# create a list of filename with a name with a name depending of the input parameter.
sub createfilename {
	my ($nboffile) = @_;
        my @listofname;
        my @validchars = ('A' .. 'Z', '0' .. '9');

	for (1 .. $nboffile){
        my $filename = '';
        for (1 .. 12) {
             # take a random position
                my $randompos = int(rand(($#validchars+1)));
                $filename .= $validchars[$randompos];
        }
     push @listofname,$filename;
    # put the name on the lista
     }
    return @listofname;
}
sub createphysicalfile {
    my (@file) = @_;
    my $tmpfilepath=$file[$#file];
    my $i;
    print "MatchMakingScript: Random file creation \n";
    for $i (0 .. $#file-1) {
        my $filename="$tmpfilepath/$file[$i]";
	# Create a single file
    	my $commande="dd if=/dev/random of=$filename bs=1024 count=2";
        open(IN,"$commande |") or die $!;
        close IN;
    }

}    


sub CopyAndRegister {
    my (@filelist) = @_;
    my $SEhost=$filelist[$#filelist];
    my $RCconf=$filelist[$#filelist-1];
    my $VO = $filelist[$#filelist-2];
    my $tmpfilepath=$filelist[$#filelist-3];
    my $returnstatus=0;
    my $i;
    print "MatchMakingScript: Copy and register the file on SE \n";
    for $i (0 .. $#filelist-5) {
        print "edg-rm --vo=$VO -i --config=$RCconf copyAndRegisterFile file:$tmpfilepath/$filelist[$i]  -d $SEhost -l lfn:$filelist[$i] >> $tmpfilepath/GUIDLIST";
        print "\n";
    	my $commandresult = `edg-rm --vo=$VO -i --config=$RCconf copyAndRegisterFile file:$tmpfilepath/$filelist[$i]  -d $SEhost -l lfn:$filelist[$i] >> $tmpfilepath/GUIDLIST`;
    	my $flag = ($!) ? 0 : 1;
        if ($commandresult != 0 ){$returnstatus = 1;}
   }
   return $returnstatus;
}

sub AddInputData {
	my (@filelist) = @_;
	my $JDLfile=$filelist[$#filelist];
	my $VONAME=$filelist[$#filelist-1];
        my $i;
        open FILE, ">>$JDLfile";
        print FILE "InputData     = {";
        for $i (0 .. $#filelist-3) {
          # add the SE requieremnt on job description.
          print FILE "\"lfn:$filelist[$i]\",";
        }
        print FILE "\"lfn:$filelist[$#filelist-2]\"};\n";
        print FILE "DataAccessProtocol  = { \"file\",\"rfio\",\"gridftp\" };\n";
        print FILE "VirtualOrganisation = \"$VONAME\";";
	return 0;
}          
# Do any necessary, post-test cleanup.
sub cleanup {
    my $self = shift;

    print "MatchMakingScript: cleanup\n";
    #remove the replica
    my $RCconffile = $self->{opts}->{RCConf};
    my $VOName = $self->{opts}->{VOname};
    my $i;
    my $returnstatus=0;
    my $GUIDID;
    my $commandresult;
    # Open GUID file
    print "MatchMakingScript : Remove the logical file from the replica\n";
    my $GUIDpathfile="/tmp/$ENV{USER}-$$/GUIDLIST";
    open(GUIDD,$GUIDpathfile);
    while ($i = <GUIDD>){
    	chomp($i);
    	$commandresult = `edg-rm --vo=$VOName -i --config=$RCconffile deleteFile $i -a`;
    	my $flag = ($!) ? 0 : 1;
	if ($flag != 0 ){$returnstatus = 1;}
    }
    close (GUID);

    my $dir = $self->{tempdir};
#    rmtree($dir) if $dir;

    return $returnstatus;
} 
# the evaluation of the test
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;

    # Check that the std.out and std.err files exist.
    return 1 if (! (-e "$stdout"));
    return 1 if (! (-e "$stderr"));

    # Check that the standard error has a zero size.
    return 1 if (-s "$stderr");

    # Check the standard output contains the string [OK].
    open FILE, "<$stdout";
    my $value = 1;
    while ($value = <FILE>) {
        chomp $value;
        print $value;
        if ($value eq $self->{opts}->{SEhost}) {
	   return 0;
        }
    }
    return 1;
}
# This ensures that the module returns a true value when loaded.
# If this isn't here, loading the module may incorrectly fail.
1;

