#!/usr/bin/perl

package BaseTest::EDGLifecycle::ProxyRenewal;

use strict;
use vars qw(@ISA);
use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

ProxyRenewal.pm - A test to determine if the proxy renewal works.

=head1 SYNOPSIS

ProxyRenewal.pm

=head1 DESCRIPTION 

B<ProxyRenewal.pm> A test to determine if the proxy renewal mechanism
works.  The test just waits in a loop until checking to see if the
proxy is renewed or if it expires.  This job should be submitted with
a short-lived proxy. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "ProxyRenewal: setContext\n";

    # Add the time option.
    $self->addOption('time=s',
		     'Seconds to wait for renewal',
                     '18000');

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::ProxyRenewal [options] 

END
    ;

    $self->{DescText} = <<'END'

This submits a job which waits in a loop checking the number of
times the proxy is renewed.  The test fails if the proxy expires.

END
    ;

}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "ProxyRenewal: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script, 
			 $self->{opts}->{time});
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $time) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
Arguments     = "$time";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
    
END
    ; 

    print JDL "Requirements = $self->{opts}->{requirements};\n" 
	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl

	use strict;

    my $limit = shift or die "Must supply time limit in seconds.\n";

    my $cmd = 'grid-proxy-info -timeleft';
    my $start = time;

    # Get the current limit on the proxy. 
    my $remainder = `$cmd`;
    chomp($remainder);

    # Count number of proxy renewals.  Loop until the time limit
    # or until the proxy expires. 
    my $nrenew = 0;
    while (((time()-$start)<$limit) && $remainder>=0) {

	my $last = $remainder;

	sleep(60);

	# Determine the time remaining for the proxy.
	$remainder = `$cmd`;
	chomp($remainder);

	# Test if the proxy was renewed. 
	if ($remainder>$last) {
	    $nrenew++;
	    print "Proxy renewed (" . (scalar localtime()) . ")\n";
	}
    }

    # Check if the test was a success.
    if ($remainder<0) {
	print "Proxy expired.\n";
	print "[FAIL]\n";
    } elsif ($nrenew<=0) {
	print "No proxy renewals.\n";
	print "[FAIL]\n";
    } else {
	print "Renewals: $nrenew\n";
	print "[OK]\n";
    }

    exit;
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
