#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::RMSetupTest;

use strict;
use vars qw(@ISA);
use EDGUtils;

use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

RMSetupTest.pm - Check that globusrun authentication works on the hosts
# $Id: RMSetupTest.pm,v 1.8 2003/09/05 14:13:07 loomis Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<RMSetupTest.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "RMSetupTest: setContext\n";

    # Add the resource option.
    $self->addOption('mds=s',
                     'The top-level mds node.',
                     'gppinfo06.gridpp.rl.ac.uk');
    $self->addOption('mdsport=i',
                     'The top-level mds port number.',
                     '2135');
    $self->addOption('mdsname=s',
                     'The mds-vo-name for MDS.',
                     'local');
    $self->addOption('bdii=s',
                     'The default BDII node.',
                     'ccedgli02.in2p3.fr');
    $self->addOption('bdiiport=i',
                     'The BDII port number.',
                     '2170');
    $self->addOption('bdiiname=s',
                     'The mds-vo-name for BDII.',
                     'local');
    $self->addOption('VO=s',
		     'VO to use in the repStorm.',
                     'tstg');

    # The command needed to run this test.
    $self->{commands} = ["grid-proxy-info",
			 "edg-vo-env",
			 "edg-gridftp-rm",
                         "edg-gridftp-ls",
			 "edg-gridftp-mkdir",
#p			 "gdmp_ping",
# 			 "gdmp_host_subscribe",
# 			 "gdmp_job_status",
# 			 "globus-replica-catalog",
# 			 "ldapsearch",
# 			 "edg-replica-manager-replicateFile",
# 			 "edg-replica-manager-copyAndRegisterFile",
# 			 "edg-replica-manager-deleteFile",
# 			 "edg-replica-manager-listReplicas"
 			 "edg-replica-manager"
			 ];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::RMSetupTest [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'
NEEDS TO BE REWRITTEN ...
Check the basic setup of an SE.  This test checks:
 * That the SE GRIS is running.
 * Supported VO(s) are defined in the GRIS.
 * The GridFTP server is running.
 * The storage path exists.
 * The "magic" softlink exists for NFS access (via BrokerInfo).
 * The edg_query_vo_storage cron job is running.
This does not check the data management software or configuration.

This test was supplied by Stephen Burke. 

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "RMSetupTest: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    print "RMSetupTest: runTest\n";

    my $rc;
    my @SEs;
    my %SPs = {};
    if (0) {
	($rc, @SEs) = getSEs($self);
	return $rc if $rc;

	($rc, %SPs) = getSPs($self, @SEs);
	return $rc if $rc;
    } else {
	@SEs = @{$self->{args}};
    }

#   exit 0;

    # Keep a list of all SEs which fail
    my @failed;

    # Loop over each SE pair doing a full test for each.
    for my $n ( 0 .. ($#SEs-1) ) {
	my @SEpair = ($SEs[$n], $SEs[$n+1]);
        my $error = $self->testRMSetupTest(\@SEpair, \%SPs);
        push @failed, @SEpair if ($error);
    }

    # If there are any servers in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("RMSetupTest test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual test is here.  It just takes a server name, extracts the
## supported VOs and runs checks. 
##

sub testRMSetupTest {
    my ($self, $arrayref, $hashref) = @_;
    my @SEpair = @$arrayref;
    my %SPs = %$hashref;
    my $rc = 0;
    print "  Working this SE pair: @SEpair with these mount points: \n";
    foreach my $se ( @SEpair ) { 
	print "     FS: $se => $SPs{$se} \n";
    }

    my $subDir = "testGG";
    my $edgRMopts = "--config=edg-replica-manager-CertTB.conf --insecure --vo=wpsix";
#    my $edgRMopts = "--vo=atlas --verbose";

    $rc = mutualSPs($subDir, \@SEpair, \%SPs) if (0);
    return $rc if $rc;

    ## Create a temporary file ...
    my $file = "$ENV{USER}-$$-rmstorm.txt";
    open TMP, ">/tmp/$file";
    print TMP "Good Morning to all Everybody !\n";
    close TMP;

    my $nn = int(rand(2));
    my $SE1 = $SEpair[$nn];
    my $SE2 = $SEpair[1-$nn];
    my $SP1 = "$SPs{$SE1}/$subDir" if (0);
    my $SP2 = "$SPs{$SE2}/$subDir" if (0);
    chomp(my $hostname = `hostname`);

    ## Initialization of the location stuff ...
    my $needed = 0;
    if ( $needed ) { 
	$rc = locationStuff ($SE1, $SE2, $SP1, $SP2, $subDir, $file, $hostname);
	return $rc if $rc;
    }

##edg2.0    my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s ${hostname}//tmp/${file} -d $SE1/$SP1/$file";
    my $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:/tmp/${file} -l lfn:${file} -d $SE1";
    print " CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
##edg2.0	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
    }
    my %PFNs = ();
    my %GUIDs = ();
    my %SRMs = ();
    $GUIDs{$file} = $resRcCARF;
#    $PFNs{"$SE1/$SP1/$file"} = "created";
    $PFNs{"$SE1/$file"} = "created";

##edg2.0    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    my @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:${file} 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
##edg2.0        my @i = split /: /, $l;
        my @i = split /[:\/]/, $l;
	if ( @i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $gsrm previously existed: $l ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    $SRMs{$l} = "existing";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

##edg2.0    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    my $resRcRF = `edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2 2>&1`;

#GOOD    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2/$SP2/$file 2>&1`;
#GOOD!    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2/$SP2/$file 2>&1`;
#BAD    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
##edg2.0        print STDERR " File: ${subDir}/${file} on: $SE2 not correctly registered or created \n";
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcRF);
	if ( $resRcRF !~ "srm:" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $resRcRF \n";
	    return 1;
	} else {
	    $PFNs{"$SE2/$file"} = "created";
	    $SRMs{$resRcRF} = "existing";
	}
    }
##    $PFNs{"$SE2/$SP2/$file"} = "created";

##edg2.0       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
       @resRcLR = `edg-replica-manager $edgRMopts listReplicas lfn:${file} 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
	chomp $l;
##edg2.0	my @i = split /: /, $l;
        my @i = split /[:\/]/, $l;
	if ( @i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
	    print STDERR " Step: $SE2 File: $gsrm previously existed: $l ! \n";
	    return 1;
	} else {
	    $PFNs{$gsrm} = "existing";
	    if ( ! $SRMs{$l} ) {
		print STDERR " Step: $SE2 SRM: $l not previously existing ! \n";
		return 1;
	    }
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    exit 0 if (0);  ##  TEMP


    ## Section where the JDL job is being run ...

    ## Adding required options to JDL file ...
    my $resE = `echo '    InputData = "lfn:${subDir}/${file}";' >> $self->{jdl}`;
    $resE =    `echo '    Arguments = "${file}";'    >> $self->{jdl}`;
##edg2.0    $resE =    `echo '    ReplicaCatalog = "$ENV{RC_LOGICAL_COLLECTION}";' >> $self->{jdl}`;


    my $resJob = $self->SUPER::runTest();
    if ( $resJob ) { print " RMSetupTest::Job : Failed ! \n"; }
    ## Temp ...
    if (1) { exit 0;}

    my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
    print "\n resRcDF3: $file -$?- =$#resRcDF= \n@resRcDF";
    foreach my $l ( @resRcDF ) {
	chomp $l;
        my @i = split /deleting /, $l;
        $PFNs{$i[1]} = "deleting";
    }

       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    print "\n resRcLR3: $file -$?- =$#resRcLR= \n";
    if ( $#resRcLR > -1 ) {
	foreach my $l ( @resRcLR ) {
	    chomp $l;
	    my @i = split /: /, $l;
	    $PFNs{$i[1]} = "still-existing";
	}
    }

    print "\nPFNs: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    foreach my $se ( $SE1, $SE2 ) {
	my $resDel = `edg-gridftp-rm gsiftp://${se}$SPs{$se}/$subDir`;
    }

## To delete a location:
## source grc.conf
## 1183  globus-replica-catalog -host "$RC_LOGICAL_COLLECTION" -manager "$RC_REP_CAT_MANAGER_DN" -password pwd.conf  -location lxshare0232.cern.ch -delete
## 1184  globus-replica-catalog -host "$RC_LOGICAL_COLLECTION" -manager "$RC_REP_CAT_MANAGER_DN" -password pwd.conf  -location lxshare0303.cern.ch -delete
    unlink "/tmp/$file";

    return $rc;
}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.)
sub setupTest {
    my $self = shift;

    print "RMSetupTest: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};
#p  print " tmpdir: $tmpdir \n";

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script);
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


sub createJDL {
    my ($self, $dir, $script, $site) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
    Executable    = "$sname";
    StdOutput     = "std.out";
    StdError      = "std.err";
    InputSandbox  = {"$script"};
    OutputSandbox = {"std.out","std.err"};
#   Executable = "World.sh";
#   Arguments = "test/HelloWorld";
##  Arguments = "$ENV{HW_INP}";
    StdInput = "/dev/null";
#   StdOutput = "HW.out";
#   StdError = "HW.err";
#   InputSandbox = "World.sh";
#   OutputSandbox = {"HW.out", "HW.err"};
#   InputData = "LF:test/HelloWorld";
##  InputData = "LF:$ENV{HW_INP}";
#   ReplicaCatalog = "ldap://testbed014.cern.ch:12389/lc=TestCollection,rc=ITeamRC,dc=testbed014,dc=cern,dc=ch";
#    ReplicaCatalog = "$ENV{RC_LOGICAL_COLLECTION}";
    DataAccessProtocol = "file";
END
    ;

    print JDL "Requirements = $self->{opts}->{requirements};\n"
        if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n"
        if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/bash
echo Arg: $1
GSF=`edg-brokerinfo getSelectedFile $1 file`
echo GSF: $GSF
TFN=`echo $GSF | cut -d " " -f 2`
echo Printing local file $TFN:
cat $TFN
echo "+-+-+-+-+-+-+-+-"
/bin/uname -a
/bin/echo "Hello From Sgript"
/bin/echo "Goodbye. [OK]"
GSE=`edg-brokerinfo getSEs`
echo SEs: $GSE
CSE=`edg-brokerinfo getCloseSEs`
echo getCloseSEs: $CSE
CSE=`echo $CSE | cut -d " " -f 1`
SMP=`edg-brokerinfo getSEMountPoint $CSE`
echo getSEMountPoint: $SMP
END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


## To extract the available SE list for a given VO 
sub getSEs {
    my $self = shift;

    ## Set up some variables ...
#   my $edgl = `$ENV{EDG_LOCATION}/bin/edg-vo-env --shell=csh tstg`;
    my $edgl = `edg-vo-env --shell=csh $self->{opts}->{VO}`;
#p  print " EDGL: $edgl \n";
    my @ls = split /\n/, "$edgl";
    foreach my $l ( @ls ) {
	my @its = split /\s+/, $l;
	chop($its[2]);
	$ENV{$its[1]} = $its[2];
	print " ENV $its[1]: $ENV{$its[1]} \n";
    }
    my $gdmb = dirname( dirname( $ENV{RC_CONFIG_FILE}));
#   my $gdmb = basename( $ENV{RC_CONFIG_FILE});
    my $gdmp = `grep ^GDMP_LOCAL_HOST ${gdmb}/gdmp.shared.conf`;
#p  print " GDMP: $gdmp";
    chomp($gdmp);
    my @it = split /=/, $gdmp;
    $ENV{$it[0]}= $it[1];
    print " ENV $it[0]: $ENV{$it[0]} \n";

    # Collect the information from MDS.
    my $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{mds} .
        ":" .
	    $self->{opts}->{mdsport} .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=computingelement)(objectclass=storageelement))'" .
	    " SEId ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command.
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on MDS');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %mdsres;
    my @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $mdsres{$1} =1 if ($line =~ m/([CS]EId:\s*[^\s]+)\s*/);
    }

## On CertTB: MDS==0242:2135, BDII==0243:2170
## On  DevTB: MDS==0406:2135, BDII==0410:2170
## On CertTB: ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid'
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBind)(objectclass=storageelement))' GlueCESEBindSEUniqueID
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBind)(objectclass=storageelement))' GlueCESEBindCEAccesspoint
## ldapsearch -LLL -H ldap://lxshare0243.cern.ch:2170 -x -b 'mds-vo-name=local,o=grid' '(|(objectclass=GlueCESEBindGroup)(objectclass=storageelement))' GlueCESEBindGroupSEUniqueID
## Use lxshare0410 for DevTB BDII

    # Collect the information from BDII.
    $command = "ldapsearch -LLL -H ldap://" .
        $self->{opts}->{bdii} .
        ":" .
	    $self->{opts}->{bdiiport} .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(|(objectclass=computingelement)(objectclass=storageelement))'" .
	    " SEId ";
    $arrayref = timeout($command,90);
    @output = @$arrayref;

    # Catch errors from the execution of the command.
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out on BDII');
        return 1;
    }

    # Pull out the list of CEs and SEs from the output.
    my %bdiires;
    @lines = split("\n","$output[1]\n");
    foreach my $line (@lines) {
        $bdiires{$1} =1 if ($line =~ m/([CS]EId:\s*[^\s]+)\s*/);
    }

## For this, run it with:
## ./edg-testbed-test BaseTest::EDGLifecycle::RMSetupTest --mds=lxshare0233.cern.ch --bdii=lxshare0231.cern.ch

#   print map { "$_ => $mdsres{$_}\n" } keys %mdsres;
    print " BDIIres: @{[ %bdiires ]}\n";
    print " MDSres:  @{[ %mdsres ]}\n";

    my %seen;
    @seen{keys %mdsres} = ();
    my @unseen = grep { ! exists $seen{$_} } keys %bdiires;
    print " NOT in BDII: @unseen $#unseen\n";
    my $unSeenBdii = $#unseen;
    %seen = ();
    @unseen = ();
    @seen{keys %bdiires} = ();
    @unseen = grep { ! exists $seen{$_} } keys %mdsres;
    print " NOT in MDS:  @unseen $#unseen\n";
    my $unSeenMds = $#unseen;

    if ( ($unSeenMds+$unSeenBdii) != -2 ) {
	$self->setErrorMessage('MDS/BDII mismatch in SE resources');
	return 1;
    }
    @unseen = keys %mdsres;
    my @SEs = ();
#    my %SPs = ();
#    my %SPst= ();
    foreach my $se ( @unseen ) {
	$se =~ s/SEId: //;
	push @SEs, $se;
    }
    print " SEs: @SEs \n";

    if ( $#SEs <= 0 ) {
	$self->setErrorMessage('Less than 2 SEs: Test is Impossible. \n');
	return 1;
    }

    ## Temp ...
    ##if (1) { exit 0;}

    return (0, @SEs);

}


## To extract the SE Mount Points and all that ...
sub getSPs {

    my $self = shift;
    my @SEs = @_;

    my %SPs = ();
    my %SPst= ();

    ## Check that "tstg" VO exists on all SEs ...
    foreach my $se ( @SEs ) {
        my $command = "ldapsearch -x -h $se -p 2135 -b \"mds-vo-name=local,o=grid\" \"objectclass=storageelement\" sevo | grep -i sevo:";
        my $arrayref = timeout($command, 90);
        my @output = @$arrayref;
#p      print " SE: $se \n @output \n";

    # Catch errors from the execution of the command.
	if ($output[0]==1) {
	    $self->setErrorMessage('could not fork ldapsearch');
	    return 1;
	} elsif ($output[0]==2) {
	    $self->setErrorMessage('ldapsearch timed out on SE $se');
	    return 1;
	}
        my @lines = split("\n","$output[1]\n");
        my @voex = grep { /$self->{opts}->{VO}/ } @lines;
        print " VOEX: $se @voex \n";
        if ( $#voex != 0 ) {
            $self->setErrorMessage("VO \"$self->{opts}->{VO}\" does not exist on this SE: $se");
            return 1;
        }
        my @p = split /\s+/, $voex[0];
        @p = split /:/, $p[1];
        if ( $p[0] ne "$self->{opts}->{VO}" ) {
            $self->setErrorMessage("Wrong VO \"$self->{opts}->{VO}\" exists on this SE: $se, $voex[0]");
            return 1;
        }
        $SPs{$se} = $p[1];


        ## Another check about these mount points ...
        my $gdmpSt = `gdmp_job_status -S $se -c gdmp.conf | grep GDMP_STORAGE_DIR`;
#p      print " GDMPST: $se $gdmpSt";
        @p = split /=/, $gdmpSt;
        $SPst{$se} = $p[1];
        chomp($SPst{$se});
        if ( $SPst{$se} ne $SPs{$se} ) {
            print STDERR " Mount point mismatch on SE $se: $SPst{$se} $SPs{$se} \n";
            $self->setErrorMessage("Mount point mismatch between GDMP_STORAGE_DIR and LDAPSEARCH on this SE: $se");
            return 1;
        }

    }
    print "SPs: \n", map { "$_ => $SPs{$_}\n" } keys %SPs;

    return (0, %SPs);

}


## Mutual subscription check for SPs
sub mutualSPs {
    my ($subDir, $arrayref, $hashref) = @_;
    my @SEpair = @$arrayref;
    my %SPs = %$hashref;
    my $rc = 0;

    ## Mutual subscription test ...
    my $errSum = 0;
    for my $n ( 0..1 ) {
#       my $gdmpS = `gdmp_host_subscribe -c -S $SEpair[$n] -r $SEpair[1-$n]`;
#       print " gdmp_host_subscribe: \n $gdmpS";
        my $command = "gdmp_host_subscribe -c -S $SEpair[$n] -r $SEpair[1-$n]";
        my $arrayref = timeout ($command, 60);
        my @output = @$arrayref;
        print " gdmp_host_subscribe: $output[0]\n $output[1]";
        $errSum += $output[0];

        ## Creating the individual subdir whenever needed ... and checking the actual contents if already there !
        my $se = $SEpair[$n];
        my $resMk = `edg-gridftp-mkdir gsiftp://${se}$SPs{$se}/$subDir`;
        print " resMk: $se $SPs{$se} -$?- \n$resMk";
        chomp(my $resLs = `edg-gridftp-ls --verbose gsiftp://${se}$SPs{$se}/$subDir`);
        print " resLs1: $se $SPs{$se} -$?- \n$resLs";

        ## Cleaning up left over files ...
        chomp(my @resLs = `edg-gridftp-ls gsiftp://${se}$SPs{$se}/$subDir`);
        if ( $#resLs > -1 ) {
            foreach my $f ( @resLs ) {
                print " ==> Removing file: -$f- \n";
                if ( "$f" eq "" ) { last; }
                my $resDel = `edg-gridftp-rm gsiftp://${se}$SPs{$se}/$subDir/$f`;
            }
        }
        chomp(my $resLs = `edg-gridftp-ls --verbose gsiftp://${se}$SPs{$se}/$subDir`);
        print " resLs2: $se $SPs{$se} -$?- \n$resLs";
        if ( $resLs !~ "total 0" ) {
            $rc = 1;
            print STDERR " Dir: $subDir on: $se not created or not empty \n";
        }
    }

    if ( $errSum ) {
        $rc = 1;
        print STDERR " GDMPs are not mutually subscribed: @SEpair \n";
    }

    return $rc;

}


## Managing the GLOBUS locations if they both exist or do not exist ...
sub locationStuff {
    my ($SE1, $SE2, $SP1, $SP2, $subDir, $file, $hostname) = @_;

    ## Initialization of the location stuff ...

        print "\n";
        open CNF, "$ENV{RC_CONFIG_FILE}";
        while (<CNF>) {
            if ( $_ =~ /(.*?)=(.*)/ ) {
                my $var = $1;
                chomp (my $val = "$2");
                print " CNF: $var => $val \n";
                $ENV{$var} = $val;
            }
        }
        close CNF;
        print "\n";
        my $echo = `echo $ENV{RC_REP_CAT_MANAGER_PWD} >> /tmp/$ENV{USER}-$$-pwd.conf`;

        foreach my $se ( $SE1, $SE2 ) {
            my @resD = `globus-replica-catalog -host "$ENV{RC_LOGICAL_COLLECTION}" -manager "$ENV{RC_REP_CAT_MANAGER_DN}" -password /tmp/$ENV{USER}-$$-pwd.conf  -location $se -delete 2>&1`;
            print " resD: on $se with $file -$?- =$#resD= \n@resD";
        }
        unlink "/tmp/$ENV{USER}-$$-pwd.conf";

        my @resRcCARF = `edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s ${hostname}//tmp/${file} -d $SE1/$SP1/$file 2>&1`;
#p      print "   resRcCARF0: $file -$?- \n@resRcCARF";
        print "   resRcCARF0: $file -$?- \n";
        my @resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1/$SP1/$file -d $SE2/$SP2/$file 2>&1`;
#p      print "   resRcRF0:   $file -$?- \n@resRcRF";
        print "   resRcRF0:   $file -$?- \n";
        my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
#p      print "   resRcDF0:   $file -$?- =$#resRcDF= \n@resRcDF";
        print "   resRcDF0:   $file -$?- =$#resRcDF= \n";
        my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
#p      print "   resRcLR0:   $file -$?- =$#resRcLR= \n@resRcLR";
        print "   resRcLR0:   $file -$?- =$#resRcLR= \n";

	return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
