#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack;

use File::Basename;
use EDGUtils;

use POSIX qw(:signal_h :errno_h :sys_wait_h);
use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

Stack.pm - Check that globusrun authentication works on the hosts
# $Id: Stack.pm,v 1.15 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<Stack.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

#    print "Stack: setContext\n";
    $self->{debug} = 1;  ## auxiliary to new printLocation method.
    $self->printLocation( (caller(0))[3] );

    # Add some storm size options.

    $self->addOption('zombSlep=i',
		     'Monitoring idling time (in sec).',
                     '10');
    $self->addOption('childTimeout=i',
		     'Elapse time after which the child is killed (in sec).',
                     '1800');
    $self->addOption('pollingPeriod=i',
		     'Polling period for edg-job-status (in sec).',
                     '30');
    $self->addOption('batchSleep:i',
		     'Sleep time for the batch job.');
    $self->addOption('jobCancel=i',
		     "To run a edg-job-cancel --all before starting the submission.\n\tIt specifies the waiting time in sec after this command is run.");
    $self->addOption('resubmit!',
		     'Allow resubmitting a request when it failed during edg-job-submit.',
		     1);
    $self->addOption('useCEList',
		     'Use CE list build from a pre-matching test to submit the requests (disables RB matchmaking).');

    # Commands necessary to run this test.
    $self->{commands} = [dgjob("submit"), 
			 dgjob("get-output"), 
			 dgjob("status"),
			 dgjob("cancel"),
			 dgjob("get-logging-info"),
			 "SubmitUtils.pm",
			 "grid-proxy-info"];

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack [options]

END
    ;

    $self->{DescText} = <<'END'

This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "";
        $self->{debug} = 1;  ##  To print subroutine location headers

## Given thru options ...
	$self->{maxStack}  = "";
	$self->{maxSubs}   = "";
	$self->{reqLapse}  = "";
	$self->{zombSlep}  = "";
	$self->{serie}     = "";

## Working variables ...
	$self->{harvest} = "";
	$self->{zombies} = "";
	$self->{user1}   = "";

# Hashes 
	$self->{plist} = {};    ## Hash
	$self->{hlist} = {};    ## Hash
	$self->{reqs}  = {};    ## Hash

	$self->{jdlFile} = "";
	$self->{shFile}  = "";

	$self->{outFile} = "";
	$self->{saveOut} = "";

## Internals ...
	$self->{date}     = "";
	$self->{hostname} = "";

}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

#    print "Stack: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

    chomp($self->{hostname} = `hostname`);
    chomp(my $date = "@" . `date`);
    $date =~ s/\s+/_/g;
    $date .= "@";
    $self->{date} = $date;

    ## Specific to the Stack ...
    $self->{maxStack} = $self->{opts}->{maxStack};
    $self->{maxSubs} = $self->{opts}->{maxSubs};
    $self->{reqLapse} = $self->{opts}->{reqLapse};
    $self->{zombSlep} = $self->{opts}->{zombSlep};
    $ENV{STACK_SERIE} = $self->{opts}->{serie};
    $self->{outFile} = "/tmp/$ENV{USER}-$$-"."$self->{type}.log$ENV{STACK_SERIE}";
    $ENV{STACK_POLLING_PERIOD} = $self->{opts}->{pollingPeriod};
    $ENV{STACK_CHILD_TIMEOUT} = $self->{opts}->{childTimeout};

    if ( $self->{opts}->{batchSleep} ) {
	if ( $self->{opts}->{childTimeout} < $self->{opts}->{batchSleep} ) {
	    print STDERR " When using --batchSleep option, take care to increase --childTimeout (default=1800 sec) \n by the same value\n";
	    exit 1;
	}
    }

    ## Redirect STDOUT and STDERR to outFile
    $self->{saveOut} = switchStdouterr($self->{outFile}) if (0) ;

    # Create the necessary setup used prior to the Storm itself (variables ...)
    $error = $self->setupEnvironment();
    if ($error) {
        $self->setErrorMessage('Specific Environment Setup failed');
        return 1;
    }

    ## Check and/or create the Storm Specific setup ...
    $error = $self->stormSetup();
    if ($error) {
        $self->setErrorMessage('Specific Storm Setup failed');
        return 1;
    }

#old    my $namJDL = "/tmp/$ENV{USER}-$$-"."$self->{type}.jdl";
#old    my $namSCR = "/tmp/$ENV{USER}-$$-"."$self->{type}.sh";
    my $namJDL = "$self->{tempdir}/"."$self->{type}.jdl";
    my $namSCR = "$self->{tempdir}/"."$self->{type}.sh";


    # Create the script for this job.
##    my $script = $self->createScript($tmpdir);
    my $script = $self->createScript($namSCR);
    if ($script) {
	$self->{shFile} = $script;
    } else {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
##    my $file = $self->createJDL($tmpdir, $script, $self->{opts}->{site});
    my $file = $self->createJDL($namJDL, $script, $self->{opts}->{site});
    if ($file) {
        $self->{jdlFile} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    ## Set the Child Reaper up ...
##    $self->{zombies} = 0;
##    $SIG{CHLD} = sub { $self->{zombies}++ };

## gg for XML creation and processing ...
    if ( $ENV{MAINSCRIPT} ) {
#old	my $base = "/tmp/$ENV{USER}-$$";
	my $base = "$self->{tempdir}";
#	$self->{conffile} = "$base/GridFTPconf";
##	$self->{outfile}  = "$base/GatekeeperOut";
	$self->{xmlfile}  = "$base/".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";
    }

    ## Print a banner for the beginning of the test itself
    print " SUM Checking GG Type: $self->{type} Serie: $ENV{STACK_SERIE} Jobs: $self->{maxSubs} Date: $self->{date} Delay: $self->{reqLapse} and StackSize: $self->{maxStack} \n";

    ## Erase any left over job for this user ...
    if ( $self->{opts}->{jobCancel} ) { ## temporary disabled: does not work very well in edg-1.4.3 ...
	my $cmd = dgjob("cancel")." --all --noint";
	my $ca = `$cmd 2> /dev/null `;
	my $sleep = `sleep $self->{opts}->{jobCancel}`;
	print " CANCEL: $ca";
    }

    return 0;
}


# This takes the JDL file and submits it.  This overrides the original
# to allow the multiple streams and multiple jobs.
sub runTest {
    my $self = shift;

#    print "Stack: runTest\n";
    $self->printLocation( (caller(0))[3] );

    ## To set the Child Reaper up ...
    $self->{zombies} = 0;
    $SIG{CHLD} = sub { $self->{zombies}++ };

    ## To be able to catch CTRL-C ... (USR1 => kill -10 under RH7.3)
    $self->{user1} = 0;
    $SIG{INT}  = sub { print STDOUT " Signal SIGINT received: $$ \n";  $self->{user1}++ };
    $SIG{USR1} = sub { print STDOUT " Signal SIGUSR1 received: $$ \n"; $self->{user1}++ };

#my $ph  = 0;
#my $pid = 0;
my $index = 0;
$self->{harvest} = 0;
$self->{plist} = {};
$self->{hlist} = {};
my $currTime = time();
my $reqLapse = $self->{reqLapse};

MAIN: while ( 1 ) {

## To actually enforce the delay ...
  if ( (time()-$currTime) < $reqLapse && $index > 0 ) {
    my $sleep = `sleep $reqLapse`;
    next MAIN;
  } 

  if ( ! $self->{user1} ) {

## Submitting the request ...
  if ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} && $index < $self->{maxSubs} ) {

    ## Building the argument list
#old    my $jdlTemp = "/tmp/$ENV{USER}-$$-tmparg-$index.jdl";
    my $jdlTemp = "$self->{tempdir}/tmparg-$index.jdl";
    my $bid = `cat $self->{jdlFile} > $jdlTemp`;

    my $rc = $self->addArguments($jdlTemp, $index);
##    exit 1 if ( ! $rc );
    return $rc if $rc;

#p    my @resJDL = `cat $jdlTemp`;
#p    print STDOUT " resJDL: \n@resJDL\n";

      ## Creating the log name (if required ...)
#      my $tmpdir = $self->{tempdir};
#old      my $logFile = "/tmp/$ENV{USER}-$$-"."$self->{type}-${index}.log";
      my $logFile = "$self->{tempdir}/"."$self->{type}-${index}.log";
      $self->{reqs}->{$index}->{olog} = $logFile;
      my $outFile = "$self->{tempdir}/"."$self->{type}-${index}.out";

##    my ($pid, $ph) = submit($jdlTemp, $index, $logFile);
    my ($pid, $ph) = submitTimeout($jdlTemp, $index, $logFile, $outFile, $self->{opts}->{childTimeout});
    $self->{plist}->{$pid} = $index;
    $self->{hlist}->{$pid} = $ph;
    $self->{reqs}->{$index}->{pid} = $pid;
#useless    $self->{reqs}->{$index}->{ph}  = $ph;
    print "PID $index / $self->{maxSubs} $pid \n";
    $index++;
    $currTime = time();
    my $sleep = `sleep $reqLapse`;

  }

}  ## End of if_user1

## We come here when we're done ...
  if ( $self->{harvest} >= $self->{maxSubs} ) { 
#     print " We are done with $index $self->{harvest} ", scalar(keys(%{$self->{plist}})), " \n";
#     print " SUM Ending GG $self->{type}: serie $ENV{STACK_SERIE}: We are done with $index $self->{harvest} \n";
#     last; 
      my $rc = $self->submitOver($index);
      last MAIN; 
  }

## Harvesting the dead children ...
  my $nzomb = 0;
  while ( 1 ) {
      last if $self->{zombies};
#old      next MAIN if ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} );
      if ( $self->{user1} ) {
	  print STDOUT " SelfUsr1: $self->{harvest} $self->{maxSubs} $index \n";
#bad?	  if ( $self->{harvest} >= $self->{maxSubs} ) {
	  if ( $self->{harvest} >= $index ) {
	      my $rc = $self->submitOver($index);
	      last MAIN;
	  }
      } else {
	  next MAIN if ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} );
      }
      $nzomb++;
      last if ( $nzomb*$self->{zombSlep} > 900 );
      my $sleep = `sleep $self->{zombSlep}`;
  }
  print "ZOMBIES $self->{zombies} $nzomb\n";
  $self->{zombies} = 0;

  my $rc = $self->waitforChildren();

}  ## End of MAIN while

  return 0;

}  ## End of runTest routine


sub submitOver {

    my $self = shift;
    my $index = shift;

    print " We are done with $index $self->{harvest} ", scalar(keys(%{$self->{plist}})), " \n";
    print " SUM Ending GG $self->{type}: serie $ENV{STACK_SERIE}: We are done with $index $self->{harvest} \n";

    return 0;

}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $site) = @_;

#    print "Stack: createJDL\n";
    $self->printLocation( (caller(0))[3] );

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
END
    ; 

    print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my ($self, $dir) = @_;

#    print "Stack: createScript\n";
    $self->printLocation( (caller(0))[3] );

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
/bin/echo "Hello From Script"
#/bin/sleep 30
/bin/echo "Goodbye. [OK]"
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Build any necessary setup prior to the storm.
sub setupEnvironment {
    my $self = shift;

#    print "Stack: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

    return 0;
}


# Build any necessary setup for during the storm.
sub stormSetup {
    my $self = shift;

#    print "Stack: stormSetup\n";
    $self->printLocation( (caller(0))[3] );

    return 0;
}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "Stack: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    return 0;
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

#    print "Stack: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    return 0 if (0);

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	$self->{globalresult} .= " $self->{reqs}->{$index}->{stat1}" if $self->{globalresult};
	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
    }

    return 0 if ( ! -e $self->{outFile} );

    restoreStdouterr($self->{saveOut}) if (1);

    my $resOutA = `cat $self->{outFile} `;
    print " All Output of GG---GG $self->{type} done: \n$resOutA \n";

    my $resOut = `grep SUM $self->{outFile} `;
    print " After GG $self->{type} done: \n$resOut \n";

    return 0;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

#    print "Stack: checkPrerequisites\n";
    $self->printLocation( (caller(0))[3] );

       $rc = checkProxyTime(2);
    $self->setErrorMessage('user proxy is not valid long enough (at least 2 hours)') if $rc;

    return $rc;
}


# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

#    print "Stack: cleanup\n";
    $self->printLocation( (caller(0))[3] );

#     unlink $self->{jdlFile};
#     unlink $self->{shFile};
#     unlink $self->{outFile};

    my $rc = 0;
    $rc = $self->SUPER::cleanup() if (1);

#     my $dir = $self->{tempdir};
#     rmtree($dir) if $dir;

    return $rc;
}


sub waitforChildren {
    my $self = shift;

  my @pidL = reaper();
  print " Reaper List: @pidL \n";

## Reading the stack of dead children ...
  my $pid = 0;
  foreach $pid ( @pidL ) {

      $self->{harvest}++;
      my $line = "";
      my $vue = 0;
      my @mstore = ();
#oldi      my $index;
      my $index = $self->{plist}->{$pid};
      my $ph    = $self->{hlist}->{$pid};
      while ( $line = readline($ph) ) {
	push @mstore, $line;
##prev	if ( $line =~ /Linux|Aborted|nothing|Error|Summary/ ) { 
	if ( $line =~ /Status:|Linux|Aborted|nothing|Error|Summary/ ) { 
	  if ( $line =~ s/^ Summary / / ) { 
	      print " SUM $line"; 
	      $line =~ s/^\s+//;
	      my @m = split/\s+/, $line;
#	      if ( $line =~ /=>/ ) { my @l = split/, /, $line; $self->fillResHash(@l); next; }
#oldi	      if ( $line =~ /=>/ ) { chomp(my $ll=$line); $index = $self->fillResHash($ll); }
	      if ( $line =~ /=>/ ) { chomp(my $ll=$line); $index = $self->fillResHashN($index, $ll); }
	      $vue++;
	      next;
	  }
##NO	  print " Read $self->{plist}->{$pid} $line"; 
	  if ( $line =~ /Aborted|Error/ ) { $self->{maxSubs}++ if $self->{opts}->{resubmit}; }
	  $vue++;
	}
      }
      close ($ph);

      if ( $vue == 0 ) {
	print "ERROR BIZarre \n";
	print @mstore;
	$self->{maxSubs}++ if $self->{opts}->{resubmit};
	if ( ! $self->{reqs}->{$index}->{stat1} ) {
	    $self->{reqs}->{$index}->{stat1} = "[FAIL]";
	    $self->{reqs}->{$index}->{stat2} = "BIZARRE";
	}
      }

      if ( ! -e $self->{reqs}->{$index}->{olog} || $self->{reqs}->{$index}->{stat2} eq "TIMEOUT" ) {
	  delete $self->{reqs}->{$index}->{olog};
      } else {
	  print "ERROR Ugly failure ! The command returned: \n";
	  system("cat $self->{reqs}->{$index}->{olog}");
      }


      delete $self->{plist}->{$pid};
      delete $self->{hlist}->{$pid};

  } ## end of foreach loop

    return 0; # ($harvest, $maxSubs);

}


## Submit the JDL with a fork but keep reading from it thru PH filehandle
sub submit {

  local *PH;
  my $ljdl  = shift;
  my $index = shift;

  my $pid = 0;

  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

  if ( $pid ) { # Parent

    print "Submit $index $pid $? \n";
    return ($pid, *PH);

  } else { # Child

    STDOUT->autoflush(1);
    exec("SubmitUtils.pm", "$ljdl", "$index");
    exit;    ## for safety ...

  }

}


## Submit the JDL with a fork but keep reading from it thru PH filehandle
sub submiTimeoutGood {

  local *PH;
  my $ljdl  = shift;
  my $index = shift;
  my $timeout = 500;

  my $pid = 0;

  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

  if ( $pid ) { # Parent

    print "Submit $index $pid $? \n";
    return ($pid, *PH);

  } else { # Child

    STDOUT->autoflush(1);

        local $SIG{ALRM} = sub { die "time out" }; 
        alarm $timeout;

    exec("SubmitUtils.pm", "$ljdl", "$index");

        alarm 0;

    exit;    ## for safety ...

  }

}


## Submit the JDL with a fork but keep reading from it thru PH filehandle
sub submitTimeout {

  local *PH;
  my $ljdl    = shift;
  my $index   = shift;
  my $logFile = shift;
  my $outFile = shift;
  my $timeout = shift;
##  my $timeout = $self->{opts}->{childTimeout};

  my $pid = 0;

  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

  if ( $pid ) { # Parent

    print "Submit $index $pid $? \n";
    return ($pid, *PH);

  } else { # Child

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

        local $SIG{ALRM} = sub { print STDERR " Signal USR1 sent: $$ \n"; kill 'USR1' => $$ }; 

    eval {
	print STDERR " Starting the CHILD process: $$ for timeout: $timeout sec. \n";
        alarm $timeout;

    exec("SubmitUtils.pm", "$ljdl", "$index", "$logFile", "$outFile");

        alarm 0;
    };

    exit;    ## for safety ...

  }

}


## Retrieves the dead children, and store their pid on the local stack
sub reaper {

    my @stack = ();
    my $stpid;
    while ( ($stpid = waitpid(-1, &WNOHANG)) > 0 ) {
	# Put $stpid on a stack ...
	push @stack, $stpid;
    }

    return @stack;

}


sub fillResHash {
    my ($self, $sel) = @_;
    my $index;
    my @l = split/, /, $sel;
    foreach my $it ( @l ) {
	my ( $k, $v ) = split /=>/, $it;
	if ( $k eq "index" ) { $index = $v; next; }
#good	if ( $k =~ /index|flag|none|mess/ ) { next; }
	elsif ( $k =~ /flag|none/ ) { next; }
	$self->{reqs}->{$index}->{$k} = $v;
    }

    return $index;

}


sub fillResHashN {
    my ($self, $index, $sel) = @_;
#    my $index;
    my @l = split/, /, $sel;
    foreach my $it ( @l ) {
	my ( $k, $v ) = split /=>/, $it;
#	if ( $k eq "index" ) { $index = $v; next; }
	if ( $k eq "index" ) { next; }
#good	if ( $k =~ /index|flag|none|mess/ ) { next; }
	elsif ( $k =~ /flag|none/ ) { next; }
	$self->{reqs}->{$index}->{$k} = $v;
    }

    return $index;

}


sub buildXMLFile {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
    my $brf_info = "Rather Simple Storm";
    $brf_info = $self->{brfinfo} if ( $self->{brfinfo} );
    my $Jdl = `cat $self->{jdlFile}`;
    my $Scr = `cat  $self->{shFile}`;
#old    my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    my $det_info = "$brf_info.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

    my $cols = {};
    my $order = 0;
    my $machType = 'mace';
#    my $machType = 'mawn';
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
        $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
        if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++; }
	push @lof_rows, "Job_".$index;
    }

    foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#    foreach my $col ( sort keys %{$cols} ) {
#p	print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#	$col =~ s/\./ /g;
	push @lof_cols, "$col";
#p	print "  $col \n";
    }

    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
	my $logfn;
	my $logf;
	if ( $retv == 0 ) {
	    $logfn = $self->{reqs}->{$index}->{olout};
	    $logf = `cat $logfn` if $logfn;
	} elsif ( $retv == 5 ) {
##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	} else {
##edg2.0	    if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
##edg2.0	    else {
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
##edg2.0	}
	}
	unlink $logfn if $logfn;
	my $coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
	my $rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step"), "col");
	    } else {
		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
	    }
	}
	$colM .= makeX($rowM, "row");
    }

my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}


## Type with 1 column only
sub buildXMLFile1Col {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
    my $brf_info = "Rather Simple Storm";
    my $Jdl = `cat $self->{jdlFile}`;
    my $Scr = `cat  $self->{shFile}`;
    my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile1Col() to write $self->{xmlfile} \n";

    my $cols = {};
    my $order = 0;
#    my $machType = 'mawn';
    my $machType = 'mace';
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
        $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
        if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++; }
	my $mac = " ";
	if ( $self->{reqs}->{$index}->{mawn} ) { $mac = $self->{reqs}->{$index}->{mawn}; }
	else { $mac = "CE:".$self->{reqs}->{$index}->{mace}; }
	push @lof_rows, $mac;
    }

#prev	 foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#prev	     print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#prev	     push @lof_cols, $col;
#prev	 }
    push @lof_cols, "Status";
    
    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
	my $logfn;
	my $logf;
	if ( $retv == 0 ) {
	    $logfn = $self->{reqs}->{$index}->{olout};
	    $logf = `cat $logfn` if $logfn;
	} elsif ( $retv == 5 ) {  ## UNTESTED
##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	} else {
##edg2.0            if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
##edg2.0            else {
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
##edg2.0	}
	}
	unlink $logfn if $logfn;
	my $coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
	$coln = 0;
	my $rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step"), "col");
	    } else {
		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
	    }
	}
	$colM .= makeX($rowM, "row");
    }

my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}


# sub makeX {
#   my $str  = shift;
#   my $flag = shift;
#   return "<$flag>\n$str\n</$flag>\n";
# }


sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
