#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::CopyStorm;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

CopyStorm.pm - Achieves a GridFTP Storm from the WNs.
# $Id: CopyStorm.pm,v 1.8 2003/09/11 15:12:18 grodid Exp $

=head1 SYNOPSIS

 The CopyStorm is a regular JobStorm where the jobs submitted for execution on 
the WNs are achieving GridFTP transfers between a pair of machines choosen 
randomly among the CEs and the SEs of the site.

 A regular JobStorm is the following: 
a batch of 100 (maxSubs) identical jobs is sent 
asynchronously to the RB, with a small Input and Output SandBox. 

 Asynchronously means that one is NOT waiting for the previous request
to be over before sending the next one, but:
- a given fixed delay is required between 2 submissions (25 sec. in this case);
- NO more than 25 jobs (maxStack parameter) can be 
visible simultaneously from the RB, whatever their state is;
- when one job is achieved (DONE or ABORTED or CANCELLED), one more is added to 
the stack;
- in this case, NO resubmission can occur when the job is aborted.

 A job is considered successful when the all output files can be properly 
retrieved and parsed to check their contents.

=head1 DESCRIPTION 

B<CopyStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '5');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '169');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::CopyStorm [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "copyStorm";
	$self->{brfinfo} = "GridFTP Transfers";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash

    return 0;

}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

    print "CopyStorm: addArguments\n";

    my @hosts = @{$self->{SEs}};
    push @hosts, ${$self->{SEs}}[0];
    my $ih = int(rand(scalar(@hosts)-2));

#    my $bid = `echo 'Arguments = "@{$self->{SEs}}"; ' >> $jdlTemp`;
    my $bid = `echo 'Arguments = "$hosts[$ih] $hosts[$ih+1]"; ' >> $jdlTemp`;
    
    return 0;
}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

    print "CopyStorm: setupTest\n";

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

    print "CopyStorm: createJDL\n";
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
##Arguments = "a 2 c";
##nomore InputSandbox = {"/u/eb/grodid/public/globus/grodid/edg-test/script.sh"};
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
##edg2.0 # OutputSandbox = {"result.out","result.err","mumbojombo"};
##OutputSE = "bbr-gate02.slac.stanford.edu";
##bad Rank = other.MaxCpuTime;
##nomore Requirements  = Member ( other.RunTimeEnvironment , "BABAR-Test-slac-01");
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

    print "CopyStorm: createScript\n";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl -w

#     my ($a, $b, $c) = @ARGV;
#     print STDERR "GridFTP: runTest ARGS: $a $b $c\n";
    my (@hosts) = @ARGV;
    print STDERR "GridFTP: runTest ARGS: @hosts\n";
    my $error;

    # Loop over each argument doing a full test for each.
    @dirs = ("/tmp", "~");
#    @dirs = ("/tmp");
#good    @hosts = qw/lxshare0303.cern.ch lxshare0232.cern.ch/;
    foreach $dir (@dirs) {
        print "Checking transfers to $dir\n";

	$error = testGridFTP($dir, \@hosts);
#test	last; ##temp
    }

my @res = `host $ENV{HOSTNAME}; host $ENV{HOST}; uname -a`;
print STDOUT @res;

    exit 0;


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testGridFTP {

    my ($prefix, $arrayref) = @_;
    my @hosts = @{$arrayref};

    # Setup some constants.
    $ok = "[OK]";
    $fail = "[FAIL]";

    # Create a file name which is always in the temp area for the 
    # local machine as well as one in the location specified by
    # the prefix.
    $local = "/tmp/$ENV{USER}-$$";

    # Choose a host and check connections to all of the other hosts.
    foreach $host1 (@hosts) {
	
	# Fill up the file with a random number.
	$value = fillFile($local);
	print "\nUsing prefix/value: $prefix, $value\n";
	
	$remote = "$ENV{USER}-$$" . "_local-$host1";
	$check = "$local--$remote";
	
	# Check that transfers from local machine to remote machine works.
	if ( $ENV{MAINSCRIPT} ) { print "Bheck local --> $host1:  \n"; }
	$result1 = doTransfer("file://$local","gsiftp://$host1/$prefix/$remote");
	$result2 = doTransfer("gsiftp://$host1/$prefix/$remote","file://$check");
	$result3 = checkResult($value,$check);
	print "Check local --> $host1:  ";
	print "$result1 $result2 $result3\n";
	
	# These are then third party transfers.
	foreach $host2 (@hosts) {
##test	    next;  ##temp
	    if ("$host1" ne "$host2") {
		if ( $ENV{MAINSCRIPT} ) { print "Bheck $host1 --> $host2:  \n"; }
		$third = "$ENV{USER}-$$" . "_$host1-$host2";
		my $check = "$local--$third";
		$result1 = doTransfer("gsiftp://$host1/$prefix/$remote","gsiftp://$host2/$prefix/$third");
		$result2 = doTransfer("gsiftp://$host2/$prefix/$third","file://$check");
		$result3 = checkResult($value,$check);
		print "Check $host1 --> $host2:  ";
		print "$result1 $result2 $result3\n";
		my $rmes = `edg-gridftp-rm "gsiftp://$host2/$prefix/$third" 2>&1 `;
		my $lmes = `rm $check 2>&1 `;
		if ( "$rmes"."$lmes" ne "" ) {
		    print " Removing failed on $host2/local RM: :$third:$check: -$rmes-$lmes- \n";
		}
	    }
	}
	print "\n";
	my $rmes = `edg-gridftp-rm "gsiftp://$host1/$prefix/$remote" 2>&1 `;
	my $lmes = `rm $local $check 2>&1 `;
	if ( "$rmes"."$lmes" ne "" ) {
	    print " Removing failed on $host1/local RM: :$remote:$check: -$rmes-$lmes- \n";
	}
    }
}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransfer {

    my ($src, $dest) = @_;
    my $output = `globus-url-copy $src $dest 2>&1 `;
    my $flag = ($!) ? $ok : $fail;
##good    if ( $output ne "" ) {
##fake    $output = "FAKE \n";
    if ( $output ne "" ) {
	print " doTransfer $src TO $dest FAILED ==> \n$output";
    }
    return ($flag);
}

# Fill the given file with a new random number.
sub fillFile {
    my $fname = shift;
    my $value = rand();
    open RAN, ">$fname";
    print RAN "$value\n";
    close RAN;
    return ($value);
}


sub checkResult {
    my ($value, $fname) = @_;

    $val = `cat $fname`;
    chomp($val);
    $result = ("$value" eq "$val") ? $ok : $fail;
    if ( "$result" ne "$ok" ) {
	print " checkResult $value DIDN'T COMPARE TO $val \n";
    }
    return ($result);
}

END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);

#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
#     }

    return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
