#!/usr/bin/perl

package BaseTest::GatekeeperFileLimits;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

EDGUtils.pm - A set of utilities for EDG site certification tests

=head1 SYNOPSIS

EDGUtils.pm 

=head1 DESCRIPTION 

B<EDGUtils.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GatekeeperFileLimits: setContext\n";

    # Commands necessary to run this test. 
    $self->{commands} = ["globus-job-run", 
			 "grid-proxy-info"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test GatekeeperFileLimits [options] <gatekeeper> [gatekeeper...]

END
    ;

    $self->{DescText} = <<'END'

Check that the file limits on the gatekeeper node are large 
enough to accomodate a heavy load.  Note this test only works
for linux nodes. 

The values can be obtained by cat\'ting these files:
/proc/sys/fs/file-max
/proc/sys/fs/inode-max
The minimums are 16384 and 65536 respectively.

The inode-max value does not exist in the 2.4-series kernel because
the inode allocation is done dynamically.  If this file does not
exist, that part of the check is skipped. 

END
    ;
}

sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GatekeeperFileLimits: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    print "GatekeeperFileLimits: runTest\n";

    # Keep a list of all of the resources which fail.
    my @failed;

    # Loop over each argument doing a full test for each.
    my $host;
    foreach $host (@{$self->{args}}) {
        print "Checking... $host\n";

	my $error = testFileLimits($host);
	push @failed, $host if ($error);
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(", ",@failed);
        $self->setErrorMessage("file limits test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper.  Note that the fork job
## manager should be used for this test. 
##

sub testFileLimits {

    my $gk = shift;

    # Add jobmanager to end of hostname if not provided.
    $gk = "$gk/jobmanager" unless ($gk =~ /.*jobmanager.*/);

    # Setup some constants.
    my $filemin = 16384;
    my $inodemin = 65536;

    print "Obtaining file limits for $gk\n";
    my $filemax = readLimit($gk,'/proc/sys/fs/file-max');
    my $inodemax = readLimit($gk,'/proc/sys/fs/inode-max');

    my $fstr = $filemax;
    $fstr = 'FAIL' . "($filemax)" if ($filemax < $filemin);
    print "File max ($filemin):  $fstr\n";

    my $istr = $inodemax;
    $istr = 'OK' if ($inodemax == -1);
    $istr = 'FAIL' if ($inodemax == -2);
    print "Inode max ($inodemin): $istr\n";

    return 1 if ($filemax < 0);
    return 1 if ($filemax < $filemin);
    return 1 if ($inodemax == -2);
    return 1 if (($inodemax < $inodemin) && ($inodemax >= 0));
    return 0;
}


# Get the inode-max parameter from the file system.
# Returns -1 if the file doesn't exist, -2 if format of file is
# incorrect; otherwise value in file. 
sub readLimit {

    # Get gatekeeper and file to check.
    my ($gk, $file) = @_;

    # Check first that the file exists.  Return -1 if it does not exist. 
    my $command = 
	"globus-job-run $gk /bin/sh -c \"/usr/bin/test ! -e $file; echo \\\$?\"";
    my $exists = `$command`;
    chomp($exists);
    return -1 unless ($exists);

    # Get the value from the file. 
    $command = "globus-job-run $gk /bin/cat $file";
    my $out = `$command`;
    chomp($out);
    return -2 unless ($out =~ m/[^\d]*(\d+).*/m);
    return $1;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
