#!/usr/bin/perl

package BaseTest::GlobusGatekeeper;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

GlobusGatekeeper.pm 

=head1 SYNOPSIS

GlobusGatekeeper.pm - Test that the globus gatekeeper works.
# $Id: GlobusGatekeeper.pm,v 1.16 2003/10/06 15:33:04 loomis Exp $

=head1 DESCRIPTION 

B<GlobusGatekeeper.pm> This test checks that a globus-job-run, a
globus-job-submit, and a globusrun with two-phase commit all work.
The two-phase commit test is extremely important as edg-job-submits
will fail unless this works correctly.  (Note this involves an (as
yet) unidentified port on the Gatekeeper.)


=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GlobusGatekeeper: setContext\n";

    # Commands required to run this test.
    $self->{commands} = ["globus-job-run", 
			 "globus-job-submit", 
			 "globus-job-status",
			 "globus-job-get-output", 
			 "globus-job-clean", 
			 "grid-proxy-info"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test GlobusGatekeeper [options] <resource> [resource...]

END
    ;

    $self->{DescText} = <<'END'

Check that a globus gatekeeper is minimally functioning.  For
the two-phase test to be useful, this should be run from a
machine outside of the gatekeeper\'s local firewall.

END
    ;
}

sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GlobusGatekeeper: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub cleanup {
    my $self = shift;

    print "GlobusGatekeeper: cleanup\n";

    unlink "/tmp/$$";

    my $rc = $self->SUPER::cleanup();
    return $rc;
}


sub runTest {
    my $self = shift;

## gg
    my $saveOut;
    if ( $ENV{MAINSCRIPT} ) {
	my $base = "/tmp/$ENV{USER}-$$";
#	$self->{conffile} = "$base-GridFTPconf";
	$self->{outfile}  = "$base-GatekeeperOut";

# 	$self->{xmlfile}  = "$base-GatekeeperXML";
# 	$self->{xmlxist}  = __PACKAGE__;
#	$self->{xmlxist}  = "print";

	$self->{reqs} = {};    ## Hash
	$self->{steps} = {};    ## Hash
	$self->{mach} = {};    ## Hash
	$self->{case} = {};    ## Hash
##	$self->{title}  = __PACKAGE__;

	my $package = __PACKAGE__;
	my @m = split /::/, $package;
	$self->{type} = "$m[$#m]";
	$self->{xmlfile}  = "$base-".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
#old	$html_dir = "${html_dir}/cal_gatekeeper";
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";

#	$saveOut = switchStdouterr("$self->{outfile}");
    }

    print "GlobusGatekeeper: runTest\n";

    # Keep a list of all of the resources which fail.
    my @failed;

    # Loop over each argument doing a full test for each.
    foreach my $host (@{$self->{args}}) {
        print "Checking... $host\n";

	my $error = $self->testGatekeeper($host);
	push @failed, $host if ($error);
    }

## gg
#     if ( $ENV{MAINSCRIPT} ) {
# 	restoreStdouterr($saveOut);
#     }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("gatekeeper test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testGatekeeper {

    my ($self, $gk) = @_;
    my $mace = $gk;

    # Add jobmanager to end of hostname if not provided.
    $gk = "$gk/jobmanager" unless ($gk =~ /.*jobmanager.*/);

    if (0) { ## test hook
	$self->fillResHash('[OK]', "message 0", $mace, "Trying globus-job-run...", 0, "sendRandom", "globus-job-run $gk /bin/echo") unless ( $mace =~ "408" );
	$self->fillResHash('[FAIL]', "message 1", $mace, "Trying globus-job-run with staging...", 1, "stageRandom", "globus-job-run $gk -stage");
	return 0 if ( $mace =~ "adc0010" );
	$self->fillResHash('[OK]', "message 2", $mace, "Trying globus-job-submit...", 2, "submitRandom", "globus-job-submit $gk /bin/echo") unless ( $mace =~ "adc0006" );
	$self->fillResHash('[OK]', "message 3", $mace, "Testing job status...", 3, "submitStatus", "globus-job-status jobid");
	return 0 if (1);
    }

    if (0) {			## 2D test hook
	foreach my $c ( 0..1 ) {
	    my $loop = "case$c";
	    next if ( $mace =~ "405" && $c == 0 );
	    next if ( $mace =~ "adc0006" && $c == 1 );
	    $self->fillResHash2D($loop, '[OK]', "message 0", $mace, "Trying globus-job-run...", 0, "sendRandom", "globus-job-run $gk /bin/echo") unless ( $mace =~ "408" );
	    $self->fillResHash2D($loop, '[FAIL]', "message 1", $mace, "Trying globus-job-run with staging...", 1, "stageRandom", "globus-job-run $gk -stage");
	    next if ( $mace =~ "adc0010" );
	    $self->fillResHash2D($loop, '[OK]', "message 2", $mace, "Trying globus-job-submit...", 2, "submitRandom", "globus-job-submit $gk /bin/echo") unless ( $mace =~ "adc0006" );
	    $self->fillResHash2D($loop, '[OK]', "message 3", $mace, "Testing job status...", 3, "submitStatus", "globus-job-status jobid");
	}
	return 0 if (1);
    }

    print "Trying globus authentication...";
    my ($result, $out) = tryAuthentication($gk);
    print $result . "\n";
#p    print STDERR " sendRandom: $out \n";
    $self->fillResHash($result, $out, $mace, "Trying globus authentication...", 0, "tryAuthentication", "globusrun -a -r $gk");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Trying globus-job-run...";
    my ($result, $out) = sendRandom($gk);
    print $result . "\n";
#p    print STDERR " sendRandom: $out \n";
    $self->fillResHash($result, $out, $mace, "Trying globus-job-run...", 1, "sendRandom", "globus-job-run $gk /bin/echo");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Trying globus-job-run with staging...";
    my ($result, $out) = stageRandom($gk);
    print $result . "\n";
    $self->fillResHash($result, $out, $mace, "Trying globus-job-run with staging...", 2, "stageRandom", "globus-job-run $gk -stage");
    return 1  if ("$result" eq '[FAIL]') ;

    return 0 if (0);

    print "Trying globus-job-submit...";
    my ($result,$r,$jobid, $out) = submitRandom($gk);
    print $result . "\n";
    $self->fillResHash($result, $out, $mace, "Trying globus-job-submit...", 3, "submitRandom", "globus-job-submit $gk /bin/echo $r");
    return 1  if ("$result" eq '[FAIL]') ;
    print "JobId: $jobid\n";
    print "Random: $r\n";

    print "Testing job status...\n";
    my ($result, $out) = submitStatus($jobid,100);
    print "Status...$result" . "\n";
    $self->fillResHash($result, $out, $mace, "Testing job status...", 4, "submitStatus", "globus-job-status $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Check job output...\n";
    my ($result, $out) = checkOutput($jobid,$r);
    print $result;
    $self->fillResHash($result, $out, $mace, "Check job output...", 5, "checkOutput", "globus-job-get-output $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Clean up job...";
    my ($result, $out) = clearJob($jobid);
    print $result . "\n";
    $self->fillResHash($result, $out, $mace, "Clean up job...", 6, "clearJob", "globus-job-clean -force $jobid");
    return 1  if ("$result" eq '[FAIL]');

    print "Trying globusrun (two_phase)...";
    my ($result,$r,$jobid,$out) = submitRandomGlobusrun($gk);
    print $result . "\n";
    $self->fillResHash($result, $out, $mace, "Trying globusrun (two_phase)...", 7, "submitRandomGlobusrun", "globusrun -r $gk -batch -file");
    return 1  if ("$result" eq '[FAIL]') ;
    print "JobId: $jobid\n";
    print "Random: $r\n";

    print "Testing job status...\n";
    my ($result, $out) = submitStatus($jobid,100);
    print "Status...$result\n";
    $self->fillResHash($result, $out, $mace, "Testing job status...", 8, "submitStatus", "globus-job-status $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Check job output...";
    my ($result, $out) = checkOutput($jobid,$r);
    print $result . "\n";
    $self->fillResHash($result, $out, $mace, "Check job output...", 9, "checkOutput", "globus-job-get-output $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Clean up job...";
    my ($result, $out) = clearJob($jobid);
    print $result . "\n";
    $self->fillResHash($result, $out, $mace, "Clean up job...", 10, "clearJob", "globus-job-clean -force $jobid");
    return 1  if ("$result" eq '[FAIL]');
}

# Send an authentication request.
sub tryAuthentication {

    my ($gk) = @_;

    my $out = `globusrun -a -r $gk`;
    return (($?) ? '[FAIL]' : '[OK]', $out);
#    return ($out =~ m/.*$r.*/m) ? '[OK]' : $out;
}


# Generate a random number, send it to the gatekeeper, and ensure that
# the number is present in the output.
sub sendRandom {

    my ($gk) = @_;

    my $r = rand();
    my $out = `globus-job-run $gk /bin/echo $r 2>&1`;
    return (($out =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $out);
#    return ($out =~ m/.*$r.*/m) ? '[OK]' : $out;
}


# Generate a random number, send it to the gatekeeper, and ensure that
# the number is present in the output.
sub stageRandom {

    my ($gk) = @_;

    my $r = rand();

    # Create a temporary file to hold the script.
    my $fn = "/tmp/script-$$.sh";
    open TMP, ">$fn";
    print TMP << "END"
#!/bin/sh
/bin/echo $r
exit 0
END
    ;
    close TMP;
    chmod 0740, $fn;

    # Run the script.
    my $command = "globus-job-run $gk -stage $fn";
    my $out = `$command 2>&1`;

    # Clean up the script.
    unlink $fn;

    # Now return the result.
    return (($out =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $out);
#    return ($out =~ m/.*$r.*/m) ? '[OK]' : $out;
}


# This submits a job via globus.  It returns a list with a success
# flag, the random number which should be returned, and the job
# contact ID.
sub submitRandom {

    my ($gk) = @_;

    my $r = rand();
    my $out = `globus-job-submit $gk /bin/echo $r 2>&1`;
    chomp(my $jobid=$out);
    my $flag = ($!) ? '[OK]' : '[FAIL]';
    return ($flag, $r, $jobid, $out);
}

# This submits a job via globus using globusrun.  It returns a list
# with a success flag, the random number which should be returned, and
# the job contact ID.  
sub submitRandomGlobusrun {

    my ($gk) = @_;

    my $r = rand();

# Create the RSL file.
    open RSL, ">/tmp/$$";
    print RSL << "END"
  &(executable="/bin/echo")
   (arguments= "$r")
   (two_phase=300)
   (stdout=x-gass-cache://\$(GLOBUS_GRAM_JOB_CONTACT)stdout anExtraTag)
   (stderr=x-gass-cache://\$(GLOBUS_GRAM_JOB_CONTACT)stderr anExtraTag)
END
    ;
    close RSL;

    my $out = `globusrun -r $gk -batch -file /tmp/$$ 2>&1`;
    chomp(my $jobid=$out);
    $jobid =~ m/.*(https\S*)\s*/;
    $jobid = $1;
    my $flag = ($!) ? '[OK]' : '[FAIL]';

    return ($flag, $r, $jobid, $out);
}

# This waits in a loop until the timeout is reached or the "DONE"
# status is obtained.
sub submitStatus {

    my ($jobid, $limit) = @_;

    my $out;
    my $status = "INITIALIZE";
    CHECK: while ($limit>0) {
	sleep(5);
	$status = `globus-job-status $jobid 2>&1`;
	$out = $status;
	$status="SYSTEM ERROR" unless ($!);

	chomp($status);
	print "$status\n";

	last CHECK if ("$status" eq "DONE" || ("$status" eq "SYSTEM ERROR"));
	$limit--;
    }

    return (("$status" eq "DONE") ? '[OK]' : '[FAIL]', $out);
}


# Retrieve the job output and check that the random number was
# correctly echoed.
sub checkOutput {

    my ($jobid, $r) = @_;

    my $out = `globus-job-get-output $jobid 2>&1`;
    print "$out\n";
    return (($out =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $out);
}


# This submits a job via globus.  It returns a list with a success
# flag, the random number which should be returned, and the job
# contact ID.
sub clearJob {

#     if ( $ENV{MAINSCRIPT} ) {
# 	return ('[OK]', ""); ## TEMP gg
#     }

    my ($jobid) = @_;
    chomp($jobid);
    my $out = `globus-job-clean -force $jobid 2>&1`;
    return (($!) ? '[OK]' : '[FAIL]', $out);
}


sub buildXMLFileNOTVERYGOODandquiteold {

    my $self = shift;

    print "GlobusGatekeeper: buildXMLFile\n";

    print STDERR " Called buildXMLFile() to process $self->{outfile} \n";

##test    my $resBXML = `build-XML.pl $self->{outfile} $self->{xmlfile}`;
    my $resBXML = `build-XML-col-wise.pl $self->{outfile} $self->{xmlfile}`;
    print STDERR " After buildXMLFile: \n$resBXML \n";

    return 0;
}


sub buildXMLFile {
    my $self = shift;
    my $ret = $self->buildXMLFile1D();
    return $ret;
}


## Type with 1 row only, several steps and several columns 
sub buildXMLFile1D {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows = ("");
#notbad    my $Title = (caller(0))[3];
#good    my $Title = __PACKAGE__;
    my $Title = $self->{type};
    chomp(my $date = `date`);
#     my $brf_info = "Rather Simple Storm";
#     my $Jdl = `cat $self->{jdlFile}`;
#     my $Scr = `cat  $self->{shFile}`;
#     my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
#     $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{reqs}} ) {
	push @lof_cols, $mace;
    }

    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
#p    print STDERR " Called buildXMLFile() to write @lof_cols \n";
    my $rowM = "";
    foreach my $mace ( sort keys %{$self->{reqs}} ) {
	my $stepM = "";
	for my $stepn (0..$nsteps) {
	    my $retv = $ret{$self->{reqs}->{$mace}->{$stepn}->{stat1}};
	    my $logf = $self->{reqs}->{$mace}->{$stepn}->{olout};
	    $stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	}
	$rowM .= makeX($stepM, "col");
    }
    my $colM = makeX($rowM, "row");

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}


sub fillResHash {
    return unless $ENV{MAINSCRIPT};
    my ($self, $stat1, $olout, $mace, $sname, $stepn, $sdesc, $scomm) = @_;
    $self->{globalresult} .= " $stat1" if $self->{globalresult};

    $self->{reqs}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = $scomm;

    return;

}


## Type with several rows, several steps and several columns 
sub buildXMLFile2Dold {  ##  See EDGUtils from now on
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}


sub fillResHash2D {
    return unless $ENV{MAINSCRIPT};
    my ($self, $case, $stat1, $olout, $mace, $sname, $stepn, $sdesc, $scomm) = @_;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = $scomm;
    $self->{mach}->{$mace}++;
    $self->{case}->{$case}++;
    return;

}


# sub makeX {
#   my $str  = shift;
#   my $flag = shift;
#   return "<$flag>\n$str\n</$flag>\n";
# }

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
