#!/usr/bin/perl

package BaseTest::GridFTP;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

GridFTP.pm 

=head1 SYNOPSIS

GridFTP.pm - Verify that transfers to and between GridFTP servers works.
# $Id: GridFTP.pm,v 1.13 2003/10/01 12:22:20 grodid Exp $

=head1 DESCRIPTION 

B<GridFTP.pm> This test checks that transfers between the user
interface (UI) machine and all of the given hosts work.  It checks
both absolute URLs and URLs relative to the home area of the user.  

Additionally, third party transfers between all pairs of hosts are
checked.  The number of checks performed goes as n^2; so be careful on
the number of hosts specified.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GridFTP: setContext\n";

    # The command needed to run this test.
    $self->{commands} = ["globus-url-copy", 
			 "grid-proxy-info",
			 "edg-gridftp-rm"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::GridFTP [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'

Check that a GridFTP server works by transfering files to and from the
local machine.  Also checks third-party transfers by transfering files
between each pair of remote servers.

For local->machine the three checks are:
 Step  1) local->machine
 Step  2) machine->local
 Step  3) check that contents are correct

For machine1->machine2 the three checks are:
 Step  1) machine1->machine2
 Step  2) machine2->local
 Step  3) check that contents are correct

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GridFTP: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    my $saveOut;
    if ( $ENV{MAINSCRIPT} ) {
	my $base = "/tmp/$ENV{USER}-$$";
	$self->{conffile} = "$base-GridFTPconf";
	$self->{outfile}  = "$base-GridFTPout";
	$self->{stepfile} = "$base-GridFTPstep";

# 	$self->{xmlfile}  = "$base-GridFTP.XML";
# 	$self->{xmlxist}  = __PACKAGE__;
# 	$self->{htmldim}  = "2D";

	$self->{reqs} = {};    ## Hash
	$self->{steps} = {};    ## Hash
	$self->{mach} = {};    ## Hash
	$self->{case} = {};    ## Hash
#	$self->{debug}  = 1;

	my $package = __PACKAGE__;
	my @m = split /::/, $package;
	$self->{type} = "$m[$#m]";
	$self->{xmlfile}  = "$base-".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
#old	$self->{htmldir} = "$html_dir/cal_gridftp";;
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";

#nomore	$saveOut = switchStdouterr("$self->{outfile}");
    }

    print "GridFTP: runTest\n";

    # Keep a list of all directories which fail.
    my @failed;

    # Loop over each argument doing a full test for each.
    foreach my $dir (("/tmp", "~")) {
        print "Checking transfers to $dir\n";

	my $error = $self->testGridFTP($dir, \@{$self->{args}});
	push @failed, $dir if ($error);
    }

    if ( $ENV{MAINSCRIPT} ) {
#nomore	restoreStdouterr($saveOut);
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("GridFTP test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testGridFTP {

    my ($self, $prefix, $arrayref) = @_;
    my @hosts = @{$arrayref};

    if ( "$prefix" eq "/tmp" ) { $self->{store}->{stpoff} = -1; }
    else { $self->{store}->{stpoff} = 2; }

    # Create a file name which is always in the temp area for the 
    # local machine as well as one in the location specified by
    # the prefix.
    my $local = "/tmp/$ENV{USER}-$$";

    # Choose a host and check connections to all of the other hosts.
    foreach my $host1 (@hosts) {
	
	# Fill up the file with a random number.
	my $value = fillFile($local);
	print "\nUsing prefix/value: $prefix, $value\n";
	
	my $remote = "$ENV{USER}-$$" . "_local-$host1";
	my $check = "$local--$remote";
	
	# Check that transfers from local machine to remote machine works.
	if ( $ENV{MAINSCRIPT} ) { 
	    $self->{store}->{label} = "Check local --> host1"; 
	    $self->{store}->{src} = "local";
	    $self->{store}->{tgt} = "$host1";
	    $self->{store}->{stepn} = 0;
	}
	my $result1 = $self->doTransfer("file://$local",
				 "gsiftp://$host1/$prefix/$remote");
	my $result2 = $self->doTransfer("gsiftp://$host1/$prefix/$remote",
				 "file://$check");
	my $result3 = $self->checkResult($value,$check);

	print "Check local --> $host1:  ";
	print "$result1 $result2 $result3\n";
	$self->{globalresult} .= " $result1 $result2 $result3" if $self->{globalresult};

	# Clean up the check file.
	unlink $check;
	
	# These are then third party transfers.
	foreach my $host2 (@hosts) {
	    if ("$host1" ne "$host2") {
		if ( $ENV{MAINSCRIPT} ) { 
		    $self->{store}->{label} = "Check host1 --> host2"; 
		    $self->{store}->{src} = $host1;
		    $self->{store}->{tgt} = $host2;
		    $self->{store}->{stepn} = 0;
		}
		my $third = "$ENV{USER}-$$" . "_$host1-$host2";
		my $check = "$local--$third";
		$result1 = $self->doTransfer("gsiftp://$host1/$prefix/$remote",
				      "gsiftp://$host2/$prefix/$third");
		$result2 = $self->doTransfer("gsiftp://$host2/$prefix/$third",
				      "file://$check");
		$result3 = $self->checkResult($value,$check);
		print "Check $host1 --> $host2:  ";
		print "$result1 $result2 $result3\n";
		$self->{globalresult} .= " $result1 $result2 $result3" if $self->{globalresult};

		# Delete files related to the "third" host. 
		unlink $check;
                my $rmes = `edg-gridftp-rm gsiftp://$host2/$prefix/$third 2>&1 `;
		if ($rmes) {
		    print "Removed failed: gsiftp://$host2/$prefix/$third.\n";
		    print "$rmes\n";
		}

	    }
	}

	# Clean up the first file that was transfered. 
	my $rmes = `edg-gridftp-rm gsiftp://$host1/$prefix/$remote 2>&1 `;
	if ($rmes) {
	    print "Removed failed: gsiftp://$host1/$prefix/$remote.\n";
	    print "$rmes\n";
	}

	# Remove the local file and the check file. 
	unlink $local;
	unlink $check;

	print "\n";
    }

    if ( $self->{globalresult} =~ "[FAIL]" ) { return 1; }
    else { return 0; }

}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransfer {

    my ($self, $src, $dest) = @_;
    $self->{store}->{stepn}++;
    my $output = `globus-url-copy $src $dest 2>&1`;
    my $flag = ($!) ? '[OK]' : '[FAIL]';
    if ($output ne "") {
        $output = "doTransfer $src TO $dest FAILED ==> \n$output\n";
	print STDOUT $output;
    }
    my $stepn = $self->{store}->{stepn} + $self->{store}->{stpoff};
    $self->fillResHash2D($stepn, $flag, $output, "doTransfer", "globus-url-copy $src $dest");
    return ($flag);
}

# Fill the given file with a new random number.
sub fillFile {
    my $fname = shift;
    my $value = rand();
    open RAN, ">$fname";
    print RAN "$value\n";
    close RAN;
    return $value;
}

sub checkResult {
    my ($self, $value, $fname) = @_;
    $self->{store}->{stepn}++;
    my $val = `cat $fname`;
    chomp($val);
    my $result = ("$value" eq "$val") ? '[OK]' : '[FAIL]';
    my $output = "";
    if ( "$result" ne "[OK]" ) {
	$output = " checkResult $value DIDN'T COMPARE TO $val \n";
	print STDOUT $output;
    }
    my $stepn = $self->{store}->{stepn} + $self->{store}->{stpoff};
    $self->fillResHash2D($stepn, $result, $output, "checkResult", "compare");
    return ($result);
}

sub fillResHash2D {
    return unless $ENV{MAINSCRIPT};
    my ($self, $stepn, $stat1, $olout, $sdesc, $scomm) = @_;
    my $case = $self->{store}->{src};
    my $mace = $self->{store}->{tgt};
    my $sname = $self->{store}->{label};
    my @m = split /\s+/, $sname;
#     if ( "$scomm" eq "globus-url-copy" ) {
# 	if ( $stepn%3 == 0 ) { $scomm .= " $m[1] $m[3]"; }
# 	else { $scomm .= " $m[3] $m[1]"; }
#     }
    if ( $stepn < 3 ) { $sname = "Steps 0 to 2 are for Transfers to/from \"/tmp\" directory on remote node"; }
    else { $sname = "Steps 3 to 5 are for Transfers to/from USER's HOME directory on remote node"; }
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = "Ex: $scomm";
    $self->{mach}->{$mace}++;
    $self->{case}->{$case}++;
#p    print " fillResHash2D: $stepn $case $mace \n";
    return;
}

sub buildXMLFile {
    my $self = shift;
#p    $self->printStack(3);
    my $ret = $self->buildXMLFile2D();
    return $ret;
}

## Type with several rows, several steps and several columns 
sub buildXMLFile2Dold {  ##  See EDGUtils from now on
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
