#!/usr/bin/perl

package BaseTest::GridFTPClient;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

GridFTPClient.pm 

=head1 SYNOPSIS

GridFTPClient.pm - Check that GridFTP client commands work correctly.

=head1 DESCRIPTION 

B<GridFTPClient.pm> This checks that the basic GridFTP client commands
(edg-gridftp-*) commands work on the server.  This includes renaming,
listing, etc. of the files in the server.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GridFTPClient: setContext\n";

    # The command needed to run this test.
    $self->{commands} = ["globus-url-copy", 
			 "grid-proxy-info",
			 "edg-gridftp-exists",
                         "edg-gridftp-ls",
                         "edg-gridftp-mkdir",
                         "edg-gridftp-rename",
                         "edg-gridftp-rm",
                         "edg-gridftp-rmdir"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::GridFTPClient [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'

Check that the edg-gridftp client commands work on the given server(s).

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GridFTPClient: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    print "GridFTPClient: runTest\n";

    # Keep a list of all directories which fail.
    my @failed;

    # Loop over each argument doing a full test for each.
    my $server;
    foreach $server (@{$self->{args}}) {
        my $error = testGridFTPClient($server);
        push @failed, $server if ($error);
    }

    # If there are any servers in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("GridFTPClient test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testGridFTPClient {

    my ($server) = @_;

    my $rc = 0;

    # Create a file name which is always in the temp area for the 
    # local machine as well as one in the location specified by
    # the prefix.
    my $rdir = "~/$ENV{USER}-$$";
    my $rfile = "$ENV{USER}-file-$$";
    my $lfile = "/tmp/$rfile";

    # Fill up the file with a random number.
    my $value = fillFile($lfile);

    print "Checking $server...\n";

    # Create the remote directory.
    my $result = `edg-gridftp-mkdir gsiftp://$server/$rdir`;
    if ($?) {
        print STDERR "Can't make remote directory ($server/$rdir).\n";
	$rc = 1;
        return $rc;
    }

    # Verify that this exists. 
    $result = `edg-gridftp-exists gsiftp://$server/$rdir`;
    if ($?) {
	$rc = 1;
        print STDERR "Verify of directory ($rdir) failed.\n";
	return $rc;
    }

    # Check that transfers from local machine to remote machine work.
    $result = doTransfer("file://$lfile",
                             "gsiftp://$server/$rdir/$rfile");
    if ("$result" ne "[OK]") {
        print STDERR "Transfer of file to server failed.\n";
	$rc = 1;
	return $rc;
    }

    # Verify that this exists. 
    $result = `edg-gridftp-exists gsiftp://$server/$rdir/$rfile`;
    if ($?) {
	$rc = 1;
	print STDERR "Remote file ($rfile) doesn't exist.\n";
    }

    # Rename the file.
    $result = `edg-gridftp-rename gsiftp://$server/$rdir/$rfile gsiftp://$server/$rdir/$rfile-moved `;
    if ($?) {
	$rc = 1;
	print STDERR "Rename of remote file failed.\n";
    }

    # Verify that this exists. 
    $result = `edg-gridftp-exists gsiftp://$server/$rdir/$rfile 2>/dev/null`;
    if (!($?)) {
	$rc = 1;
	print STDERR "Rename failed; old file exists.\n";
    }
    $result = `edg-gridftp-exists gsiftp://$server/$rdir/$rfile-moved`;
    if ($?) {
	$rc = 1;
	print STDERR "Rename failed; new file doesn't exist.\n";
    }

    # Remove the local file.  
    $result = `edg-gridftp-rm gsiftp://$server/$rdir/$rfile-moved`;
    if ($?) {
	$rc = 1;
	print STDERR "Removal of moved file ($rdir/$rfile-moved) failed.\n";
    }

    # Verify that this doesn't exist. 
    $result = `edg-gridftp-exists gsiftp://$server/$rdir/$rfile-moved 2>/dev/null`;
    if (!($?)) {
	$rc = 1;
	print STDERR "Moved file exists after removal.\n";
    }

    # Now remove the directory.
    $result = `edg-gridftp-rmdir gsiftp://$server/$rdir`;
    if ($?) {
	$rc = 1;
	print STDERR "Removing directory (gsiftp://$server/$rdir) failed.\n";
    } 

    # Verify that directory is now gone. 
    $result = `edg-gridftp-exists gsiftp://$server/$rdir 2>/dev/null`;
    if (!($?)) {
	$rc = 1;
	print "Removed directory (gsiftp://$server/$rdir) still exists.\n";
    }
    
    # Clean up the check file.
    unlink $lfile;

    return $rc;
}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransfer {

    my ($src, $dest) = @_;
    my $output = `globus-url-copy $src $dest 2>&1`;
    my $flag = ($!) ? '[OK]' : '[FAIL]';
    if ($output ne "") {
        print "doTransfer $src TO $dest ==> \n$output\n";
    }
    return ($flag);
}

# Fill the given file with a new random number.
sub fillFile {
    my $fname = shift;
    my $value = rand();
    open RAN, ">$fname";
    print RAN "$value\n";
    close RAN;
    return $value;
}


sub checkResult {
    my ($value, $fname) = @_;

    my $val = `cat $fname`;
    chomp($val);
    return ("$value" eq "$val") ? '[OK]' : '[FAIL]';
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
