#!/usr/bin/perl

package BaseTest::GridFTPUmask;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

GridFTPUmask.pm 

=head1 SYNOPSIS

GridFTPUmask.pm - Checks that umask is set for group access on GridFTP
server.

=head1 DESCRIPTION 

B<GridFTPUmask.pm> This checks that the umask is set correctly to
allow group write privilege by default on files transferred to the
server.  This is necessary with pooled accounts otherwise users may
lose control of files they have written when accounts are recycled.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GridFTP: setContext\n";

    # Required commands for this test.
    $self->{commands} = ["globus-url-copy",
			 "edg-gridftp-ls",
			 "edg-gridftp-rm",
			 "edg-gridftp-exists",
			 "grid-proxy-info"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::GridFTPUmask [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'

For normal operation of a GridFTP server as a storage element and with
pool accounts, the Umask of the server must be set to allow at least
group read and write.  This checks that the GridFTP server is
configured to do so. 

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GridFTPUmask: checkPrerequisites\n";

       $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;

}


sub runTest {
    my $self = shift;

    print "GridFTPUmask: runTest\n";

    # Keep a list of all directories which fail.
    my @failed = ();

    # Loop over each argument doing a full test for each.
    foreach my $host (@{$self->{args}}) {
        print "Checking Umask to $host\n";

	my $error = testUmask($host);
	push @failed, $host if ($error);
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("Umask test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testUmask {

    my ($host) = @_;

    # Setup some constants.
    my $ok = "[OK]";
    my $fail = "[FAIL]";

    # Create the temporary file to transfer.  Ensure that the
    # permissions are correct.
    my $local = "/tmp/$$";
    open LOCAL, ">$local";
    close LOCAL;
    chmod 0400, $local;

    # Transfer the file to the remote machine. 
    my $result = doTransfer("file://$local","gsiftp://$host/$local");
    print "Transfer to $host $result\n";

    # Do a verbose listing to get the permissions.
    my $output = `edg-gridftp-ls --verbose gsiftp://$host/$local`;
    $output = '' if ($?);

    # Parse output and check that the read and write bits are set for
    # the group. 
    $output =~ /^([rwxd-]{10})(.*)/;
    my $perms = $1;
    my $permsok = ($output =~ /^([rwxd-]{4}rw[rwxd-]{4})\s+(.*)/s) ? 0 : 1; 

    print "Permissions for $host ($perms)  ";
    print ($permsok ? $fail : $ok);
    print "\n";

    # Delete the files.
    $output = `edg-gridftp-rm gsiftp://$host/$local`;
    unlink $local;

    return $permsok;
}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransfer {

    my ($src, $dest) = @_;
    my $output = `globus-url-copy $src $dest`;
    my $flag = ($!) ? '[OK]' : '[FAIL]';
    return ($flag);
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
