#!/usr/bin/perl

package BaseTest::LongNameRegister;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

LongNameRegister.pm 

=head1 SYNOPSIS

LongNameRegister.pm - Register large number of long file names with RM. 
# $Id: LongNameRegister.pm,v 1.6 2003/09/05 14:13:06 loomis Exp $

=head1 DESCRIPTION 

B<LongNameRegister.pm> This test checks that a large number of files
    with long file names can be registered via the replica manager.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "LongNameRegister: setContext\n";

    # The number of copies to register. 
    $self->addOption('lfnsize=i',
		     'size of lfn in characters',
		     '80');

    # The number of copies to register. 
    $self->addOption('copies=i',
		     'number of copies to register',
		     '1');

    # Allow the virtual organization to be specified.  Needed for job
    # submissions involving data requirements.  
    $self->addOption('vo=s',
		     'Virtual Organization (VO) name');

    # The command needed to run this test.
    $self->{commands} = ["grid-proxy-info",
                         "edg-rm"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::LongNameRegister [options] <Storage Element> 

END
    ;

    $self->{DescText} = <<'END'

Check that a large number of files with long file names can be
registered via the replica manager. 

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "LongNameRegister: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    # The virtual organization must exist. 
    if (! ($self->{opts}->{vo})) {
	$rc = 1;
	$self->setErrorMessage('Virtual Organization is required option');
    }

    # Ensure a reasonable value for copies. 
    $self->{opts}->{copies} = 1 unless ($self->{opts}->{copies} > 0);

    return $rc;
}


sub runTest {
    my $self = shift;

    print "LongNameRegister: runTest\n";

    $self->testRegister();

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testRegister {

    my $self = shift;

    # Make a temporary local file. 
    my $lurl = makeLocalURL();

    # Make the required number of copies.
    print "\nMaking copies to SE...\n";
    my $aref = makeCopies($self->{opts}->{vo}, 
			  $lurl,
			  $self->{opts}->{copies}, 
			  'grid02.lal.in2p3.fr', 
			  '/flatfiles/SE00/wpsix');

    print "\nRegistering copies...\n";
    my $aref = registerCopies($self->{opts}->{vo},
			      $self->{opts}->{lfnsize},
			      $aref);

    print "\nCleaning up all copies...\n";
    my $aref = removeCopies($self->{opts}->{vo},$aref);
}

# Make a local file with a random number in it and pass back a URL.
sub makeLocalURL {

    # Create a local file to transfer (many times) to storage element.
    my $local = "/tmp/$ENV{USER}-$$";
    
    # Fill up the file with a random number.
    my $value = fillFile($local);

    # pass back the URL for the temporary file
    return "file://$local";
}

# This makes a random prefix for the files with the given number of
# characters.  
sub makePrefix {

    my ($n) = @_;
    $n = 1 if ($n<=0);

    my @okchars = ('A' .. 'Z', 'a' .. 'z', '0' .. '9');

    my $prefix = '';
    for (1 .. $n) {
	my $r = int(rand(($#okchars+1)));
	$prefix .= $okchars[$r];
    }
    
    return $prefix;
}

# Make the given number of copies of the local file onto the given
# SE. 
sub makeCopies {

    my ($vo, $lurl, $n, $sehost, $prefix) = @_;
    $n = 1 if ($n <= 0);

    my @urls;
    foreach (1 .. $n) {
	my $guid = `edg-rm --vo $vo cr $lurl -d $sehost`;
	chomp $guid;
	my $srm = `edg-rm --vo $vo lr $guid`;
	chomp $srm;
	`edg-rm --vo $vo uf $guid $srm`;
	my $url = $srm;
	$url =~ s%$sehost%$sehost$prefix%;
	$url =~ s/srm:/gsiftp:/;
	`edg-gridftp-exists $url`;
	push @urls, $srm unless ($?);
    }
    return \@urls;
}


# Register all copies in the replica manager.
sub registerCopies {

    my ($vo, $lfnsize, $aref) = @_;

    # Prepare the LFN names.
    my %lfns;
    foreach (@$aref) {
	my $lfn = 'lfn:' . makePrefix($lfnsize);
	$lfns{$_} = $lfn;
    }

    print localtime() . "\n";
    my $start = time();

    my @guids;
    foreach (keys %lfns) {
	my $output = `edg-rm --insecure --vo $vo rf -l $lfns{$_} $_`;
	chomp($output);
	my $guid = $output;
	print "GUID/RESULT: $guid/$?\n";
	push @guids, $guid unless ($?);
    }

    my $end = time();
    print localtime() . "\n";
    my $seconds = $end-$start;
    print "\n\nELAPSED TIME: $seconds\n\n";

    return \@guids;
}


# Remove the copies from all locations. 
sub removeCopies {

    my ($vo,$aref) = @_;

    foreach (@$aref) {
	my $output = `edg-rm --insecure --vo $vo del --all $_`;
	print "Removing $_ failed.\n" if ($?);
    }
}


# Fill the given file with a new random number.
sub fillFile {
    my $fname = shift;
    my $value = rand();
    open RAN, ">$fname";
    print RAN "$value\n";
    close RAN;
    return $value;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
