#!/usr/bin/perl

package BaseTest::RM;

use EDGUtils;

require BaseStep; ## NB I only add BaseStep here for the getSelf method invocation from TestCycle! Add getSelf to BaseTest and remove this!
@ISA=qw(BaseTest BaseStep); ## IDEM

=head1 NAME

RM.pm
# $Id: RM.pm,v 1.9 2003/09/19 15:21:06 grodid Exp $

=head1 SYNOPSIS

RM.pm - BaseTest derived class containing common stuff for any
        Replica Manager tests.

setContext() - Common option flags for RM tests.
checkArguments() - Checks common arguments for RM tests.
checkPrerequisites() - Deals with common option flags, etc.

=head1 DESCRIPTION

RM.pm - BaseTest derived class containing common stuff for any
        Replica Manager tests.

setContext() - Common option flags for RM tests.
checkArguments() - Checks common arguments for RM tests.
checkPrerequisites() - Deals with common option flags, etc.

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;
  $self->SUPER::setContext();

  $self->printLocationUU("setContext");

  # Default value for savevars array ref: empty.
  $self->{SAVEVARS} = [];

  # Number of args to run RM tests: 2 Storage Elements
  $self->{NARGS} = 2;

  # Add the filesize option
  $self->addOption('filesize=i',
		   'Size in kB of the file used for replication. (Default: 256)',
		   256);

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    # Add the remotedir option
    $self->addOption('rdir=s',
		     'Sub-directory (on the SE) where to put the file. (Default: edg_testing)',
		     'edg_testing');
  }

  # Add the logfile option
  $self->addOption('log=s','Logfile','RM.log');

  # Add the VO option
  $self->addOption('vo=s','Virtual Organisation');

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    # Add the rcconf option
    $self->addOption('rcconf=s','Full path of rc.conf file');
  } else {
    # Add the config option
    $self->addOption('config=s','Full path of RM config file');
  }

  # Add the --insecure option
  $self->addOption('insecure','Connect in an insecure manner, i.e. not https','');

  # Add the debug option
  $self->addOption('debug=i','Debug flag: 0=>Silent, 1=>Verbose (Default 1)',1);

}

# Check that mandatory arguments have been given
sub checkArguments {
    my $self = shift;

    my $error = $self->SUPER::checkArguments();
    if ($error) {
        $self->setErrorMessage('standard argument checking failed');
        return 1;
    }

    # ?GM After parsing args for a step, I see that $self->{args} still contains
    #     all the option flags, while ARGV has been truncated by GetOptions and
    #     has only arguments. I don't quite understand why this happens.
    #     Anyhow, I want to ensure that from here on $self->{args} has only arguments.
    @{$self->{args}} = @ARGV;

    # Ensure that two SE hostnames have been specified.
    if ($#{$self->{args}} + 1 < $self->{NARGS}) {
      print "ERROR - Wrong number of arguments found, expect at least $self->{NARGS}\n";
      print "<@{$self->{args}}>\n";
      return 1;
    }

    return 0;
}

# Check that:
#  -The mandatory options have been given
#  -The Grid proxy exists
sub checkPrerequisites {
  my $self = shift;
  my $rc = $self->SUPER::checkPrerequisites();
  return $rc if ($rc);

  # Ensure that the mandatory options have been given
  if (!defined $self->{opts}->{vo}) {
    $self->setErrorMessage("Mandatory option -vo was not provided");
    return 1;
  };

  # The default value for the rcconf option depends on the vo
  if (!defined $self->{opts}->{rcconf}) {
    $self->{opts}->{rcconf} = "/opt/edg/etc/" . $self->{opts}->{vo} . "/rc.conf";
  }

  # Check that Grid proxy exists
  my $rc = checkProxyExists();
  $self->setErrorMessage('user proxy does not exist') if ($rc);

  if (defined $self->{opts}->{config}) { $self->{CONFIG} = "--config=$self->{opts}->{config}"; }

  if (defined $self->{opts}->{insecure}) { $self->{INSECURE} = "--insecure";}

  return $rc;
}

sub runTest {
  my $self = shift;
  $self->{STEPRC} = 0;

  return 0;
}

sub evaluateTest {
  my $self = shift;

  return 0;
}

# Utility function to compare checksums of a remote and a local file
# Arguments:
#  $rHost: Remote SE host
#  $rPath: Remote /flatfiles path of the test file
#  $lPath: Local path of the test file
# Returns a hash reference with keys:
#  rc: Return code
#  err: Error message string
sub checkSumTest {
  my $self = shift;

  my ($rHost,$rPath,$lPath) = @_;

  my $rc = 0;
  my $err = "";

  # Run checksum on the remote host using globus-job-run
  $href = $self->runCommand("globus-job-run $rHost /usr/bin/cksum $rPath");

  if ($href->{rc} != 0){
    $err = "Remote checksum to $rHost failed";
    $rc = 1;
  } else {

    # Parse output of cksum and get the numeric data
    if ($href->{out} =~ /([0-9]+\s[0-9]+)\s.+/) {$rCksum = $1;}

    # Run checksum on the local host and parse the output as well
    $href = $self->runCommand("/usr/bin/cksum $lPath");

    if ($href->{out} =~ /([0-9]+\s[0-9]+)\s.+/) {$lCksum = $1;}

    if ($rCksum != $lCksum) {
      $err = "Files checksum or size differ r=<$rCksum> l=<$lCksum>";
      $rc = 1;
    }
  }

  return {RC=>$rc, ERR=>$err};
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
