#!/usr/bin/perl

package BaseTest::ReverseDNS;

use strict;
use vars qw(@ISA);

@ISA=qw(BaseTest);

=head1 NAME

ReverseDNS.pm - Check that reverse DNS lookups of host names works

=head1 SYNOPSIS

Check that reverse DNS lookups of host names works

=head1 DESCRIPTION 

B<ReverseDNS.pm> tests that reverse DNS lookups for the given host
names works. This is required for Globus GSI security to properly
validate a host certificate.  This test should be run from outside a
site's firewall for each machine which runs one or more grid services
identified via a host certificate.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "ReverseDNS: setContext\n";

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::ReverseDNS [options] <hostname> [hostname ...]

END
    ;

    $self->{DescText} = <<'END'

Check that a reverse DNS lookup works for all of the supplied 
host names.   The host names must be fully qualified.

The reverse DNS lookup is done as a basic part of checking the 
validity of host certificates.  If the lookup does not work, then
secured requests to that machine will fail (e.g. job submission,
GridFTP transfers, etc.).

END
    ;

}

sub runTest {
    my $self = shift;

    print "ReverseDNS: runTest\n";

    # Keep a list of all of the host names which fail.
    my @failed;

    # Loop over each argument doing a reverse DNS lookup for each.
    foreach my $host (@{$self->{args}}) {
        print "Checking... $host\n";

        # Forward direction to get IP number.
        my ($hname, $aliases, $addrtype, $length, @binaddrs) = 
	    gethostbyname($host);
        if ($?) {
	    print "Failed in forward direction. $?\n";
            push @failed, $host;
        } else {

            # Reverse to get the name.  It must return exactly the 
            # same hostname!
            $hname = gethostbyaddr($binaddrs[0], $addrtype);
	    if ("$hname" ne "$host") {
		print "Failed in reverse direction. ($hname != $host)\n";
		push @failed, $host;
	    }
	}
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("reverse DNS lookup failed for $names");
        return 1;
    }

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
