#!/usr/bin/perl

package BaseTest::SESetupTest;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

SESetupTest.pm 

=head1 SYNOPSIS

SESetupTest.pm - Check basic setup requirements for a storage element.

=head1 DESCRIPTION 

B<SESetupTest.pm> This checks that the basic SE configuration is
correct.  It specifically, checks:
 * That the SE GRIS is running.
 * Supported VO(s) are defined in the GRIS.
 * The GridFTP server is running.
 * The storage path exists.
 * The "magic" softlink exists for NFS access (via BrokerInfo).
 * The edg_query_vo_storage cron job is running.
This does not check the data management software or configuration.

=head1 AUTHOR

Stephen Burke (adapted by Charles Loomis).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "SESetupTest: setContext\n";

    # The command needed to run this test.
    $self->{commands} = ["grid-proxy-info",
			 "edg-gridftp-exists",
                         "edg-gridftp-ls"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::SESetupTest [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'

Check the basic setup of an SE.  This test checks:
 * That the SE GRIS is running.
 * Supported VO(s) are defined in the GRIS.
 * The GridFTP server is running.
 * The storage path exists.
 * The "magic" softlink exists for NFS access (via BrokerInfo).
 * The edg_query_vo_storage cron job is running.
This does not check the data management software or configuration.

This test was supplied by Stephen Burke. 

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "SESetupTest: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    print "SESetupTest: runTest\n";

    # Keep a list of all SEs which fail
    my @failed;

    # Loop over each argument doing a full test for each.
    my $server;
    foreach $server (@{$self->{args}}) {
        my $error = $self->testSESetupTest($server);
        push @failed, $server if ($error);
    }

    # If there are any servers in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("SESetupTest test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual test is here.  It just takes a server name, extracts the
## supported VOs and runs checks. 
##

sub testSESetupTest {

    my ($self, $server) = @_;

    my $rc = 0;

    print "Checking SE: $server\n";

    # First do the LDAP search to discover what VOs are supported.
    my $command = "ldapsearch -LLL -H ldap://$server:2135" .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(objectclass=storageelement)'" . 
        " SEvo ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out');
        return 1;
    }
    
    # Pull out the supported VOs and the paths.
    my @entries = split /\s*\n\s*/, $output[1];
    my %paths;
    foreach (@entries) {
	/SEvo:\s*(\w+)\s*:(.*)\/\1/;
	$paths{$1} = "$2" if "$1";
    }

    # Verify that there is at least one supported VO.
    my $vocount = (keys %paths);
    if ($vocount <= 0) {
	$rc = 1;
	print STDERR "No supported VOs for $server.\n";
	return $rc;
    }

    # Now check to see if the file transfer protocol is supported.
    my $command = "ldapsearch -LLL -H ldap://$server:2135" .
        " -x -b 'mds-vo-name=local,o=grid' " .
        "'(objectclass=storageelementprotocol)'" . 
        " SEProtocol ";
    my $arrayref = timeout($command,90);
    my @output = @$arrayref;

    # Catch errors from the execution of the command. 
    if ($output[0]==1) {
        $self->setErrorMessage('could not fork ldapsearch');
        return 1;
    } elsif ($output[0]==2) {
        $self->setErrorMessage('ldapsearch timed out');
        return 1;
    }
    
    # Pull out the protocols and check for file.
    my @entries = split /\s*\n\s*/, $output[1];
    my $filesupported = 0;
    foreach (@entries) {
	/SEProtocol:\s*(\w+)\s*/;
	$filesupported = 1 if ("\U$1" eq "FILE");
    }

    # For each of the supported VOs, verify that the mount point exists. 
    print "Checking mountpoints...\n";
    my $key;
    foreach $key (sort keys %paths) {
	my $path = "$paths{$key}/$key";
	my $result = `edg-gridftp-exists gsiftp://$server/$path`;
	if ($?) {
	    $rc = 1;
	    print STDERR "Mountpoint for $key, $path, doesn't exist.\n";
	}
    }
    return $rc if $rc;

    # For each VO, check the "magic" mountpoint.
    if ($filesupported) {
	print "Checking magic mountpoints...\n";
	my $key;
	foreach $key (sort keys %paths) {
	    my $path = "$paths{$key}/$paths{$key}/$key";
	    my $result = `edg-gridftp-exists gsiftp://$server/$path`;
	    if ($?) {
		$rc = 1;
		print STDERR "Magic mountpoint for $key, $path, doesn't exist.\n";
	    }
	}
	return $rc if $rc;
    } else {
	print "File protocol not supported. Not checking magic mount point.\n";
    }

    # For each VO, check the storage file is available.
    print "Checking free space file...\n";
    my $key;
    foreach $key (sort keys %paths) {
	my $path;
	if ($filesupported) {
	    $path = "$paths{$key}/$paths{$key}/$key/edg_query_vo_storage.dat";
	} else {
	    $path = "$paths{$key}/$key/edg_query_vo_storage.dat";
	}
	my $result = `edg-gridftp-exists gsiftp://$server/$path`;
	if ($?) {
	    $rc = 1;
	    print STDERR "Free space file for $key, $path, doesn't exist.\n";
	}
    }
    return $rc if $rc;

    # Date checking of these files must be done my hand. 
    print "Checking date of free space file...\n";
    print "Printed dates should be 'recent'.  You must check manually. \n";
    my $key;
    foreach $key (sort keys %paths) {
	my $path;
	if ($filesupported) {
	    $path = "$paths{$key}/$paths{$key}/$key/edg_query_vo_storage.dat";
	} else {
	    $path = "$paths{$key}/$key/edg_query_vo_storage.dat";
	}
	my $result = `edg-gridftp-ls --verbose gsiftp://$server/$path`;
	my @fields = split /\s+/, $result;
	print "$key: $fields[5] $fields[6] $fields[7]\n";
	if ($?) {
	    $rc = 1;
	    print STDERR "Free space file for $key, $path, doesn't exist.\n";
	}
    }
    return $rc if $rc;

    return $rc;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
