#!/usr/bin/perl

package BaseTest::SEUtils;

use strict;
use vars qw(@ISA);
use File::Path;
use EDGUtils;
use IO::Handle;

@ISA=qw(BaseTest);
###use vars qw($globus_url_copy $edg_gridftp_ls);
use vars qw($globus_url_copy $edg_gridftp_ls $edg_gridftp_rm $globus_job_run $edg_rm);
###use vars qw($globus_url_copy $globus_job_run);
use vars qw($dg_job_submit $dg_job_get_output $dg_job_status $statusTag $destTag);

=head1 NAME

SEUtils.pm - A set of utilities for SE certification

=head1 SYNOPSIS

SEUtils.pm - BaseTest derived class containing common stuff for any
             edg-se-webservice CLI tests

=head1 DESCRIPTION

B<SEUtils.pm> - BaseTest derived class containing common stuff for any
edg-se-webservice CLI tests. This is not intended to be called directly
but to be used through the edg-tb-test command.

=head1 AUTHOR

Andrea Formica (formica@hep.saclay.cea.fr).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Andrea Formica (CEA/Saclay).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

use vars qw($statusTag $destTag $jobidTag);
$statusTag = 'Current Status';
$destTag = 'Destination';
$jobidTag = 'Printing status info for the Job';

use vars qw($doneState $abortState $cancelState $runningState $clearState);
$doneState = 'Done (Success)';
$abortState = 'Aborted';
$cancelState = 'Done (Cancelled)';
$runningState = 'Running';
$clearState = 'Cleared';


sub setContext {
  my $self = shift;
  $self->SUPER::setContext();

  print "SEUtils::setContext \n";

  # Add the resource option.
  $self->addOption('resource=s',
		     'Use the given CE.');

  # Add the number of files
  $self->addOption('nfiles=i',
		   'Number of test files to be created. (Default: 1)',1);

  # Add the filesize option
  $self->addOption('bs=i',
		   'Size in kB of a single block. (Default: 1024)',1024);

  # Add the number of count option
  $self->addOption('count=i',
		   'Number of blocks used. (Default: 1)',1);

  # Add the output filename
  $self->addOption('of=s',
		   'Output file name. (Default: seTestFile)','seTestFile');

  # Add the protocol used
  $self->addOption('stp=s',
		   'Comunication Protocol. (Default: gsiftp)',
		   'gsiftp');

  # Add the destination SE url
  $self->addOption('seurl=s',
		   'Destination SE url. (Default: lxshare0408.cern.ch/tmp)',
		   'lxshare0408.cern.ch/tmp');

  # Add the vo name
  $self->addOption('vo=s',
		   'Virtual Organisation name. (Default: wpsix)',
		   'wpsix');
  # Add the vo name
  $self->addOption('config=s',
		   'Configuration for R-GME file name. (Default: edg-replica-manager.conf)',
		   'edg-replica-manager.conf');

  # Number of identical jobs to send and number of parallel streams. 
  # These are useful for job storms. 
  $self->addOption('njobs=i',
                   'Number of jobs in each stream.',
                   '1');
  $self->addOption('nstreams=i',
  		   'Number of streams.',
                   '1');
  
  # Add the logfile option
  $self->addOption('log=s','Logfile','SEUtils.log');

  # Add the --insecure option
  $self->addOption('insecure','Connect in an insecure manner, i.e. not https','');

  # Commands necessary to run this test.
  if ( $ENV{EDG_FLAVOR} eq 'edg1' ) {
      (${dg_job_submit}, ${dg_job_get_output}, ${dg_job_status}) = ( "dg-job-submit", "dg-job-get-output", "dg-job-status");
  } else {
      (${dg_job_submit}, ${dg_job_get_output}, ${dg_job_status}) = ( "edg-job-submit", "edg-job-get-output", "edg-job-status");
  }
  (${globus_url_copy}, ${edg_gridftp_ls}, ${edg_gridftp_rm}, ${globus_job_run}, ${edg_rm}) = ("globus-url-copy", "edg-gridftp-ls", "edg-gridftp-rm", "globus-job-run", "edg-rm");
  
  $self->{commands} = ["${globus_url_copy}",
		       "${edg_gridftp_ls}" ,
		       "${edg_gridftp_rm}" ,
		       "${globus_job_run}" ,
		       "${edg_rm}",
		       "${dg_job_submit}",
		       "${dg_job_get_output}",
		       "${dg_job_status}"]; 

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::SEUtils [options] 

END
    ;

    $self->{DescText} = <<'END'

This is a parent skeleton of tests which use a typical EDG SE.

If the "resource" option is used, this resource is used as CE.

END
    ;
}

# Check that:
#  -The Grid proxy exists
sub checkPrerequisites {
  my $self = shift;
  my $rc = $self->SUPER::checkPrerequisites();
  return $rc if ($rc);

  # Check that Grid proxy exists
  my $rc = checkProxyExists();
  $self->setErrorMessage('user proxy does not exist') if ($rc);

  # Check that destination URL seurl contains an existing host
  my (@host) = split("\/",$self->{opts}->{seurl});
  print "Pinging $host[0] ... ";
  $self->{sehost} = $host[0];
  my $ping = `${edg_gridftp_ls} gsiftp://$self->{opts}->{seurl}`;
  if ($?) {
      $self->setErrorMessage('Cannot ping host $host');
  } else { print "OK \n"; }

  return $rc;
}

# The setup creates a temporary directory to be used to hold the
# file(s) generated for the SE test. 
# (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "SEUtils: setupTest \n";

    # Create a temporary directory to hold generated files and
    # job output.
    my $tmpdir;
    do {
        my $value = int(rand(32768));
        $tmpdir = "/tmp/$ENV{USER}-$$-$value";
    } while (-e $tmpdir);

    # Now create the directory.
    mkpath($tmpdir, 0, 0755);
    if ($?) {
        $self->setErrorMessage('could not create temporary directory');
        return 1;
    }
    print "Temporary directory: $tmpdir\n";
    $self->{tempdir} = $tmpdir;

    my $evaluateResult="$tmpdir/EvaluateResult.txt";
    print "Evaluation of results is based on file $evaluateResult \n";
    $self->{result}=$evaluateResult;

    $self->createFiles();
    if ($?) {
        $self->setErrorMessage('could not create files...');
        return 1;
    }
#    $self->copyFiles();
#    if ($?) {
#        $self->setErrorMessage('could not copy files...');
#        return 1;
#    }
    $self->registerFiles();
    if ($?) {
        $self->setErrorMessage('could not register files...');
        return 1;
    }

    return 0;
}

# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

    print "SEUtils: cleanup\n";

    my $dir = $self->{tempdir};
    if ( $self->{evaluatetest} == "OK" ) {
#	rmtree($dir) if $dir;
    }

    $self->deleteRegisteredFiles();
    if ($?) {
        $self->setErrorMessage('could not delete registered files...');
        return 1;
    }
    $self->deleteFiles();
    if ($?) {
        $self->setErrorMessage('could not delete files...');
        return 1;
    }

    return 0;
}

# By default this will simply check that the output of the job 
# contains the same cksum info than the EvaluateTest.txt file.  The
# names of the standard output and error files are passed through the
# class context with keys 'stdout' and 'stderr', respectively. 
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;

    # Check that the std.out and std.err files exist.
    return 1 if (! (-e "$stdout"));
    return 1 if (! (-e "$stderr"));

    # Check that the standard error has a zero size.
    return 1 if (-s "$stderr");

    # Check the standard output contains the correct cksum. 
    open FILE, "<$stdout";
    my $value = 0;
    $self->{evaluatetest}="OK";
    my $fkey;
    my %allResults;
    while (<FILE>) {
	if ( /\b(cksum)\s+(\d+)\s+.*\/(.*\.\d+)$/i ) {
	    $fkey = $3;
	    $allResults{$fkey} = $2;
	}
    }
    close FILE;
    open FILE, "<$self->{result}";
    my $evfkey;
    my %evallResults;
    while (<FILE>) {
	if ( /\b(\d+)\s+.*\/(.*\.\d+)$/i ) {
	    $evfkey = $2;
	    $evallResults{$evfkey} = $1;
	    if ( $evallResults{$evfkey} != $allResults{$evfkey}) {
		print "Wrong cksum result for file $evfkey from file $self->{result}\n";
		$value = 1;
		$self->{evaluatetest}="NOK";
	    } else {
		print "Checking result for $evfkey ... [orig] $evallResults{$evfkey} == [remote] $allResults{$evfkey}\n";
	    }
	}
    }
    close FILE;

    if ( $ENV{MAINSCRIPT} ) {
	my @resC = `cat $stdout`;
	print " stdout: $stdout \n @resC";
    }

    return $value;
}

# This takes the JDL file and submits it.  It follows the status of
# the job and retrieves the output when it finishes.  It sets the
# locations of the standard output and error for evaluation. 
sub runTest {

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

    my $self = shift;

    print "SEUtils: runTest\n";

    # Set the prefix to use for the job ids.  Currently hardcoded
    # here.  It contains the full path of the file excepting the
    # stream suffix. 
    $self->{prefix} = "$self->{tempdir}/jobids";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nstreams}) {
	my $fname = "$self->{prefix}.$str";
	open IDS, ">$fname";
	close IDS;
	print "Created Job ID file: $fname\n";
    }

    my $idfile = "$self->{prefix}.1";
    print "JobID File: $idfile\n";
    
    # Submit the job.
    my $pidref = $self->parallelSubmit();

    # Waiting on child processes. 
    my $list = join(", ",@$pidref);
    print localtime() . "\n";
    print "Waiting on children... $list \n";
    waitforpids($pidref);

    # Monitor the progress of the jobs.
    print "Beginning monitoring  \n\n";
    $self->monitorJobs();

    # Retrieve the output for all jobs.
    print "Retrieving output from jobs... $self->{prefix}.total\n";
    my $rc = $self->retrieveAll();
    if ($rc) {
        $self->setErrorMessage('output retrieval failed');
        return 1;
    }
    return 0;
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

    print "SEUtils: evaluateTest\n";

    # Loop over all of the streams. 
    my $numTotal = 0;
    my $numOK = 0;

    # Loop over all jobids.
    open IDS, "<$self->{prefix}.total";
    while (<IDS>) {
	my $jobid = $_;
	chomp($jobid);
	$jobid =~ /(.*\/)(\S+)/;
	my $subdir = $2;
	
	my $rc = $self->evaluateOutput("$self->{tempdir}/$subdir/std.out",
				       "$self->{tempdir}/$subdir/std.err");
	print STDERR "Output check failed for $jobid.\n" if ($rc); 
	
	$numTotal++;
	$numOK++ unless ($rc);
    }
    close IDS;

    # Returns 0 is all OK, 1 if some OK, 2 if all failed. 
    # Rene Metery fix: when 0 jobs were submitted the test failed
    if ($numOK == $numTotal) {
	if (($numOK == 0) && ($numTotal == 0)) {
	    return 2;
	} else {
	    return 0;
	}
    } elsif ($numOK > 0) {
	return 1;
    } else {
	return 2;
    }
}

##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##

sub createFiles {

    my $self = shift;

    print "SEUtils: createFiles\n";

    # Set the prefix to use for the test files.  
    $self->{prefix} = "$self->{tempdir}";
    open (RESULT,">>$self->{result}") || die "Cannot open comparaison file $self->{result}\n";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nfiles}) {
        my $dname = getDestinationName($self->{prefix},$self->{opts}->{of});
	my $fname = "$self->{prefix}/$dname.$str";
	my $command = "dd if=/dev/urandom of=$fname bs=$self->{opts}->{bs} count=$self->{opts}->{count}";
	print "Creating $fname \n";
	my $jobid = `$command`;
	if ($?) {
	    print "SEUtils: command $command error \n ==> result $? \n";
	    return 1;
	}
	print "Created Test File file: $fname\n";
# Generate the cksum file for comparaison with the output
	my $cksum="cksum $fname";
	my $ckres=`$cksum`;
        print RESULT "$ckres";
    }
    return 0;
}

sub copyFiles {

    my $self = shift;

    print "SEUtils: copyFiles\n";

    # Set the prefix to use for the test files.  
    $self->{prefix} = "$self->{tempdir}";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nfiles}) {
        my $dname = getDestinationName($self->{prefix},$self->{opts}->{of});
	my $fname = "$self->{prefix}/$dname.$str";
	my $destfname = "$self->{opts}->{stp}://$self->{opts}->{seurl}/$dname.$str";
	my $command = "${globus_url_copy} file://$fname $destfname";
	print "Copying $fname using $command \n";
	my $jobid = `$command`;
	if ($?) {
	    print "SEUtils: command $command error \n ==> result $? \n";
	    return 1;
	}
	print "Copied Test File file: $fname to $destfname \n";
    }
    return 0;
}


sub deleteFiles {

    my $self = shift;

    print "SEUtils: deleteFiles\n";

    # Set the prefix to use for the test files.  
    $self->{prefix} = "$self->{tempdir}";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nfiles}) {
        my $dname = getDestinationName($self->{prefix},$self->{opts}->{of});
	my $fname = "$self->{prefix}/$dname.$str";
	my $command = "rm $fname";
	print "Removing $fname \n";
	my $jobid = `$command`;
	if ($?) {
	    print "SEUtils: command $command error \n ==> result $? \n";
	    return 1;
	}
	print "Delete Test File file: $fname\n";
    }
    return 0;
}

sub registerFiles {

    my $self = shift;
    my $edgoption = "cr";
    print "SEUtils: registerFiles\n";

    # Set the prefix to use for the test files.  
    $self->{prefix} = "$self->{tempdir}";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nfiles}) {
        my $dname = getDestinationName($self->{prefix},$self->{opts}->{of});
	my $fname = "$self->{prefix}/$dname.$str";
	my $tempurl = join("","srm://","$self->{opts}->{seurl}/","$dname.$str");
	my $logicalfname = getLogicalName("$dname.$str");
# Register files
#	my $command = "${edg_rm} --vo=$self->{opts}->{vo} -i --config=$self->{opts}->{config} $edgoption $tempurl -l lfn:$logicalfname";
	my $command = "${edg_rm} --vo=$self->{opts}->{vo} -i --config=$self->{opts}->{config} $edgoption file://$fname -l lfn:$logicalfname -d $tempurl";
	print "Registering local file $fname \n using $command \n";
	my $jobid = `$command`;
	if ($?) {
	    print "SEUtils: command $command error \n ==> result $? \n";
	    return 1;
	}
	$self->{logicalfile} .= " lfn:$logicalfname";
	print "Register Test File ==>  file: $fname has logical file name: $logicalfname\n";
        print "Updating list $self->{logicalfile} \n";
# Get GUID from rgma
	print "Test if we can retrieve the GUID \n";
	$command = "${edg_rm} --vo=$self->{opts}->{vo} -i --config=$self->{opts}->{config} listGUID lfn:$logicalfname"; 
##	$command = "${edg_rm} --vo=$self->{opts}->{vo} -i --config=$self->{opts}->{config} listGUID lfn:pippo"; 
	my $jobresult = `$command`;
	if ($?) {
	    print "SEUtils: command $command error \n ==> result $? \n";
	    return 2;
	} 
	my ($string,$theGuid) = split(":",$jobresult,2); 
	chomp($theGuid);
	print "Register Test File ==>  lfn:$logicalfname has guid: $theGuid\n"; 
	$self->{logicallist} .= "\"guid:$theGuid\",";
	print "Update list $self->{logicallist} \n";
    }
    return 0;

}

sub deleteRegisteredFiles {

    my $self = shift;

    print "SEUtils: deleteRegisteredFiles\n";

    # Set the prefix to use for the test files.  
    $self->{prefix} = "$self->{tempdir}";

    # Create all of the necessary jobids files. 
    foreach my $str (1 .. $self->{opts}->{nfiles}) {
        my $dname = getDestinationName($self->{prefix},$self->{opts}->{of});
	my $logicalfname = getLogicalName("$dname.$str");
# Register files
	print "Delete Registered file $logicalfname \n";
	my $command = "${edg_rm} --vo=$self->{opts}->{vo} -i --config=$self->{opts}->{config} deleteFile -s $self->{sehost} lfn:$logicalfname";
	print "Delete from Register $logicalfname using $command \n";
	my $jobid = `$command`;
	if ($?) {
	    print "SEUtils: command $command error \n ==> result $? \n";
	    return 1;
	}
    }
    return 0;
}

sub getDestinationName() {

    my (@theName) = @_;
    my (@extension) = split("/",$theName[0]);
#    print "getDestinationName: $_[0], $_[1], $_[2] and extension $extension[$#extension]\n";
#    print "getDestinationName: $theName[0] - $theName[1] - $theName[2]\n";
    return "$extension[$#extension]_$theName[1]";
}
sub getLogicalName() {

    my (@theName) = @_;
#    print "getLogicalName: $theName[0] \n";
    return "LOGICAL_$theName[0]";
}

#### Job submission part is here

#  The name of the jdl file is taken from the "$self->{jdl}".  If
# "$self->{opts}->{resource}" is set, then the job bypasses the match
# making sending the job directly to that resource.  The resource must
# be fully specified.
sub submit { 

    my ($self, $argref) = @_;

    # Set the defaults if not specified. 
    my ($idfile, $njobs) = @$argref; 
    $idfile ||= "jobids.txt";
    $njobs ||= 1;

    # Construct the submit command.  
    my $command = "${dg_job_submit} -nomsg -noint ";
    my $logfile = "$self->{tempdir}/${dg_job_submit}_$<_$$.log";
    $command .= " -logfile $logfile";
    $command .= " -vo $self->{opts}->{vo}";
    $command .= " -resource $self->{opts}->{resource} " 
	if ($self->{opts}->{resource});
    $command .= " $self->{jdl}";

    # Open the output file for the jobids. 
    print "SEUtils: submit ==> Writing output file $idfile \n";
    open IDS, ">$idfile";

    # Loop over the job submissions writing out job id for each job. 
    my $numOK = 0;
    my $numNotOK = 0;
    foreach (1 .. $njobs) {

        my $jobid = '';

        # Note that the noint option should suppress all interactive
        # commands, but doesn't suppress obtaining a proxy if one
        # doesn't exist.  This screws up getting the jobid.  A valid
        # proxy is required in the prerequisites, but if it gets to
        # this point without one, this function will fail.

        print "SEUtils: Submission of job $command \n\n==================\n";
        $jobid = `$command`;
#p	print " jobid: $jobid $? \n" if ( $jobid =~ /https/ );
	$self->{job}->{submit}->{log} = $jobid if ( $njobs == 1 );
        chomp $jobid;

        if ( $jobid =~ /https/ ) {
            print IDS "$jobid\n";
            $numOK++;
        } else {
	    print STDERR " Jobid: $jobid \n Submission status: $? \n";
            print STDERR "Job submission failed.  Continuing...\n";
            $numNotOK++;
        }
    }

    # Close the output file.
    close IDS;

    # Set the return code depending on the situation. 
    if ($numOK>0 && $numNotOK==0) {
        # All OK. 
        return 0;
    } elsif ($numOK>0 && $numNotOK>0) {
        # Some OK. 
        return 1;
    } else {
        # None OK. 
        return 2;
    }
}
# Follows the job progress.  It stops when the job state becomes
# OutputReady or Aborted.  (There is currently no timeout, but this
# should be added.) 
sub follow {
    my $jobid = shift(@_);

    my $running = 1;
    while ($running) {
	sleep 10;

	my $pref = status($jobid);
	my %params = %{$pref};
	$running = 0 if ("$params{$statusTag}" eq $doneState);
	print "Status: $params{$statusTag}, $params{$destTag}\n";

	if ("$params{$statusTag}" eq $abortState) {
	    foreach my $key (keys %params) {
		print "$key: $params{$key}\n";
	    }
            return 1;
	}
    }
    return 0;
}


# Returns a hash with the status parameters and the values.
sub status {
    my $jobid = shift(@_);

    open RESPONSE, "${dg_job_status} $jobid 2>/dev/null |";
    my %params;
    while (<RESPONSE>) {
	if (m/(\S[^:]+\S)(\s*:\s*)(\S.*\S)\s*/) {
	    my $param = $1;
	    my $value = $3;
	    $params{$param} = $value;
	}
    }
    close RESPONSE;

# Send back a reference to the hash of parameters.
    return \%params;
}


# Retrieve the job output.
sub retrieve {
    my ($jobid, $dir) = @_;
    my $logfile = "$dir/${dg_job_get_output}_$<_$$.log";
    mkdir $dir, 0777;
    `${dg_job_get_output} -noint -dir $dir -logfile $logfile $jobid`;
    return "" if ($?);

    opendir OUTPUT, "$dir";
    my @files = readdir(OUTPUT);

    # This loop assumes that there is only one subdirectory!
    my $outdir;
    foreach my $file (@files) {
	$outdir = "$dir/$file" if (-d "$dir/$file");
    }
    return $outdir;
}



# Retrieve all of the outputs from a list of jobs. 
sub retrieveAll {
    my ($self) = @_;

    open IDS, "<$self->{prefix}.total";
    while (<IDS>) {
        my $jobid = $_;
	chomp($jobid);

	my $pref = status($jobid);
	my %params = %{$pref};

        if ("$params{$statusTag}" eq $doneState) {
	    my $logfile = "$self->{tempdir}/${dg_job_get_output}_$<_$$.log";
            my @resDJGO = `${dg_job_get_output} -noint -dir $self->{tempdir} -logfile $logfile $jobid`;
#p	    print " resDJGO: \n@resDJGO";
            print STDERR "${dg_job_get_output} failed for $jobid\n" if ($?);
	    $self->{job}->{getout}->{log} = "@resDJGO";
        } else {
            print STDERR "$jobid in status '$params{$statusTag}'\n";
        }
    }
    close IDS;

    return 0;
}



# This routine monitors the jobs listed in a job id file.  It keeps
# following them until all of the jobs are in a terminal state
# (Aborted, Cancelled, or 'Done (Success)'). 
sub monitorJobs {

    my ($self, $idfile) = @_;

    # Pull in the job ids from all streams.
    opendir TEMP, "$self->{tempdir}";
    my @idfiles = grep /jobids\.\d+/, readdir TEMP;
    closedir TEMP;
    my @full = map { $_ = "$self->{tempdir}/$_"} @idfiles;

    # Merge all of these into single file.
    my $tfile = "$self->{prefix}.total";
    open TOTAL, ">$tfile";
    foreach my $file (@full) {
	open IDS, "<$file";
	while (<IDS>) {
	    print TOTAL $_ if (m/^https:.*/);
	}
	close IDS;
    }
    close TOTAL;

    my %status = ();
    my %location = ();

#p    my $notDone;
    use vars qw($notDone);
    do {

        # Reinitialize all existing keys to be zero in status and location. 
        foreach (keys %status) {
            $status{$_} = 0;
        }
        foreach (keys %location) {
            $location{$_} = 0;
        }

        # Update the status from the logging and bookkeeping. 
	my $logfile = "$self->{tempdir}/${dg_job_status}_$<_$$.log";
	my $scount = 0;
	my $response;

      STAT: {
        open STATUS, "${dg_job_status} -noint -logfile $logfile -i $tfile |";
	$response = "";
        while (<STATUS>) {

	    $response .= $_;
#p	    print " DJS: $_ ";

	    if ( /ECONNRESET/ ) {
		print STDERR " DJS: $_ ";
		if ( ! $scount++ ) {
		    close STATUS;
		    sleep 5;
		    redo STAT; 
		}
	    }

            if (/\s*${statusTag}\s*:\s*(\S.*\S)\s*/o) {
                my $key = $1;
                if ($status{$key}) {
                    $status{$key}++;
                } else {
                    $status{$key} = 1;
                }
            }

            if (/\s*${destTag}\s*:\s*(\S+)\s*/o) {
                my $key = $1;
                if ($location{$key}) {
                    $location{$key}++;
                } else {
                    $location{$key} = 1;
                }
            }

        }
        close STATUS;
    }

        ## The last response should be enough ...
    	$self->{job}->{status}->{log} = $response;
        
        # Print out the summary of the jobs. 
        print localtime() . " ($self->{tempdir})\n";
        my $total = 0;
        $notDone = 0;
        foreach my $key (sort keys %status) {
            print "$key: $status{$key}\n";
            $total += $status{$key};
            if (("$key" ne $abortState) && ("$key" ne $doneState) &&
		("$key" ne $clearState) && ("$key" ne $cancelState)) {
                $notDone += $status{$key};
            }
        }
        my $completed = $total - $notDone;
        print "TOTAL: $completed of $total completed.\n\n";

        foreach my $key (sort keys %location) {
            print "$key: $location{$key}\n";
            $total += $location{$key};
        }


        # Sleep for a while.
        sleep 30 if ($notDone>0);

    } while ($notDone>0); 
}

# Run job submissions in parallel.  This takes all of the parameters
# from the state of the object. 
sub parallelSubmit {

    my ($self) = @_;
    my @pids;

    my $i;
    foreach $i (1 .. $self->{opts}->{nstreams}) {
      FORK: { 
	    my $pid;
          if ($pid = fork) {
              # parent here
              # child process pid is available in $pid
              push @pids, $pid;
          } elsif (defined $pid) {
              # $pid is zero here if defined
              # child here
              # parent process available with getppid

	      # Do the actual submission.  The error return is ignored
	      # because there is no way to feed back this information
	      # easily.  
	      my $rc = $self->submit(["$self->{prefix}.$i",
				      "$self->{opts}->{njobs}"]);
	      
              # This is important otherwise child may execute parent code!
              exit;
          } elsif ($! =~ /No more process/) {
              # Recoverable error (EAGAIN); try again. 
              sleep 5;
              redo FORK;
          } else {
              # Strange error! Print error message. 
              print STDERR "Fork failure.\n";
          }
      }
    }

    return \@pids;
}


# Wait for the processes with the given pids to end. 
sub waitforpids {
    my $pidref = shift;

    # Wait for each process in turn.
    foreach my $pid (@$pidref) {
        my $result = waitpid $pid, 0;
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
