#!/usr/bin/perl

package BaseTest::SEUtils::SEtest;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::SEUtils);

=head1 NAME

SEUtils.pm - A set of utilities for EDG site certification tests

=head1 SYNOPSIS

SEUtils.pm 

=head1 DESCRIPTION 

B<SEUtils.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Andrea Formica (formica@hep.saclay.cea.fr).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.
=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "SEtest: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::SEUtils::SEtest [options]

END
    ;

    $self->{DescText} = <<'END'

This creates a number of files of fixed size, and copied them to a SE.

END
    ;

}

# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "SEtest: setupTest \n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};
    print "SEtest: Retrieved temporary directory $tmpdir \n";
    my $tmpfile = $self->{logicalfile};
    print "SEtest: Retrieved logical file names $tmpfile \n";
    my $tmplist = substr($self->{logicallist}, 0, -1);
    print "SEtest: Retrieved inputData string $tmplist \n";

    # Create the SH file. 
    my $exe  = $self->createSH($tmpdir);
    # Create the JDL file. (There is no script needed for this job.) 
    my $file = $self->createJDL($exe,$tmpdir,$tmpfile,$tmplist);

    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##

# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $exe, $dir, $logicalname, $inputlist) = @_;

    my $file = "$dir/JobDescription.jdl";
    my @scriptname = split(/\//,$exe);

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$scriptname[$#scriptname]";
Arguments     = "$logicalname";
InputData     = {$inputlist};
DataAccessProtocol  = {"file"};
StdOutput     = "std.out";
StdError      = "std.err";
OutputSandbox = {"std.out","std.err"};
InputSandbox = {"$exe"};
END
    ; 

#    print JDL "Requirements = $self->{opts}->{requirements};\n" 
#	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if $self->{opts}->{retrycount};
#    print JDL "MyProxyServer = $self->{opts}->{myproxyhost};\n" 
#	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# Creates the script file that will be send in execution.
sub createSH {

    my ($self, $dir) = @_;

    my $file = "$dir/mm.sh";

    # Write out the script
    open JSH, ">$file";
    print JSH <<'END'
#!/bin/sh
cat .BrokerInfo

# get the file list
flist="$@"

# To get the physical file name of the needed file
fname=`edg-brokerinfo getInputData`
echo $fname
declare -a idata
idata=($fname)

i=0
for lfn in $flist; do
#  echo $lfn
  theData=${idata[$((i))]}
# To get the hostname of the appropriate SE 
  SE=`edg-brokerinfo getCloseSEs` 
#  echo $SE
# get the physical file name
  PFn=`edg-brokerinfo getLFN2SFN $theData | cut -d '/' -f 4-`
#  echo $PFn
  mntpt=`edg-brokerinfo getSEMountPoint $SE`
#  echo $mntpt
  name="$mntpt/wpsix/$PFn"
#  echo $name
  theSum=`cksum $name`
  echo "cksum $theSum"
  i=$((i+1))
done
END
    ; 
    close JSH;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
