#!/usr/bin/perl

package BaseTest::SecurityTest;

use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

SecurityTest.pm - Check that globusrun authentication works on the hosts
# $Id: SecurityTest.pm,v 1.4 2003/09/05 14:13:09 loomis Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<SecurityTest.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "SecurityTest: setContext\n";

    # The command needed to run this test.
    $self->{commands} = ["globus-url-copy", 
			 "edg-gridftp-ls",
			 "grid-proxy-info"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::SecurityTest [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'

Check that a GridFTP server works by transfering files to and from the
local machine.  Also checks third-party transfers by transfering files
between each pair of remote servers.

For local->machine the three checks are:
 Step  1) local->machine
 Step  2) machine->local
 Step  3) check that contents are correct

For machine1->machine2 the three checks are:
 Step  1) machine1->machine2
 Step  2) machine2->local
 Step  3) check that contents are correct

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "SecurityTest: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub cleanup {
    my $self = shift;
    print "SecurityTest: cleanup\n";
    my @output = `ls -l /tmp/Sec; rm -r /tmp/Sec`;
    print " Cleanup: \n@output\n ";
    return $?;
}


sub runTest {
    my $self = shift;

    my $saveOut;
    if ( $ENV{MAINSCRIPT} ) {
	my $base = "/tmp/$ENV{USER}-$$";
	$self->{conffile} = "$base-SecurityTestconf";
	$self->{outfile}  = "$base-SecurityTestout";
	$self->{stepfile} = "$base-SecurityTeststep";
	$self->{htmldim}  = "2D";

#p	$saveOut = $self->switchStdout("$self->{outfile}");
	$saveOut = switchStdouterr("$self->{outfile}");
    }

    print "SecurityTest: runTest\n";

    # Keep a list of all directories which fail.
    @failed ;

    # Loop over each argument doing a full test for each.
#    @dirs = ("/tmp", "~");
    @dirs = ("/tmp");
    foreach $dir (@dirs) {
        print "Checking transfers to $dir\n";

	$error = testGridFTP($dir, \@{$self->{args}});
	push @failed, $dir if ($error);
#test	last; ##temp
    }

    if ( $ENV{MAINSCRIPT} ) {
#p	$self->restoreStdout($saveOut);
	restoreStdouterr($saveOut);
#good    close STDOUT;
#p    print STDERR " Calling buildStepFile() \n";
##old    $self->buildStepFile();
##old    buildConfFile(\@{$self->{args}});
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("SecurityTest test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testGridFTP {

    my ($prefix, $arrayref) = @_;
    my @hosts = @{$arrayref};

    # Setup some constants.
    $ok = "[OK]";
    $fail = "[FAIL]";

    # These check the ftp transfers to the tmp area and to the 
    # home area of the user. 
##gg    @dirs = ("/tmp","~");

    # Create a file name which is always in the temp area for the 
    # local machine as well as one in the location specified by
    # the prefix.
    my $locald = "/tmp/Sec";
    my $cre = mkdir ("$locald", 0777);
    my $local = "/tmp/Sec/$ENV{USER}-$$";

    # Choose a host and check connections to all of the other hosts.
    foreach $host1 (@hosts) {
	
	# Fill up the file with a random number.
	$value = fillFile($local);
	print "\nUsing prefix/value: $prefix, $value\n";
	
	$remote = "$ENV{USER}-$$" . "_local-$host1";
	$check = "$local--$remote";

	my $count = 0;
	open HTL, "edg-gridftp-ls gsiftp://$host1//tmp | grep http | ";
	while (<HTL>) {
	    chomp;
	    $count++;
	    last if ($count > 9);
	    chomp(my $res = `edg-gridftp-ls --verbose gsiftp://$host1//tmp/$_/user.proxy | grep http`);
	    if ($?) { next; }
#p	    print " http: $res $count \n";
	    my $target = "$locald/"."$_"."--user.proxy";
	    $res = doTransfer("gsiftp://$host1//tmp/$_/user.proxy", "file://$target--$count");
	}
	close HTL;
	return 0 if (1);

	# Check that transfers from local machine to remote machine works.
	if ( $ENV{MAINSCRIPT} ) { print "Bheck local --> $host1:  \n"; }
	$result1 = doTransfer("file://$local","gsiftp://$host1/$prefix/$remote");
	$result2 = doTransfer("gsiftp://$host1/$prefix/$remote","file://$check");
	$result3 = checkResult($value,$check);
	print "Check local --> $host1:  ";
	print "$result1 $result2 $result3\n";
	
	# These are then third party transfers.
	foreach $host2 (@hosts) {
##test	    next;  ##temp
	    if ("$host1" ne "$host2") {
		if ( $ENV{MAINSCRIPT} ) { print "Bheck $host1 --> $host2:  \n"; }
		$third = "$ENV{USER}-$$" . "_$host1-$host2";
		my $check = "$local--$third";
		$result1 = doTransfer("gsiftp://$host1/$prefix/$remote","gsiftp://$host2/$prefix/$third");
		$result2 = doTransfer("gsiftp://$host2/$prefix/$third","file://$check");
		$result3 = checkResult($value,$check);
		print "Check $host1 --> $host2:  ";
		print "$result1 $result2 $result3\n";
		my $rmes = `edg-gridftp-rm "gsiftp://$host2/$prefix/$third" 2>&1 `;
		my $lmes = `rm $check 2>&1 `;
		if ( "$rmes"."$lmes" ne "" ) {
		    print " Removing failed on $host2/local RM: :$third:$check: -$rmes-$lmes- \n";
		}
	    }
	}
	print "\n";
	my $rmes = `edg-gridftp-rm "gsiftp://$host1/$prefix/$remote" 2>&1 `;
	my $lmes = `rm $local $check 2>&1 `;
	if ( "$rmes"."$lmes" ne "" ) {
	    print " Removing failed on $host1/local RM: :$remote:$check: -$rmes-$lmes- \n";
	}
    }
}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransfer {

    my ($src, $dest) = @_;
    my $output = `globus-url-copy $src $dest 2>&1 `;
    my $flag = ($!) ? $ok : $fail;
##fake    $output = "FAKE \n";
    if ( $output ne "" ) {
	print " doTransfer $src TO $dest FAILED ==> \n$output";
    }
    return ($flag);
}

# Fill the given file with a new random number.
sub fillFile {
    my $fname = shift;
    my $value = rand();
    open RAN, ">$fname";
    print RAN "$value\n";
    close RAN;
    return ($value);
}


sub checkResult {
    my ($value, $fname) = @_;

    $val = `cat $fname`;
    chomp($val);
    $result = ("$value" eq "$val") ? $ok : $fail;
    if ( "$result" ne "$ok" ) {
	print " checkResult $value DIDN'T COMPARE TO $val \n";
    }
    return ($result);
}

sub buildStepFileNO {

    my $self = shift;
    print STDERR " Called buildStepFile() with $self->{outfile} \n";
    open OUT,  "$self->{outfile}"   or return 1;
    open STEP, ">$self->{stepfile}" or return 1;

    my $stepI = 3;
    my $stepM = $stepI;
    while (<OUT>) {
	my @m = split /\s+/;
#p	print STDERR " Parsing $m[0] $m[3] \n";
	if (/Checking/ && $m[3] ne "/tmp") { 
	    $stepI = 6; 
	    $stepM = $stepI > $stepM ? $stepI : $stepM;
	    next; 
	}
	$stepI = $stepM;

## Fetching the Error log, if any
	if ( $m[0] eq "Bheck" ) {
	    my $next;
            chop($m[3]);
            print STEP "STEPbegin $stepI $m[1] $m[3] \n";
	    while ( $next !~ /^Check/ ) {
		print STEP $next;
		$next = <OUT>;
	    }
	    @m = split /\s+/, $next;
	}

## Adding the status of each test
	if ( $m[0] eq "Check" ) {
	    chop($m[3]);
#was	    print STEP "STEPbegin $stepI $m[1] $m[3] \n";
	    print STEP "STEPend $stepI $m[1] $m[3] $m[6] \n";
	    $stepI--;
	    print STEP "STEPend $stepI $m[1] $m[3] $m[5] \n";
	    $stepI--;
	    print STEP "STEPend $stepI $m[1] $m[3] $m[4] \n";
	}
    }
    close OUT;

## Adding the Info
    print STEP <<"STEPEOF";
STEPinfo 0
$self->{DescText}
Steps 1 to 3 are for Transfers to/from "/tmp" directory on remote node,
while
Steps 4 to 6 are for Transfers to/from USER's HOME directory on remote node.
STEPEOF

## Adding the Step count
    print STEP "STEPcount $stepM \n";
    close STEP;

    return 0;
}

sub buildConfFile {
    my ($self, $arrayref) = @_;
    my @hosts = @{$arrayref};
##old    open CONF, ">/tmp/$ENV{USER}-$$-GridFTPconf";
    open CONF, ">$self->{conffile}";
#p    print CONF " @hosts \n";
    print CONF <<'EOCONF';
######################################################
# F.Chollet 31/01/03  chollet@lapp.in2p3.fr
# GG Tests for Extractor (Cal's Perl OO frame) (11/02/03)
#
# COMMON Security Test Configuration file
#
# This test is configured to run the same COMMON Security test on several nodes
# should use a version of $EDG_TESTROOT/sbin/HTML_0D_parser-script.sh that
# allows remote testing
#
# to run this test use the 1D parser
#   $EDG_TESTROOT/sbin/HTML_0D_parser-script.sh -c <path_to_this_conf_file>
#
#HTML output config
HTML_OUT=index.html
HTML_STYLE=$EDG_TEST_HTML_STYLE
##gg HTML_DIR=$EDG_TEST_HTML_OUTPUT/$(date '+%y%m%d')_gridftp_test

#generic config
##gg TEST_FILE=$EDG_TESTROOT/bin/installconfig/COMMON/Sec_config_test.sh
TEST_ARGS=""
TEST_TITLE="Cal's GridFTP Tests"
##gg TEST_ROWS_LIST="local $UIhostname $RBhostname $CEhostname $SEhostname"
##gg TEST_COLS_LIST="local $UIhostname $RBhostname $CEhostname $SEhostname"
#
#to allow remote execution
##gg TEST_RSH=/usr/bin/ssh
# Remote command executed
##gg RCOMMAND="export EDG_TESTROOT=$EDG_TESTROOT;source $EDG_TESTROOT/etc/edg-test.conf;$TEST_FILE"
# Remote node on which the test is performed
#

# Additions from now on --------------------------------------
##old TEST_FILE=/home/grodid/scripts/Extractor.pl
##gg HTML_DIR=/home/grodid/TSTG/HTML/$(date '+%y%m%d')_$USER_gridftp_test

EOCONF
    print CONF 'TEST_COLS_LIST="'."@hosts".'"'."\n";
    print CONF 'TEST_ROWS_LIST="'."local @hosts".'"'."\n";
    my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
#perv    print CONF "HTML_DIR=${html_dir}/\$(date '+%y%m%d')_"."$ENV{USER}-$$"."_gridftp\n";
    print CONF "HTML_DIR=${html_dir}/cal_gridftp\n";
##notgood    print CONF 'TEST_FILE="/home/grodid/scripts/Extractor.pl -f '."/tmp/$ENV{USER}-$$-GridFTPstep".'"'."\n";
    print CONF "TEST_FILE=$ENV{EDG_BASEUTILS}/Extractor.pm\n";
    print CONF "######################################################\n";
    close CONF;

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
