#!/usr/bin/perl

package BaseTest::TestLRCMapping;

use File::Path;
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

TestLRCMapping.pm

=head1 SYNOPSIS

TestLRCMapping.pm - Check GUID and PFN mappings in the LRC

=head1 DESCRIPTION 

B<TestLRCMapping.pm> checks the GUID and physical file name mappings
    in the local replica catalog (LRC).  This test the
    edg-local-replica-catalog command with the addMapping, removePfn,
    pfnExists, guidExists, guidForPfn, pfnsForGuid, mappingsByPfn, and
    mappingsByGuid subcommands.  The ping of
    edg-local-replica-catalog-admin is used to ensure that the server
    is responding before using the rest of the commands. 

=head1 AUTHOR

This test is based largely on a script provided by Leanne Guy (CERN).
    It has been adapted to perl to fit into the test framework and
    extended slightly to ensure that the server is left is the same
    state as when the test started.  (Assuming that the test
    succeeds.) 

Perl version by Charles Loomis (charles.loomis@cern.ch).  Please
report bugs via the European DataGrid bug reporting service (Bugzilla)
at http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "TestLRCMapping: setContext\n";

    # Add the resource option.
    $self->addOption('rlshost=s',
		     'The RLS node to test.',
                     'adc0013.cern.ch');
    $self->addOption('insecure',
		     'Use insecure connection to server.',
                     '');

    $self->addOption('vo=s',
		     'The Virtual Organization (VO) name for this serice.',
                     'wpsix');

    # Commands necessary to run this test.
    $self->{commands} = ["edg-local-replica-catalog-admin",
                         "edg-local-replica-catalog"];

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::TestLRCMapping [options]

END
    ;

    $self->{DescText} = <<'END'

This runs the basic checks on the mappings in the RLS Local Replica
Catalog.

END
    ;

}


# This uses the command line interface to check the mapping functions
# of the LRC. 
sub runTest {
    my $self = shift;

    print "TestLRCMapping: runTest\n";

    # The default is to use a secure connection. 
    my $insecure = '';
    $insecure = '--insecure' if (defined ($self->{opts}->{insecure}));
    my $server = $self->{opts}->{rlshost};

    # The default vo is wpsix
    my $vo = 'wpsix';
    $vo = $self->{opts}->{vo} if (defined ($self->{opts}->{vo}));

    # Just ping the server to see if it responds. 
    print "Pinging server $server to see if it is alive.\n";
    my $rc = $self->ping($server,$insecure,$vo);
    return $rc if $rc;

    # Now test the guid, pfn mappings. 
    my $rc = $self->testMapping($server,$insecure,$vo);
    return $rc if $rc;

    return 0;
}


# Just ping the server to see if it responds. 
sub ping {

    my ($self, $server, $insecure, $vo) = @_;
    my $command = 'edg-local-replica-catalog-admin ' . 
                  $insecure . 
		  " ping " . 
		  " -h " . $server.
		  " --vo " . $vo;
    print "Executing: $command\n";
    my $output = `$command`;
    my $rc = $? >> 8;

    if ($rc) {
        $self->setErrorMessage("Ping of LRC on $server failed with rc=$rc");
    }
    return $rc;
}

# This method does the bulk of the testing. 
sub testMapping {

    my ($self, $server, $insecure, $vo) = @_;

    my $rc = 0;

    my $command = '';

    # Create the prefixes for the files.  These are used for the
    # wildcard searches. 
    my $guidPrefix = "guid:TestLRC-$$-";
    my $pfnPrefix = "pfn:TestLRC-$$-";

    # Try to find an unused GUID to run this test. 
    print "Finding unused GUID.\n";
    my $guid = generateNewGUID($guidPrefix,$server,$insecure,$vo);
    if (! $guid) {
        $self->setErrorMessage("Cannot find unused GUID");
	$rc = 1;
	return $rc;
    }
    print "Using GUID: $guid\n";

    # Now try to create a mapping for this GUID. 
    print "Creating 10 pfns to match to this guid.\n";
    my @allpfns;
    for $i (0 .. 9) {
	my $pfn = generateNewPFN($pfnPrefix,$server,$insecure,$vo);
	if ($pfn) {
	    $command = 'edg-local-replica-catalog ' .  
		$insecure . 
		" addMapping " . 
		$guid . " " . $pfn . 
		" -h " . $server .
		" --vo " . $vo;
	    my $output = `$command`;
	    if (! $?) {
		push @allpfns, $pfn;
	    } else {
		print "Creating mapping failed for $pfn\n";
		$rc = 1;
	    }
	}
    }

    # Create the master hash of mappings.  The format is that used by
    # the output of the mapping* commands, i.e. "<guid>, <pfn>".  This
    # is used for comparisons with different method of returning the
    # mappings below. 
    my %master;
    my $pfn;
    for $pfn (@allpfns) {
	my $key = "$guid, $pfn";
	$master{$key} = 1;
    }

    # Verify that all PFNs are visible.
    print "Verifying that all PFNs exist in the LRC.\n";
    my $pfn;
    for $pfn (@allpfns) {
	$command = 'edg-local-replica-catalog ' .  
	    $insecure . 
	    " pfnExists " . 
	    $pfn . 
	    " -h " . $server .
	    " --vo " . $vo;
	$output = `$command`;
	if ($?) {
	    print "PFN $pfn doesn't exist but should\n";
	    $rc = 1;
	}
    }
    
    # Verify that the mapping back to guid is correct.
    print "Verifying that pfn -> guid mapping is correct (guidForPfn).\n";
    my %testmap;
    for $pfn (@allpfns) {
	$command = 'edg-local-replica-catalog ' .  
	    $insecure . 
	    " guidForPfn " . 
	    $pfn . 
	    " -h " . $server .
	    " --vo " . $vo;
	$output = `$command`;
	chomp($output);
	my $key = "$guid, $pfn";
	$testmap{$key} = 1 unless ($?);
    }
    my $code = compareMapKeys(\%master, \%testmap);
    $rc = 1 if ($code);
    undef %testmap;
    
    # Verify that the mapping back to guid is correct, but do this now
    # with a wildcard match.
    print "Verifying that pfn -> guid mapping is correct (mappingsByPfn).\n";
    my %testmap;
    $command = 'edg-local-replica-catalog ' .  
	$insecure . 
	" mappingsByPfn " . 
	"'$pfnPrefix*'" . 
	" -h " . $server .
	" --vo " . $vo;
    open CMD, "$command |";
    while (<CMD>) {
	chomp;
	$testmap{$_} = 1;
    }
    close CMD;
    my $code = compareMapKeys(\%master, \%testmap);
    $rc = 1 if ($code);
    undef %testmap;
    
    # Check that the mappings are correct with pfnsForGuid.
    print "Verifying guid -> pfn mappings with pfnsForGuid\n";
    $command = 'edg-local-replica-catalog ' .  
	$insecure . 
	" pfnsForGuid " .
	$guid . 
	" -h " . $server .
	    " --vo " . $vo;
    $output = `$command`;
    chomp($output);
    @testpfns = split /\s*,\s*/, $output;
    my %testmap;
    foreach $pfn (@testpfns) {
	my $key = "$guid, $pfn";
	$testmap{$key} = 1;
    }
    my $code = compareMapKeys(\%master, \%testmap);
    $rc = 1 if ($code);
    undef %testmap;

    # Check that the mappings are correct with mappingsByGuid
    print "Verifying guid -> pfn mappings with mappingsByGuid\n";
    $command = 'edg-local-replica-catalog ' .  
	$insecure . 
	" mappingsByGuid " .
	"'$guidPrefix*'" . 
	" -h " . $server .
	" --vo " . $vo;
    open CMD, "$command |";
    while (<CMD>) {
	chomp;
	$testmap{$_} = 1;
    }
    close CMD;
    my $code = compareMapKeys(\%master, \%testmap);
    $rc = 1 if ($code);
    undef %testmap;

    # Remove all of the PFNs.
    print "Removing all of the PFNs.\n";
    my $pfn;
    for $pfn (@allpfns) {
	$command = 'edg-local-replica-catalog ' . 
	    $insecure . 
	    " removePfn " . 
	    $guid . " " . $pfn . 
	    " -h " . $server .
	    " --vo " . $vo;
	$output = `$command`;
	if ($?) {
	    print "Couldn't remove PFN $pfn\n";
	    $rc = 1;
	}
    }

    # Check to see if the original guid still exists.
    print "Checking that GUID has been removed.\n";
    $command = 'edg-local-replica-catalog ' .  
	$insecure . 
	" guidExists " . 
	$guid . 
	" -h " . $server .
	    " --vo " . $vo;
    $output = `$command`;
    $code = $? >> 8;
    if ($code==1) {
    } elsif ($code==0) {
	print "$guid still exists but shouldn't\n";
	$rc = 1;
    } else {
	print "error when testing $guid\n";
	$rc = 1;
    }
    
    return $rc;
}

# Generate a new, unused GUID for testing purposes. 
sub generateNewGUID {

    my ($prefix, $server, $insecure, $vo) = @_;

    my $guid = '';

    # Try 10 times to get an unused GUID.  If it doesn't work just
    # return the empty string.
    for (1 .. 10) {
	my $testguid = $prefix . rand();
	if ($insecure) {
	    @cargs = ("edg-local-replica-catalog", 
		      $insecure,"guidExists",$testguid,"-h",$server,"--vo",$vo);
	} else {
	    @cargs = ("edg-local-replica-catalog", 
		      "guidExists",$testguid,"-h",$server,"--vo",$vo);
	}
	system(@cargs);
	my $code = $? >> 8;
	if ($code==1) {
	    $guid = $testguid;
	    last;
	}
    }

    return $guid;
}

# Generate a new, unused pfn for testing purposes.
sub generateNewPFN {

    my ($prefix, $server, $insecure, $vo) = @_;

    my $pfn = '';

    # Try 10 times to get an unused PFN.  If it doesn't work just
    # return the empty string.
    for (1 .. 10) {
	my $testpfn = $prefix . rand();
	my $command = 'edg-local-replica-catalog ' .  
		      $insecure . 
		      " pfnExists " . 
		      $testpfn . 
		      " -h " . $server .
		      " --vo " . $vo;
	my $output = `$command`;
	chomp($output);
	my $code = $? >> 8;
	if ($code==1) {
	    $pfn = $testpfn;
	    last;
	}
    }

    return $pfn;
}

# This is a utility subroutine which compares the contents of two maps
# to ensure that both maps contain exactly the same keys. 
sub compareMapKeys {

    my ($r1, $r2) = @_;
    my %hash1 = %$r1;
    my %hash2 = %$r2;

    my $rc = 0;

    # Check first that all keys of the first hash exist in the
    # second. 
    my $key;
    for $key (sort keys %hash1) {
	if (!exists($hash2{$key})) {
	    print "Missing mapping: $key\n";
	    $rc = 1;
	}
    }

    # Check now that all mappings from second are in first. 
    for $key (sort keys %hash2) {
	if (!exists($hash1{$key})) {
	    print "Extra mapping: $key\n";
	    $rc = 1;
	}
    }

    return $rc;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
