#!/usr/bin/perl

package BaseTest::TestRMCAttributes;

use File::Path;
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

TestRMCAttributes.pm

=head1 SYNOPSIS

TestRMCAttributes.pm - Check attribute handling in the RMC. 

=head1 DESCRIPTION 

B<TestRMCAttributes.pm> checks the attribute handling in the RMC.
    This uses the subcommands addGuidAttrDefn, guidattrDefnExists,
    listGuidAttrDefns, removeGuidAttrDefn, addAliasAttrDefn, 
    listAliasAttrDefns, removeAliasAttrDefn, and
    removeAliasAttrDefn.  

=head1 AUTHOR

This test is based largely on a script provided by Leanne Guy (CERN).
    It has been adapted to perl to fit into the test framework and
    extended slightly to ensure that the server is left is the same
    state as when the test started.  (Assuming that the test
    succeeds.) 

Perl version by Charles Loomis (charles.loomis@cern.ch).  Please
report bugs via the European DataGrid bug reporting service (Bugzilla)
at http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "TestRMCAttributes: setContext\n";

    # Add the resource option.
    $self->addOption('rlshost=s',
		     'The RLS node to test.',
                     'adc0013.cern.ch');
    $self->addOption('insecure',
		     'Use insecure connection to server.',
                     '');
    $self->addOption('vo=s',
		     'The Virtual Organization (VO) name for this serice.',
                     'wpsix');

    # Commands necessary to run this test.
    $self->{commands} = ["edg-replica-metadata-catalog-admin",
                         "edg-replica-metadata-catalog"];

    # Define the help and description.

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::TestRMCAttributes [options]

END
    ;

    $self->{DescText} = <<'END'

This runs the basic checks on the Alias Attribute management in the RMC Local Replica MetaData 
Catalog.

END
    ;

}


# This uses the command line interface to check the mapping functions
# of the RMC. 
sub runTest {
    my $self = shift;

    print "TestRMCAttributes: runTest\n";

    # The default is to use a secure connection. 
    my $insecure = '';
    $insecure = '--insecure' if (defined ($self->{opts}->{insecure}));
    my $server = $self->{opts}->{rlshost};

   # The default vo is wpsix
    my $vo = 'wpsix';
    $vo = $self->{opts}->{vo} if (defined ($self->{opts}->{vo}));

    # Just ping the server to see if it responds. 
    print "Pinging server $server to see if it is alive.\n";
    my $rc = $self->ping($server,$insecure,$vo);
    return $rc if $rc;

    # Now test the guid, alias mappings. 
    my $rc = $self->testAttributes($server,$insecure,$vo);
    return $rc if $rc;

    return 0;
}


# Just ping the server to see if it responds. 
sub ping {

    my ($self, $server, $insecure, $vo) = @_;
    my $command = 'edg-local-replica-catalog-admin ' . 
                  $insecure . 
		  " ping " . 
		  " -h " . $server .
		  " --vo " . $vo;
    print "Executing: $command\n";
    my $output = `$command`;
    my $rc = $? >> 8;

    if ($rc) {
        $self->setErrorMessage("Ping of RMC on $server failed with rc=$rc");
    }
    return $rc;
}

# This method does the bulk of the testing. 
sub testAttributes {

    my ($self, $server, $insecure, $vo) = @_;

    my $rc = 0;

    my $command = '';

    # Create the prefixes for the files.  These are used for the
    # wildcard searches. 
    my $guidPrefix = "guid:TestRMC-$$-";
    my $aliasPrefix = "alias:TestRMC-$$-";
    my $attrPrefix = "attr:TestRMC-$$-";

    # Try to find an unused GUID to run this test. 
    print "Finding unused GUID.\n";
    my $guid = generateNewGUID($guidPrefix,$server,$insecure,$vo);
    if (! $guid) {
        $self->setErrorMessage("Cannot find unused GUID");
	$rc = 1;
	return $rc;
    }
    print "Using GUID: $guid\n";

    # Try to find an unused attribute for this test.
    print "Finding unused attribute.\n";
    my $attr = generateNewAttribute($attrPrefix,$server,$insecure,$vo);
    if (! $attr) {
        $self->setErrorMessage("Cannot find unused attribute");
	$rc = 1;
	return $rc;
    }
    print "Using attribute: $guid\n";

    # Now try to create an aliase for this GUID. 
    print "Creating 1 aliase to match to this guid.\n";
    my $alias = generateNewALIAS($aliasPrefix,$server,$insecure,$vo);
    if ($alias) {
	$command = 'edg-replica-metadata-catalog ' .  
	    $insecure . 
	    " addAlias " . 
	    $guid . " " . $alias . 
	    " -h " . $server .
	    " --vo " . $vo;
	my $output = `$command`;
	if ($?) {
	    print "Creating mapping failed for $alias\n";
	    $rc = 1;
	    return $rc;
	}
    }

    # Now create the new alias attribute definition. 
    print "Creating new attribute definition $attr.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" addAliasAttrDefn " . 
	$attr .
	" string " .
	" -h " . $server .
	" --vo " . $vo;
    my $output = `$command`;
    if ($?) {
	print "Creating attribute definition failed for $attr\n";
	$rc = 1;
    }

    # Verify that the new attribute exists. 
    print "Verify that attribute exists: $attr.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" aliasAttrDefnExists " . 
	$attr .
	" -h " . $server .
	" --vo " . $vo;
    my $output = `$command`;
    if ($?) {
	print "Attribute $attr doesn't exist but should.\n";
	$rc = 1;
    }
    

    # Verify that attribute exists with list. 
    print "Verifying attribute exists with list (listAttrDefns).\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" listAliasAttrDefns " . 
	" -h " . $server .
	" --vo " . $vo;
    my $found = 0;
    $output = `$command`;
    chomp($output);
    my @testaliases = split /\s*,\s*/, $output;
    foreach (@testaliases) {
	chomp;
	$found = 1 if ("$_" eq "$attr");
    }
    if (! $found) {
	print "Attribute $attr not found in list.\n";
	$rc = 1;
    }

    # Set the attribute on the alias.
    my $value = rand();
    print "Set attribute $attr=$value.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" setAliasAttr " . 
	$alias . " " . $attr . " " . $value . 
	" -h " . $server .
	" --vo " . $vo;
    my $output = `$command`;
    if ($?) {
	print "Couldn't set attribute value.\n";
	$rc = 1;
    }

    
    # Retrieve the attribute to ensure the value is correct. 
    print "Get value of attribute.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" getAliasAttr " . 
	$alias . " " . $attr . 
	" -h " . $server .
	" --vo " . $vo;
    my $output = `$command`;
    chomp($output);
    if ($?) {
	print "Couldn't get attribute value.\n";
	$rc = 1;
    }
    if ("$output" ne "$value") {
	print "Attribute value mismatch $value != $output.\n";
	$rc = 1;
    }

    
    # Remove the attribute value. 
    print "Remove value of attribute.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" removeAliasAttr " . 
	$alias . " " . $attr . 
	" -h " . $server .
	" --vo " . $vo;
    my $output = `$command`;
    if ($?) {
	print "Couldn't remove attribute value.\n";
	$rc = 1;
    }

    # Verify that the attribute value has been removed. 
    print "Verify that the attribute has been removed from ALIAS.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" getAliasAttr " . 
	$alias . " " . $attr . 
	" -h " . $server .
	" --vo " . $vo;
    my $output = `$command`;
    chomp($output);
    if (! $?) {
	print "Attribute value exists, but shouldn't.\n";
	$rc = 1;
    }

    # Remove the attribute definition. 
    print "Remove the attribute definition.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" removeAliasAttrDefn " . 
	$attr . 
	" -h " . $server .
	" --vo " . $vo ;
    my $output = `$command`;
    if ($?) {
	print "Couldn't remove attribute definition.\n";
	$rc = 1;
    }


    # Verify that the attribute no longer exists.
    print "Verify that attribute no longer exists.\n";
    $command = 'edg-replica-metadata-catalog ' .  
	$insecure . 
	" aliasAttrDefnExists " . 
	$attr .
	" -h " . $server .
	 " --vo " . $vo;
    my $output = `$command`;
    if (! $?) {
	print "Attribute still exists but shouldn't.\n";
	$rc = 1;
    }
    

    # Remove all of the Aliases.
    print "Removing all of the Aliases.\n";
    $command = 'edg-replica-metadata-catalog ' . 
	$insecure . 
	" removeAlias " . 
	$guid . " " . $alias . 
	" -h " . $server .
	" --vo " . $vo;
    $output = `$command`;
    if ($?) {
	print "Couldn't remove ALIAS $alias\n";
	$rc = 1;
    }
}

# Generate a new, unused attribute for testing purposes.
sub generateNewAttribute {

    my ($prefix, $server, $insecure, $vo) = @_;

    my $attr = '';

    # Try 10 times to get an unused attribute.  If it doesn't work just
    # return the empty string.
    for (1 .. 10) {
	my $testattr = $prefix . rand();
	if ($insecure) {
	    @cargs = ("edg-replica-metadata-catalog", 
		      $insecure,"aliasAttrDefnExists",$testattr,"-h",$server,"--vo",$vo);
	} else {
	    @cargs = ("edg-replica-metadata-catalog", 
		      "aliasAttrDefnExists",$testattr,"-h",$server,"--vo",$vo);
	}
	system(@cargs);
	my $code = $? >> 8;
	if ($code==1) {
	    $attr = $testattr;
	    last;
	}
    }

    return $attr;
}

# Generate a new, unused alias for testing purposes.
sub generateNewALIAS {

    my ($prefix, $server, $insecure, $vo) = @_;

    my $alias = '';

    # Try 10 times to get an unused ALIAS.  If it doesn't work just
    # return the empty string.
    for (1 .. 10) {
	my $testalias = $prefix . rand();
	if ($insecure) {
	    @cargs = ("edg-replica-metadata-catalog", 
		      $insecure,"aliasExists",$testalias,"-h",$server,"--vo",$vo);
	} else {
	    @cargs = ("edg-replica-metadata-catalog", 
		      "aliasExists",$testalias,"-h",$server,"--vo",$vo);
	}
	system(@cargs);
	my $code = $? >> 8;
	if ($code==1) {
	    $alias = $testalias;
	    last;
	}
    }

    return $alias;
}

# This is a utility subroutine which compares the contents of two maps
# to ensure that both maps contain exactly the same keys. 
sub compareMapKeys {

    my ($r1, $r2) = @_;
    my %hash1 = %$r1;
    my %hash2 = %$r2;

    my $rc = 0;

    # Check first that all keys of the first hash exist in the
    # second. 
    my $key;
    for $key (sort keys %hash1) {
	if (!exists($hash2{$key})) {
	    print "Missing mapping: $key\n";
	    $rc = 1;
	}
    }

    # Check now that all mappings from second are in first. 
    for $key (sort keys %hash2) {
	if (!exists($hash1{$key})) {
	    print "Extra mapping: $key\n";
	    $rc = 1;
	}
    }

    return $rc;
}

# Generate a new, unused GUID for testing purposes. 
sub generateNewGUID {

    my ($prefix, $server, $insecure, $vo) = @_;

    my $guid = '';

    # Try 10 times to get an unused GUID.  If it doesn't work just
    # return the empty string.
    for (1 .. 10) {
	my $testguid = $prefix . rand();
	if ($insecure) {
	    @cargs = ("edg-replica-metadata-catalog", 
		      $insecure,"guidExists",$testguid,"-h",$server,"--vo",$vo);
	} else {
	    @cargs = ("edg-replica-metadata-catalog", 
		      "guidExists",$testguid,"-h",$server,"--vo",$vo);
	}
	system(@cargs);
	my $code = $? >> 8;
	if ($code==1) {
	    $guid = $testguid;
	    last;
	}
    }

    return $guid;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
