#!/usr/bin/perl

package EDGUtils;

use strict;
use vars qw(@ISA @EXPORT);

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(checkCommandsExist checkProxyExists checkProxyTime timeout timeoutWithRC switchStdout restoreStdout switchStdouterr restoreStdouterr printLocation printStack buildXMLFile2D makeX);

=head1 NAME

EDGUtils.pm - A set of utilities for EDG site certification tests
# $Id: EDGUtils.pm,v 1.10 2003/10/01 12:22:20 grodid Exp $

=head1 SYNOPSIS

EDGUtils.pm 

=head1 DESCRIPTION 

B<EDGUtils.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

The underlying GridFTP library is part of the Globus Toolkit (TM) and
is covered by the Globus Toolkit Public License.  See
http://www.globus.org for more information. 

=cut

# A utility function to check to see if a list of commands are available.
# It returns the unavailable subset of the commands passed into the
# routine.  
sub checkCommandsExist {

    my ($arrayref) = @_;
    my @commands = @{$arrayref};

    my @failed = ();
    foreach my $command (@commands) {
	my $result = `which $command 1>/dev/null 2>/dev/null`;
	push @failed, $command if ($?);
    }
    return \@failed;
}


# This command simply checks to see if a user proxy exists.  It does
# not currently check that the proxy is actually valid.  This command
# relies on grid-proxy-info, so that command should be listed in the
# dependencies for a test. 
sub checkProxyExists {

    # Verify that the user has a valid proxy.  Note that the time is not
    # checked. 
    my $trash = `grid-proxy-info -exists`;
    my $rc = $?;
    return $rc;
}

# Same as above, but requires that the proxy will still be valid
# for the duration (hours) provided as 1st arg (def: 1 hour)
sub checkProxyTime {
    my $time = shift;
#    if ( $time eq "" ) { $time = 1; }
    $time ||= 1;
    my $trash = `grid-proxy-info -exists -h $time `;
    my $rc = $?;
    return $rc;
}

# This will execute the given command subject to the given timeout (in
# seconds).  It returns an array reference with the first element
# being the return code and the second a scalar variable with the
# command output.
sub timeout {
    my ($command, $timeout) = @_;
    $timeout ||= 60;

    my $rc = 0;
    my $output = '';

    my $r = rand;

    eval {
        local $SIG{ALRM} = sub {$rc = 2; die "time out" }; 
        alarm $timeout;
        $output = `$command`;
        $rc = 1 if $?;
        alarm 0;
    };

    return [$rc, $output];
}


# This will execute the given command subject to the given timeout (in
# seconds).  It returns an array reference with the first element
# being the return code and the second a scalar variable with the
# command output.
sub timeoutWithRC {
    my ($command, $timeout) = @_;
    $timeout ||= 60;

    my $rc = 0;
    my $crc = 0;
    my $output = '';

    my $r = rand;

    eval {
        local $SIG{ALRM} = sub {$rc = 1; die "time out" }; 
        alarm $timeout;
        $output = `$command`;
        $crc = $?;
        alarm 0;
    };

    return [$rc, $crc, $output];
}


# A utility to temporarily switch the STDOUT to a file
sub switchStdout {
    local *FH;
    my ($file) = @_;
#p    print " Sending STDOUT to $file \n";
    open FH, ">&STDOUT" or die "Can't redirect STDOUT \n";
    open STDOUT, ">$file";
    select STDOUT; $|=1;
    return *FH;
}


# A utility to restore the STDOUT to its initial state
sub restoreStdout {
    local *FH;
    (*FH) = @_;
    close STDOUT;
    open STDOUT, ">&FH";
}


# A utility to temporarily switch the STDOUT and STDERR to a file
sub switchStdouterr {
    local *SAVEOUT;
    local *SAVEERR;
    my ($file) = @_;
#p    print " Sending STDOUT to $file \n";
    open SAVEOUT, ">&STDOUT" ;
    open SAVEERR, ">&STDERR" ;
    open STDOUT, ">$file" or die "Can't redirect STDOUT \n";
    open STDERR, ">&STDOUT" or die "Can't dup STDOUT \n";
    select STDERR; $|=1;
    select STDOUT; $|=1;
    my @fhl = (*SAVEOUT, *SAVEERR);
    return \@fhl;
}


# A utility to restore the STDOUT and STDERR to their initial state
sub restoreStdouterr {
    local *SAVEOUT;
    local *SAVEERR;
#    (*FH) = @_;
    my ($arrayref) = @_;
    (*SAVEOUT, *SAVEERR) = @$arrayref;
    close STDOUT;
    close STDERR;
    open STDOUT, ">&SAVEOUT";
    open STDERR, ">&SAVEERR";
}

sub printLocationOld {
    my $self = shift;
    if ($self->{debug} > 0) {
	my $method = shift;
	print " ==> ${method}()\n";

#  	my $caller = (caller(0))[3];
#  	print " ===> $caller \n";
    }
    return 0;
}

sub printLocation {
    my $self = shift;
#good    my $depth = shift || 1;
    my $stack = " ==> ";
    if ( $self->{opts}->{stackDebug} ) {
# 	my $stack_1 = (caller(1))[3];
# 	my $stack_2 = (caller(2))[3];
# 	print " Stack: $stack_1 $stack_2  \n";
	foreach my $deep ( 1..$self->{opts}->{stackDebug} ) {
	    my $caller = (caller($deep))[3];
	    if ( ! $caller ) { last; }
	    $stack .= "${caller}($deep) ";
	    if ( $caller eq "BaseTest::process" || $caller eq "BaseTest::new" ) { last; }
	}
	print "$stack\n";
	$self->{debug} = 0;
    }
    return 0;
}

sub printStack {
    my $self = shift;
#    my $depth = shift;
    my $depth = shift || $self->{opts}->{stackDebug};
    my $stack = " ===> ";
    if ( $depth ) {
# 	my $stack_1 = (caller(1))[3];
# 	my $stack_2 = (caller(2))[3];
# 	print " Stack: $stack_1 $stack_2  \n";
	foreach my $deep ( 1..$depth ) {
	    my $caller = (caller($deep))[3];
	    if ( ! $caller ) { last; }
	    $stack .= "${caller}($deep) ";
	    if ( $caller eq "BaseTest::process" || $caller eq "BaseTest::new" ) { last; }
	}
	print "$stack\n";
	$self->{debug} = 0;
    }
    return 0;
}

sub makeX {
  my $str  = shift;
  my $flag = shift;
  return "<$flag>\n$str\n</$flag>\n";
}

## Type with several rows, several steps and several columns 
sub buildXMLFile2D {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
#good    my $Title = __PACKAGE__;
    my $Title = $self->{type};
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
